C----------------------------------------------------------------------
C----------------------------------------------------------------------
C                               sclib2.f
C----------------------------------------------------------------------
C----------------------------------------------------------------------
C
C     SlideCont
C
C     An AUTO97 driver for sliding bifurcation analysis
C
C     BY  Fabio Dercole & Yuri Kuznetsov
C
C
C     VERSION 2.0 (last revision 4/2004)
C
C----------------------------------------------------------------------
C
C     ********************************
C     *                              *
C     *  Array manipulation library  *
C     *                              *
C     ********************************
C
C----------------------------------------------------------------------
C
c     ---------------------
c     TYPE=DOUBLE PRECISION
c     ---------------------
c
c     ------------------------------------------------
      DOUBLE PRECISION FUNCTION DARR2(ARR,N1,N2,I1,I2)
c     ------------------------------------------------
c
c       Returns the element (I1,I2) of ARR(N1,N2)
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        DOUBLE PRECISION ARR(N1,N2)
c
        DARR2=ARR(I1,I2)
c
      RETURN
      END
c
c
c     ------------------------------------------------------
      DOUBLE PRECISION FUNCTION DARR3(ARR,N1,N2,N3,I1,I2,I3)
c     ------------------------------------------------------
c
c       Returns the element (I1,I2,I3) of ARR(N1,N2,N3)
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        DOUBLE PRECISION ARR(N1,N2,N3)
c
        DARR3=ARR(I1,I2,I3)
c
      RETURN
      END
c
c
c     --------------------------------------------
      DOUBLE PRECISION FUNCTION DSCPR1(VA,N1A,I1A,
     +  VB,N1B,I1B,N)
c     --------------------------------------------
c
c       Computes the scalar product of the subvectors
c
c         VA(J1A), VB(J1B),
c
c         J1A=I1A,...,I1A+N,
c         J1B=I1B,...,I1B+N,
c
c       of the real vectors VA(N1A) and VB(N1B).
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'debug.h'
c
        DOUBLE PRECISION VA(N1A),VB(N1B)
c
c       Debug
        IF (DEBUG) THEN
          IF (DBGLEV.GE.DBGL3) THEN
            PRINT *,'enter in DSCPR1'
            PRINT *,'VA='
            CALL DARRPRN1(VA,N1A,1,N)
            PRINT *,'VB='
            CALL DARRPRN1(VB,N1B,1,N)
          END IF
        END IF
c
        PR=0.D0
        DO I=0,N-1
          PR=PR+VA(I1A+I)*VB(I1B+I)
        END DO
c
        DSCPR1=PR
c
c       Debug
        IF (DEBUG) THEN
          IF (DBGLEV.GE.DBGL3) THEN
            PRINT *,'DSCPR1=',DSCPR1
            PRINT *,'exit from DSCPR1'
          END IF
        END IF
c
      RETURN
      END
c
c
c     --------------------------------------------------------
      DOUBLE PRECISION FUNCTION DSCPR2(VA,N1A,N2A,I1A,I2A,NDA,
     +  VB,N1B,N2B,I1B,I2B,NDB,N)
c     ----------------------------------------------------
c
c       Computes the scalar product of the subvectors
c
c         VA(J1A,J2A), VB(J1B,J2B),
c
c         J1A=I1A,...,I1A+N, J2A=I2A, if NDA=1
c         J1A=I1A, J2A=I2A,...,I2A+N, if NDA=2
c         J1B=I1B,...,I1B+N, J2B=I2B, if NDB=1
c         J1B=I1B, J2B=I2B,...,I2B+N, if NDB=2
c
c       of the real vectors VA(N1A,N2A) and VB(N1B,N2B).
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'debug.h'
c
        DOUBLE PRECISION VA(N1A,N2A),VB(N1B,N2B)
c
c       Debug
        IF (DEBUG) THEN
          IF (DBGLEV.GE.DBGL3) THEN
            PRINT *,'enter in DSCPR2'
            PRINT *,'VA='
            IF (NDA.EQ.1) THEN
              CALL DARRPRN2(VA,N1A,N2A,I1A,I2A,N,1)
            ELSE
              CALL DARRPRN2(VA,N1A,N2A,I1A,I2A,1,N)
            END IF
            PRINT *,'VB='
            IF (NDB.EQ.1) THEN
              CALL DARRPRN2(VB,N1B,N2B,I1B,I2B,N,1)
            ELSE
              CALL DARRPRN2(VB,N1B,N2B,I1B,I2B,1,N)
            END IF
          END IF
        END IF
c
        PR=0.D0
        DO I=0,N-1
          IF (NDA.EQ.1) THEN
            IF (NDB.EQ.1) THEN
              PR=PR+VA(I1A+I,I2A)*VB(I1B+I,I2B)
            ELSE IF (NDB.EQ.2) THEN
              PR=PR+VA(I1A+I,I2A)*VB(I1B,I2B+I)
            END IF
          ELSE IF (NDA.EQ.2) THEN
            IF (NDB.EQ.1) THEN
              PR=PR+VA(I1A,I2A+I)*VB(I1B+I,I2B)
            ELSE IF (NDB.EQ.2) THEN
              PR=PR+VA(I1A,I2A+I)*VB(I1B,I2B+I)
            END IF
          END IF
        END DO
c
        DSCPR2=PR
c
c       Debug
        IF (DEBUG) THEN
          IF (DBGLEV.GE.DBGL3) THEN
            PRINT *,'DSCPR2=',DSCPR2
            PRINT *,'exit from DSCPR2'
          END IF
        END IF
c
      RETURN
      END
c
c
c     ----------------------------------------------------------------
      DOUBLE PRECISION FUNCTION DSCPR3(VA,N1A,N2A,N3A,I1A,I2A,I3A,NDA,
     +  VB,N1B,N2B,N3B,I1B,I2B,I3B,NDB,N)
c     ----------------------------------------------------------------
c
c       Computes the scalar product of the subvectors
c
c         VA(J1A,J2A,J3A), VB(J1B,J2B,J3B),
c
c         J1A=I1A,...,I1A+N, J2A=I2A, J3A=I3A, if NDA=1
c         J1A=I1A, J2A=I2A,...,I2A+N, J3A=I3A, if NDA=2
c         J1A=I1A, J2A=I2A, J3A=I3A,...,I3A+N, if NDA=3
c         J1B=I1B,...,I1B+N, J2B=I2B, J3B=I3B, if NDB=1
c         J1B=I1B, J2B=I2B,...,I2B+N, J3B=I3B, if NDB=2
c         J1B=I1B, J2B=I2B, J3B=I3B,...,I3B+N, if NDB=3
c
c       of the real vectors VA(N1A,N2A,N3A) and VB(N1B,N2B,N3B).
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'debug.h'
c
        DOUBLE PRECISION VA(N1A,N2A,N3A),VB(N1B,N2B,N3B)
c
c       Debug
        IF (DEBUG) THEN
          IF (DBGLEV.GE.DBGL3) THEN
            PRINT *,'enter in DSCPR3'
            PRINT *,'VA='
            IF (NDA.EQ.1) THEN
              CALL DARRPRN3(VA,N1A,N2A,N3A,I1A,I2A,I3A,N,1,1)
            ELSE IF (NDA.EQ.2) THEN
              CALL DARRPRN3(VA,N1A,N2A,N3A,I1A,I2A,I3A,1,N,1)
            ELSE
              CALL DARRPRN3(VA,N1A,N2A,N3A,I1A,I2A,I3A,1,1,N)
            END IF
            PRINT *,'VB='
            IF (NDB.EQ.1) THEN
              CALL DARRPRN3(VB,N1B,N2B,N3B,I1B,I2B,I3B,N,1,1)
            ELSE IF (NDB.EQ.2) THEN
              CALL DARRPRN3(VB,N1B,N2B,N3B,I1B,I2B,I3B,1,N,1)
            ELSE
              CALL DARRPRN3(VB,N1B,N2B,N3B,I1B,I2B,I3B,1,1,N)
            END IF
          END IF
        END IF
c
        PR=0.D0
        DO I=0,N-1
          IF (NDA.EQ.1) THEN
            IF (NDB.EQ.1) THEN
              PR=PR+VA(I1A+I,I2A,I3A)*VB(I1B+I,I2B,I3B)
            ELSE IF (NDB.EQ.2) THEN
              PR=PR+VA(I1A+I,I2A,I3A)*VB(I1B,I2B+I,I3B)
            ELSE IF (NDB.EQ.3) THEN
              PR=PR+VA(I1A+I,I2A,I3A)*VB(I1B,I2B,I3B+I)
            END IF
          ELSE IF (NDA.EQ.2) THEN
            IF (NDB.EQ.1) THEN
              PR=PR+VA(I1A,I2A+I,I3A)*VB(I1B+I,I2B,I3B)
            ELSE IF (NDB.EQ.2) THEN
              PR=PR+VA(I1A,I2A+I,I3A)*VB(I1B,I2B+I,I3B)
            ELSE IF (NDB.EQ.3) THEN
              PR=PR+VA(I1A,I2A+I,I3A)*VB(I1B,I2B,I3B+I)
            END IF
          ELSE IF (NDA.EQ.3) THEN
            IF (NDB.EQ.1) THEN
              PR=PR+VA(I1A,I2A,I3A+I)*VB(I1B+I,I2B,I3B)
            ELSE IF (NDB.EQ.2) THEN
              PR=PR+VA(I1A,I2A,I3A+I)*VB(I1B,I2B+I,I3B)
            ELSE IF (NDB.EQ.3) THEN
              PR=PR+VA(I1A,I2A,I3A+I)*VB(I1B,I2B,I3B+I)
            END IF
          END IF
        END DO
c
        DSCPR3=PR
c
c       Debug
        IF (DEBUG) THEN
          IF (DBGLEV.GE.DBGL3) THEN
            PRINT *,'DSCPR3=',DSCPR3
            PRINT *,'exit from DSCPR3'
          END IF
        END IF
c
      RETURN
      END
c
c
c     ---------------------------------
      SUBROUTINE DARRCPY1(ARRS,N1S,I1S,
     +  ARRD,N1D,I1D,N1)
c     ---------------------------------
c
c       Copies the block of elements of ARRS(N1S)
c       having indexex I1 with
c
c         I1=I1S,...,I1S+N1-1
c
c       on the block of elements of ARRD(N1D)
c       having indexex J1 with
c
c         J1=I1D,...,I1D+N1-1
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'debug.h'
c
        DOUBLE PRECISION ARRS(N1S)
        DOUBLE PRECISION ARRD(N1D)
c
        DO I1=0,N1-1
          ARRD(I1D+I1)=
     +      ARRS(I1S+I1)
        END DO
c
c       Debug
        IF (DEBUG) THEN
          IF (DBGLEV.GE.DBGL3) THEN
            PRINT *,'enter in DARRCPY1'
            PRINT *,'ARRS='
            CALL DARRPRN1(ARRS,N1S,1,N1S)
            PRINT *,'ARRD='
            CALL DARRPRN1(ARRD,N1D,1,N1D)
            PRINT *,'exit from DARRCPY1'
          END IF
        END IF
c  
      RETURN
      END
c
c
c     -----------------------------------------
      SUBROUTINE DARRCPY2(ARRS,N1S,N2S,I1S,I2S,
     +  ARRD,N1D,N2D,I1D,I2D,N1,N2)
c     -----------------------------------------
c
c       Copies the block of elements of ARRS(N1S,N2S)
c       having indexex (I1,I2) with
c
c         I1=I1S,...,I1S+N1-1
c         I2=I2S,...,I2S+N2-1
c
c       on the block of elements of ARRD(N1D,N2D)
c       having indexex (J1,J2) with
c
c         J1=I1D,...,I1D+N1-1
c         J2=I2D,...,I2D+N2-1
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'debug.h'
c
        DOUBLE PRECISION ARRS(N1S,N2S)
        DOUBLE PRECISION ARRD(N1D,N2D)
c
        DO I1=0,N1-1
          DO I2=0,N2-1
            ARRD(I1D+I1,I2D+I2)=
     +        ARRS(I1S+I1,I2S+I2)
          END DO
        END DO
c
c       Debug
        IF (DEBUG) THEN
          IF (DBGLEV.GE.DBGL3) THEN
            PRINT *,'enter in DARRCPY2'
            PRINT *,'ARRS='
            CALL DARRPRN2(ARRS,N1S,N2S,1,1,N1S,N2S)
            PRINT *,'ARRD='
            CALL DARRPRN2(ARRD,N1D,N2D,1,1,N1D,N2D)
            PRINT *,'exit from DARRCPY2'
          END IF
        END IF
c  
      RETURN
      END
c
c
c     -------------------------------------------------
      SUBROUTINE DARRCPY3(ARRS,N1S,N2S,N3S,I1S,I2S,I3S,
     +  ARRD,N1D,N2D,N3D,I1D,I2D,I3D,N1,N2,N3)
c     -------------------------------------------------
c
c       Copies the block of elements of ARRS(N1S,N2S,N3S)
c       having indexex (I1,I2,I3) with
c
c         I1=I1S,...,I1S+N1-1
c         I2=I2S,...,I2S+N2-1
c         I3=I3S,...,I3S+N3-1
c
c       on the block of elements of ARRD(N1D,N2D,N3D)
c       having indexex (J1,J2,J3) with
c
c         J1=I1D,...,I1D+N1-1
c         J2=I2D,...,I2D+N2-1
c         J3=I3D,...,I3D+N3-1
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'debug.h'
c
        DOUBLE PRECISION ARRS(N1S,N2S,N3S)
        DOUBLE PRECISION ARRD(N1D,N2D,N3D)
c
        DO I1=0,N1-1
          DO I2=0,N2-1
            DO I3=0,N3-1
              ARRD(I1D+I1,I2D+I2,I3D+I3)=
     +          ARRS(I1S+I1,I2S+I2,I3S+I3)
            END DO
          END DO
        END DO
c
c       Debug
        IF (DEBUG) THEN
          IF (DBGLEV.GE.DBGL3) THEN
            PRINT *,'enter in DARRCPY3'
            PRINT *,'ARRS='
            CALL DARRPRN3(ARRS,N1S,N2S,N3S,1,1,1,N1S,N2S,N3S)
            PRINT *,'ARRD='
            CALL DARRPRN3(ARRD,N1D,N2D,N3D,1,1,1,N1D,N2D,N3D)
            PRINT *,'exit from DARRCPY3'
          END IF
        END IF
c  
      RETURN
      END
c
c
c     ---------------------------------------------------------
      SUBROUTINE DARRCPY4(ARRS,N1S,N2S,N3S,N4S,I1S,I2S,I3S,I4S,
     +  ARRD,N1D,N2D,N3D,N4D,I1D,I2D,I3D,I4D,N1,N2,N3,N4)
c     ---------------------------------------------------------
c
c       Copies the block of elements of ARRS(N1S,N2S,N3S,N4S)
c       having indexex (I1,I2,I3,I4) with
c
c         I1=I1S,...,I1S+N1-1
c         I2=I2S,...,I2S+N2-1
c         I3=I3S,...,I3S+N3-1
c         I4=I4S,...,I4S+N4-1
c
c       on the block of elements of ARRD(N1D,N2D,N3D,N4D)
c       having indexex (J1,J2,J3,J4) with
c
c         J1=I1D,...,I1D+N1-1
c         J2=I2D,...,I2D+N2-1
c         J3=I3D,...,I3D+N3-1
c         J4=I4D,...,I4D+N4-1
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'debug.h'
c
        DOUBLE PRECISION ARRS(N1S,N2S,N3S,N4S)
        DOUBLE PRECISION ARRD(N1D,N2D,N3D,N4D)
c
        DO I1=0,N1-1
          DO I2=0,N2-1
            DO I3=0,N3-1
              DO I4=0,N4-1
                ARRD(I1D+I1,I2D+I2,I3D+I3,I4D+I4)=
     +            ARRS(I1S+I1,I2S+I2,I3S+I3,I4S+I4)
              END DO
            END DO
          END DO
        END DO
c
c       Debug
        IF (DEBUG) THEN
          IF (DBGLEV.GE.DBGL3) THEN
            PRINT *,'enter in DARRCPY4'
            PRINT *,'ARRS='
            CALL DARRPRN4(ARRS,N1S,N2S,N3S,N4S,1,1,1,1,N1S,N2S,N3S,N4S)
            PRINT *,'ARRD='
            CALL DARRPRN4(ARRD,N1D,N1D,N1D,N1D,,11,1,1,N1D,N1D,N1D,N1D)
            PRINT *,'exit from DARRCPY4'
          END IF
        END IF
c  
      RETURN
      END
c
c
c     ----------------------------
      SUBROUTINE DARRPRN1(ARR,N1R,
     +  I1R,N1)
c     ----------------------------
c
c       Print on the standard output the block of elements
c       of ARR(N1R) having indexex I1 with
c
c         I1=I1R,...,I1R+N1-1
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        DOUBLE PRECISION ARR(N1R)
c
        DO I1=0,N1-1
          WRITE (UNIT=*,FMT=100) I1R+I1,
     +      ARR(I1R+I1)
        END DO
c
 100    FORMAT ('(',I2,')=',1P7E20.10E3)
c
      RETURN
      END
c
c
c     --------------------------------
      SUBROUTINE DARRPRN2(ARR,N1R,N2R,
     +  I1R,I2R,N1,N2)
c     --------------------------------
c
c       Print on the standard output the block of elements
c       of ARR(N1R,N2R) having indexex (I1,I2) with
c
c         I1=I1R,...,I1R+N1-1
c         I2=I2R,...,I2R+N2-1
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        DOUBLE PRECISION ARR(N1R,N2R)
c
        DO I1=0,N1-1
          DO I2=0,N2-1
            WRITE (UNIT=*,FMT=100) I1R+I1,I2R+I2,
     +        ARR(I1R+I1,I2R+I2)
          END DO
        END DO
c
 100    FORMAT ('(',I2,',',I2,')=',1P7E20.10E3)
c
      RETURN
      END
c
c
c     ------------------------------------
      SUBROUTINE DARRPRN3(ARR,N1R,N2R,N3R,
     +  I1R,I2R,I3R,N1,N2,N3)
c     ------------------------------------
c
c       Print on the standard output the block of elements
c       of ARR(N1R,N2R,N3R) having indexex (I1,I2,I3) with
c
c         I1=I1R,...,I1R+N1-1
c         I2=I2R,...,I2R+N2-1
c         I3=I3R,...,I3R+N3-1
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        DOUBLE PRECISION ARR(N1R,N2R,N3R)
c
        DO I1=0,N1-1
          DO I2=0,N2-1
            DO I3=0,N3-1
              WRITE (UNIT=*,FMT=100) I1R+I1,I2R+I2,I3R+I3,
     +          ARR(I1R+I1,I2R+I2,I3R+I3)
            END DO
          END DO
        END DO
c
 100    FORMAT ('(',I2,',',I2,',',I2,')=',1P7E20.10E3)
c
      RETURN
      END
c
c
c     ----------------------------------------
      SUBROUTINE DARRPRN4(ARR,N1R,N2R,N3R,N4R,
     +  I1R,I2R,I3R,I4R,N1,N2,N3,N4)
c     ----------------------------------------
c
c       Print on the standard output the block of elements
c       of ARR(N1R,N2R,N3R,N4R) having indexex (I1,I2,I3,I4) with
c
c         I1=I1R,...,I1R+N1-1
c         I2=I2R,...,I2R+N2-1
c         I3=I3R,...,I3R+N3-1
c         I4=I4R,...,I4R+N4-1
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        DOUBLE PRECISION ARR(N1R,N2R,N3R,N4R)
c
        DO I1=0,N1-1
          DO I2=0,N2-1
            DO I3=0,N3-1
              DO I4=0,N4-1
                WRITE (UNIT=*,FMT=100) I1R+I1,I2R+I2,I3R+I3,I4R+I4,
     +            ARR(I1R+I1,I2R+I2,I3R+I3,I4R+I4)
              END DO
            END DO
          END DO
        END DO
c
 100    FORMAT ('(',I2,',',I2,',',I2,',',I2,')=',1P7E20.10E3)
c
      RETURN
      END
c
c
c     ------------
c     TYPE=INTEGER
c     ------------
c
c     ---------------------------------
      SUBROUTINE IARRCPY1(ARRS,N1S,I1S,
     +  ARRD,N1D,I1D,N1)
c     ---------------------------------
c
c       Copies the block of elements of ARRS(N1S)
c       having indexex I1 with
c
c         I1=I1S,...,I1S+N1-1
c
c       on the block of elements of ARRD(N1D)
c       having indexex J1 with
c
c         J1=I1D,...,I1D+N1-1
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'debug.h'
c
        INTEGER ARRS(N1S)
        INTEGER ARRD(N1D)
c
        DO I1=0,N1-1
          ARRD(I1D+I1)=
     +      ARRS(I1S+I1)
        END DO
c
c       Debug
        IF (DEBUG) THEN
          IF (DBGLEV.GE.DBGL3) THEN
            PRINT *,'enter in IARRCPY1'
            PRINT *,'ARRS='
            CALL IARRPRN1(ARRS,N1S,1,N1S)
            PRINT *,'ARRD='
            CALL IARRPRN1(ARRD,N1D,1,N1D)
            PRINT *,'exit from IARRCPY1'
          END IF
        END IF
c  
      RETURN
      END
c
c
c     -----------------------------------------
      SUBROUTINE IARRCPY2(ARRS,N1S,N2S,I1S,I2S,
     +  ARRD,N1D,N2D,I1D,I2D,N1,N2)
c     -----------------------------------------
c
c       Copies the block of elements of ARRS(N1S,N2S)
c       having indexex (I1,I2) with
c
c         I1=I1S,...,I1S+N1-1
c         I2=I2S,...,I2S+N2-1
c
c       on the block of elements of ARRD(N1D,N2D)
c       having indexex (J1,J2) with
c
c         J1=I1D,...,I1D+N1-1
c         J2=I2D,...,I2D+N2-1
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'debug.h'
c
        INTEGER ARRS(N1S,N2S)
        INTEGER ARRD(N1D,N2D)
c
        DO I1=0,N1-1
          DO I2=0,N2-1
            ARRD(I1D+I1,I2D+I2)=
     +        ARRS(I1S+I1,I2S+I2)
          END DO
        END DO
c
c       Debug
        IF (DEBUG) THEN
          IF (DBGLEV.GE.DBGL3) THEN
            PRINT *,'enter in IARRCPY2'
            PRINT *,'ARRS='
            CALL IARRPRN2(ARRS,N1S,N2S,1,1,N1S,N2S)
            PRINT *,'ARRD='
            CALL IARRPRN2(ARRD,N1D,N2D,1,1,N1D,N2D)
            PRINT *,'exit from IARRCPY2'
          END IF
        END IF
c  
      RETURN
      END
c
c
c     -------------------------------------------------
      SUBROUTINE IARRCPY3(ARRS,N1S,N2S,N3S,I1S,I2S,I3S,
     +  ARRD,N1D,N2D,N3D,I1D,I2D,I3D,N1,N2,N3)
c     -------------------------------------------------
c
c       Copies the block of elements of ARRS(N1S,N2S,N3S)
c       having indexex (I1,I2,I3) with
c
c         I1=I1S,...,I1S+N1-1
c         I2=I2S,...,I2S+N2-1
c         I3=I3S,...,I3S+N3-1
c
c       on the block of elements of ARRD(N1D,N2D,N3D)
c       having indexex (J1,J2,J3) with
c
c         J1=I1D,...,I1D+N1-1
c         J2=I2D,...,I2D+N2-1
c         J3=I3D,...,I3D+N3-1
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'debug.h'
c
        INTEGER ARRS(N1S,N2S,N3S)
        INTEGER ARRD(N1D,N2D,N3D)
c
        DO I1=0,N1-1
          DO I2=0,N2-1
            DO I3=0,N3-1
              ARRD(I1D+I1,I2D+I2,I3D+I3)=
     +          ARRS(I1S+I1,I2S+I2,I3S+I3)
            END DO
          END DO
        END DO
c
c       Debug
        IF (DEBUG) THEN
          IF (DBGLEV.GE.DBGL3) THEN
            PRINT *,'enter in IARRCPY3'
            PRINT *,'ARRS='
            CALL IARRPRN3(ARRS,N1S,N2S,N3S,1,1,1,N1S,N2S,N3S)
            PRINT *,'ARRD='
            CALL IARRPRN3(ARRD,N1D,N2D,N3D,1,1,1,N1D,N2D,N3D)
            PRINT *,'exit from IARRCPY3'
          END IF
        END IF
c  
      RETURN
      END
c
c
c     ---------------------------------------------------------
      SUBROUTINE IARRCPY4(ARRS,N1S,N2S,N3S,N4S,I1S,I2S,I3S,I4S,
     +  ARRD,N1D,N2D,N3D,N4D,I1D,I2D,I3D,I4D,N1,N2,N3,N4)
c     ---------------------------------------------------------
c
c       Copies the block of elements of ARRS(N1S,N2S,N3S,N4S)
c       having indexex (I1,I2,I3,I4) with
c
c         I1=I1S,...,I1S+N1-1
c         I2=I2S,...,I2S+N2-1
c         I3=I3S,...,I3S+N3-1
c         I4=I4S,...,I4S+N4-1
c
c       on the block of elements of ARRD(N1D,N2D,N3D,N4D)
c       having indexex (J1,J2,J3,J4) with
c
c         J1=I1D,...,I1D+N1-1
c         J2=I2D,...,I2D+N2-1
c         J3=I3D,...,I3D+N3-1
c         J4=I4D,...,I4D+N4-1
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'debug.h'
c
        INTEGER ARRS(N1S,N2S,N3S,N4S)
        INTEGER ARRD(N1D,N2D,N3D,N4D)
c
        DO I1=0,N1-1
          DO I2=0,N2-1
            DO I3=0,N3-1
              DO I4=0,N4-1
                ARRD(I1D+I1,I2D+I2,I3D+I3,I4D+I4)=
     +            ARRS(I1S+I1,I2S+I2,I3S+I3,I4S+I4)
              END DO
            END DO
          END DO
        END DO
c
c       Debug
        IF (DEBUG) THEN
          IF (DBGLEV.GE.DBGL3) THEN
            PRINT *,'enter in IARRCPY4'
            PRINT *,'ARRS='
            CALL IARRPRN4(ARRS,N1S,N2S,N3S,N4S,1,1,1,1,N1S,N2S,N3S,N4S)
            PRINT *,'ARRD='
            CALL IARRPRN4(ARRD,N1D,N1D,N1D,N1D,,11,1,1,N1D,N1D,N1D,N1D)
            PRINT *,'exit from IARRCPY4'
          END IF
        END IF
c  
      RETURN
      END
c
c
c     ----------------------------
      SUBROUTINE IARRPRN1(ARR,N1R,
     +  I1R,N1)
c     ----------------------------
c
c       Print on the standard output the block of elements
c       of ARR(N1R) having indexex I1 with
c
c         I1=I1R,...,I1R+N1-1
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INTEGER ARR(N1R)
c
        DO I1=0,N1-1
          WRITE (UNIT=*,FMT=100) I1R+I1,
     +      ARR(I1R+I1)
        END DO
c
 100    FORMAT ('(',I2,')=',I4)
c
      RETURN
      END
c
c
c     --------------------------------
      SUBROUTINE IARRPRN2(ARR,N1R,N2R,
     +  I1R,I2R,N1,N2)
c     --------------------------------
c
c       Print on the standard output the block of elements
c       of ARR(N1R,N2R) having indexex (I1,I2) with
c
c         I1=I1R,...,I1R+N1-1
c         I2=I2R,...,I2R+N2-1
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INTEGER ARR(N1R,N2R)
c
        DO I1=0,N1-1
          DO I2=0,N2-1
            WRITE (UNIT=*,FMT=100) I1R+I1,I2R+I2,
     +        ARR(I1R+I1,I2R+I2)
          END DO
        END DO
c
 100    FORMAT ('(',I2,',',I2,')=',I4)
c
      RETURN
      END
c
c
c     ------------------------------------
      SUBROUTINE IARRPRN3(ARR,N1R,N2R,N3R,
     +  I1R,I2R,I3R,N1,N2,N3)
c     ------------------------------------
c
c       Print on the standard output the block of elements
c       of ARR(N1R,N2R,N3R) having indexex (I1,I2,I3) with
c
c         I1=I1R,...,I1R+N1-1
c         I2=I2R,...,I2R+N2-1
c         I3=I3R,...,I3R+N3-1
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INTEGER ARR(N1R,N2R,N3R)
c
        DO I1=0,N1-1
          DO I2=0,N2-1
            DO I3=0,N3-1
              WRITE (UNIT=*,FMT=100) I1R+I1,I2R+I2,I3R+I3,
     +          ARR(I1R+I1,I2R+I2,I3R+I3)
            END DO
          END DO
        END DO
c
 100    FORMAT ('(',I2,',',I2,',',I2,')=',I4)
c
      RETURN
      END
c
c
c     ----------------------------------------
      SUBROUTINE IARRPRN4(ARR,N1R,N2R,N3R,N4R,
     +  I1R,I2R,I3R,I4R,N1,N2,N3,N4)
c     ----------------------------------------
c
c       Print on the standard output the block of elements
c       of ARR(N1R,N2R,N3R,N4R) having indexex (I1,I2,I3,I4) with
c
c         I1=I1R,...,I1R+N1-1
c         I2=I2R,...,I2R+N2-1
c         I3=I3R,...,I3R+N3-1
c         I4=I4R,...,I4R+N4-1
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INTEGER ARR(N1R,N2R,N3R,N4R)
c
        DO I1=0,N1-1
          DO I2=0,N2-1
            DO I3=0,N3-1
              DO I4=0,N4-1
                WRITE (UNIT=*,FMT=100) I1R+I1,I2R+I2,I3R+I3,I4R+I4,
     +            ARR(I1R+I1,I2R+I2,I3R+I3,I4R+I4)
              END DO
            END DO
          END DO
        END DO
c
 100    FORMAT ('(',I2,',',I2,',',I2,',',I2,')=',I4)
c
      RETURN
      END
c
