C----------------------------------------------------------------------
C----------------------------------------------------------------------
C                               sclib3.f
C----------------------------------------------------------------------
C----------------------------------------------------------------------
C
C     SlideCont
C
C     An AUTO97 toolbox for sliding bifurcation analysis
C
C     BY  Fabio Dercole & Yuri Kuznetsov
C
C
C     VERSION 2.0 (last revision 4/2004)
C
C----------------------------------------------------------------------
C
C     **********************
C     *                    *
C     *  Support routines  *
C     *                    *
C     **********************
C
C----------------------------------------------------------------------
C
c     --------------------------------------------------
      DOUBLE PRECISION FUNCTION SCGETP(CODE,IPNT,IC,UPS)
c     --------------------------------------------------
c
c       SlideCont specific - begin
c       --------------------------
c
c       For differential equations, if CODE='PNT'(or 'pnt')
c       returns the IPNT-th solution point IC-th coordinate, where
c       1 <= IPNT <= NST*NCOL
c
c       SlideCont specific - end
c       ------------------------
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'auto.h'
        INCLUDE 'debug.h'
c
        COMMON /BLPV/ DTV(NTSTX+1),RAV(NRAP),IAV(NIAP)
        DIMENSION UPS(NTSTX+1,*)
        CHARACTER*3 CODE
c
        NX=NTSTX+1
        IPS=IAV(2)
        NTST=IAV(5)
c
        IF ( IABS(IPS).LE.1 .OR. IPS.EQ.5) THEN
          IF (CODE.EQ.'NRM'.OR.CODE.EQ.'nrm') THEN
            SCGETP=ABS(UPS(IC,1))   
          ELSE IF (CODE.EQ.'INT'.OR.CODE.EQ.'int') THEN
            SCGETP=UPS(IC,1)
          ELSE IF (CODE.EQ.'MAX'.OR.CODE.EQ.'max') THEN
            SCGETP=UPS(IC,1)
          ELSE IF (CODE.EQ.'MIN'.OR.CODE.EQ.'min') THEN
            SCGETP=UPS(IC,1)
          ELSE IF (CODE.EQ.'BV0'.OR.CODE.EQ.'bv0') THEN
            SCGETP=UPS(IC,1)
          ELSE IF (CODE.EQ.'BV1'.OR.CODE.EQ.'bv1') THEN
            SCGETP=UPS(IC,1)
          ELSE IF (CODE.EQ.'STP'.OR.CODE.EQ.'stp') THEN
            SCGETP=RAV(5)
          ELSE IF (CODE.EQ.'FLD'.OR.CODE.EQ.'fld') THEN
            SCGETP=RAV(16)
          ELSE IF (CODE.EQ.'HBF'.OR.CODE.EQ.'hbf') THEN
            SCGETP=RAV(17)
          ELSE IF (CODE.EQ.'BIF'.OR.CODE.EQ.'bif') THEN
            SCGETP=RAV(14)
          ELSE IF (CODE.EQ.'SPB'.OR.CODE.EQ.'spb') THEN
            SCGETP=0.
          END IF
        ELSE
          IF (CODE.EQ.'NRM'.OR.CODE.EQ.'nrm') THEN
            SCGETP=RNRM2(IAV,NX,IC,UPS,DTV)   
          ELSE IF (CODE.EQ.'INT'.OR.CODE.EQ.'int') THEN
            SCGETP=RINTG(IAV,NX,IC,UPS,DTV)
          ELSE IF (CODE.EQ.'MAX'.OR.CODE.EQ.'max') THEN
            SCGETP=RMXUPS(IAV,NX,IC,UPS)
          ELSE IF (CODE.EQ.'MIN'.OR.CODE.EQ.'min') THEN
            SCGETP=RMNUPS(IAV,NX,IC,UPS)
          ELSE IF (CODE.EQ.'BV0'.OR.CODE.EQ.'bv0') THEN
            SCGETP=UPS(1,IC)
          ELSE IF (CODE.EQ.'BV1'.OR.CODE.EQ.'bv1') THEN
            SCGETP=UPS(NTST+1,IC)
          ELSE IF (CODE.EQ.'STP'.OR.CODE.EQ.'stp') THEN
            SCGETP=RAV(5)
          ELSE IF (CODE.EQ.'FLD'.OR.CODE.EQ.'fld') THEN
            SCGETP=RAV(16)
          ELSE IF (CODE.EQ.'HBF'.OR.CODE.EQ.'hbf') THEN
            SCGETP=0.
          ELSE IF (CODE.EQ.'BIF'.OR.CODE.EQ.'bif') THEN
            SCGETP=RAV(18)
          ELSE IF (CODE.EQ.'SPB'.OR.CODE.EQ.'spb') THEN
            SCGETP=RAV(19)
c
c         SlideCont specific - begin
c         --------------------------
          ELSE IF (CODE.EQ.'PNT'.OR.CODE.EQ.'pnt') THEN
            NDIM=IAV(1)
            NCOL=IAV(6)
            ITST=(IPNT-1)/NCOL+1
            ICOL=IPNT-(ITST-1)*NCOL
            SCGETP=UPS(ITST,(ICOL-1)*NDIM+IC)
c
c         SlideCont specific - end
c         ------------------------
c
          END IF
        END IF
c
      RETURN
      END
c
c
c     --------------------------------------------------
      DOUBLE PRECISION FUNCTION SCGRAZ(SCNDIM,PAR,UPS,I)
c     --------------------------------------------------
c
c       For differential equations, returns the maximum [minimum]
c       value of the discontinuity function H evaluated on the
c       solution points if I=1 [I=2]
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'slidecont.h'
        INCLUDE 'auto.h'
        INCLUDE 'debug.h'
c
        PARAMETER (RSMALL=1.0d-30,RLARGE=1.0d+30)
c
        COMMON /BLPV/ DTV(NTSTX+1),RAV(NRAP),IAV(NIAP)
        DIMENSION UPS(NTSTX+1,*),PAR(*)
c
c       Discontinuity boundary
c       ----------------------
        DOUBLE PRECISION H
        DOUBLE PRECISION DHDX(SCNDIMX)
        DOUBLE PRECISION DHDP(SCNPARX)
        DOUBLE PRECISION DHDXDX(SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DHDXDP(SCNDIMX,SCNPARX)
c
c       Zero initialization
        DATA H/0.0/
        DATA DHDX/SCNDIMX*0.0/
        DATA DHDP/SCNPARX*0.0/
        DATA DHDXDX/SCNDIMX2*0.0/
        DATA DHDXDP/SCNDIMXNPARX*0.0/
c
c       Other local variables
c       ---------------------
        DOUBLE PRECISION X(SCNDIMX)
        INTEGER TMPIDIFF
c
c       AUTO97 constants
c       ----------------
        NDIM=IAV(1)
        NTST=IAV(5)
        NCOL=IAV(6)
c
c       Main loop      
c       ---------
        AHMIN=RLARGE
        AHMAX=-RLARGE
c      
        DO K=1,NTST*NCOL
          DO L=1,NDIM
            X(L)=SCGETP('PNT',K,L,UPS)
          END DO
c
c         Discontinuity boundary computation
          TMPIDIFF=0
          CALL SCBOUND(SCNDIM,X,PAR,TMPIDIFF,
     +      H,DHDX,DHDP,DHDXDX,DHDXDP)
c
          IF ((I.EQ.1).AND.(H.GT.AHMAX)) THEN
            AHMAX=H
            SCGRAZ=H
          ELSE IF ((I.EQ.2).AND.(H.LT.AHMIN)) THEN
            AHMIN=H
            SCGRAZ=H
          END IF
        END DO
c
      RETURN
      END
c
c
c     ---------------------------------
      INTEGER FUNCTION SCIHOPF(RR,RI,N)
c     ---------------------------------
c
c       Returns the index of the complex conjugate eigenvalue
c       (with positive imaginary part) closest to the imaginary
c       axis. On input RR(N) and RI(N) contain real and imaginary
c       part of eigenvalues. Returns zero if all eigenvalues are real.
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'debug.h'
c
        PARAMETER (RSMALL=1.0d-30,RLARGE=1.0d+30)
c
        DIMENSION RR(N),RI(N)
c
        SCIHOPF=0
        ARMIN=RLARGE
        DO I=1,N
          IF (RI(I).GT.0.d0) THEN
            AR=DABS(RR(I))
            IF (AR.LE.ARMIN) THEN
              SCIHOPF=I
              ARMIN=AR
            END IF
          END IF
        END DO
c
      RETURN
      END
c
c
c     ---------------------------------
      INTEGER FUNCTION SCIFOLD(RR,RI,N)
c     ---------------------------------
c
c       Returns the index of the real eigenvalue closest to the
c       imaginary axis. On input RR(N) and RI(N) contain real and
c       imaginary part of eigenvalues. Returns zero if all
c       eigenvalues are complex.
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'debug.h'
c
        PARAMETER (RSMALL=1.0d-30,RLARGE=1.0d+30)
c
        DIMENSION RR(N),RI(N)
c
        SCIFOLD=0
        ARMIN=RLARGE
        DO I=1,N
          IF (RI(I).EQ.0.d0) THEN
            AR=DABS(RR(I))
            IF (AR.LE.ARMIN) THEN
              SCIFOLD=I
              ARMIN=AR
            END IF
          END IF
        END DO
c
      RETURN
      END
c
c
c     ---------------------------------
      INTEGER FUNCTION SCNSTAB(RR,RI,N)
c     ---------------------------------
c
c       Computes the number of eigenvalues with negative real part.
c       On input RR(N) and RI(N) contain real and imaginary part
c       of eigenvalues.

        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'debug.h'
c
        DIMENSION RR(N),RI(N)
c
        SCNSTAB=0
        DO I=1,N
          IF (RR(I).LT.0.d0) THEN
            SCNSTAB=SCNSTAB+1
          END IF
        END DO
c
      RETURN
      END
c
c
c     --------------------------------------------
      SUBROUTINE SCEIG(A,N,ITRANS,MATZ,RR,RI,VRET)
c     --------------------------------------------
c
c       Uses EISPACK routine RG to calculate the eigenvalues/
c       eigenvectors of A(N,N) and orders them according to their
c       real parts (smallest first).
c
c       on input
c
c         MATZ=0 if only eigenvalues are desired
c
c         ITRANS=1 if left eigenvectors are desired
c         ITRANS=2 if right eigenvectors are desired
c
c       on output
c
c         RR,RI real and imaginary parts of eigenvalues, ordered
c         with respect to real parts (smallest first). Complex
c         conjugate pairs of eigenvalues appear with the eigenvalue
c         having the positive imaginary part first
c
c         VRET contains the real and imaginary parts of the
c         eigenvectors if MATZ is not zero. If the J-th eigenvalue
c         is real, the J-th column of VRET contains its eigenvector.
c         If the J-th eigenvalue is complex with positive imaginary
c         part, the J-th and (J+1)-th columns of VRET contain the
c         real and imaginary part of its eigenvector. The
c         eigenvectors are chosen, by continuity, to be commensurate
c         with those from the previous call with the same value of
c         ITRANS.
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'slidecont.h'
        INCLUDE 'debug.h'
c
        DIMENSION A(N,*),RR(*),RI(*),VRET(N,*)
c
c       Local variables
c       ---------------
        DOUBLE PRECISION VRDUM(SCNDIMX,SCNDIMX)
        INTEGER IDUM(SCNDIMX)
        DOUBLE PRECISION FV1(SCNDIMX),IV1(SCNDIMX)
        LOGICAL LPREV(2)
        DOUBLE PRECISION VRPREV(2,SCNDIMX,SCNDIMX)
c
c       State
c       -----
        SAVE LPREV,VRPREV
c
c       Transpose A if left eigenvectors are desired
c       --------------------------------------------
        IF ((MATZ.NE.0).AND.(ITRANS.EQ.1)) THEN
          DO I=1,N
            DO J=1,N
              VRDUM(I,J)=A(J,I)
            END DO
          END DO
          DO I=1,N
            DO J=1,N
              A(I,J)=VRDUM(I,J)
            END DO
          END DO
        END IF
c
c       EISPACK call for eigenvalues and eigenvectors
c       ---------------------------------------------
        IFAIL=0
        CALL RG(N,N,A,RR,RI,MATZ,VRET,IV1,FV1,IFAIL)
c
        IF (IFAIL.NE.0) THEN  
          WRITE(*,570)
        END IF
c
c       Order eigenvectors/values
c       -------------------------
c
c       Indexes vector for eigenvalues with imaginary part >= 0
        J=1
        DO I=1,N
          IF (RI(I).GE.0.0D0) THEN
            IDUM(J)=I
            J=J+1
          END IF
        END DO
        NRID=J-1
c
c       Order indexes vector
        DO I=1,NRID-1
          DO J=I+1,NRID
            IF (RR(IDUM(I)).GT.RR(IDUM(J))) THEN
              NDUM=IDUM(I)
              IDUM(I)=IDUM(J)
              IDUM(J)=NDUM
            END IF
          END DO
        END DO
c
c       Order eigenvectors/values kepping complex pairs adjacent
        J=1
        DO I=1,NRID
          FV1(J)=RR(IDUM(I))
          IV1(J)=RI(IDUM(I))
          DO K=1,N
            VRDUM(K,J)=VRET(K,IDUM(I))
          END DO
          IF (RI(IDUM(I)).GT.0.0D0) THEN
            FV1(J+1)=RR(IDUM(I))
            IV1(J+1)=RI(IDUM(I)+1)
            DO K=1,N
              VRDUM(K,J+1)=VRET(K,IDUM(I)+1)
            END DO
            J=J+1
          END IF
          J=J+1
        END DO
c
c       Copy eigenvectors/values from temporary arrays
        DO I=1,N
          RR(I)=FV1(I)
          RI(I)=IV1(I)
          DO J=1,N
            VRET(I,J)=VRDUM(I,J)
          END DO
        END DO
c
c       Choose eigenvectors
c       -------------------
        IF (.NOT.LPREV(ITRANS)) THEN
          DO J=1,N
            DO I=1,N
              VRPREV(ITRANS,I,J)=VRET(I,J)
            END DO
          END DO
          LPREV(ITRANS)=.TRUE.
        END IF
c
        DO I=1,N
          VDOT=0.0D0
          DO J=1,N
            VDOT=VDOT+VRET(J,I)*VRPREV(ITRANS,J,I)
          END DO
          IF (VDOT.LT.0.0D0) THEN
            DO J=1,N
              VRET(J,I)=-VRET(J,I)
            END DO
          END IF
          DO J=1,N
            VRPREV(ITRANS,J,I)=VRET(J,I)
          END DO
        END DO
c
      RETURN
      END
c
c
c     ------------------------
      INTEGER FUNCTION SCFJ(I)
c     ------------------------
c
c       Returns 1 if I=2, returns 2 if I=1, returns -1 otherwise.
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'debug.h'
c
        IF (I.EQ.1) THEN
          SCFJ=2
        ELSE IF (I.EQ.2) THEN
          SCFJ=1
        ELSE
          SCFJ=-1
        END IF
c
      RETURN
      END
c
c
c     --------------------------------------
      SUBROUTINE SCFINDLB(FID,IRS,ITP,FOUND)
c     --------------------------------------
c
c       Locates restart point with label IRS and determines type.
c       If the label can not be located on unit FID then FOUND will
c       be .FALSE.
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'debug.h'
c
        INTEGER FID
        LOGICAL FOUND,EOFFID
c
        FOUND=.FALSE.
        REWIND FID
c
  10    READ(FID,*,END=20)IBR,NTOTRS,ITP,LABRS,NFPR,ISWRS,NTPLRS,
     +    NARS,NSKIP
        IF (LABRS.EQ.IRS) THEN
          FOUND=.TRUE.
          BACKSPACE FID
          RETURN
        ELSE
          CALL SCSKIP(FID,NSKIP,EOFFID)
          IF (EOFFID)GOTO 20
        END IF
        GOTO 10
c
  20    CONTINUE
c
      RETURN
      END
c
c
c     ---------------------------------------
      SUBROUTINE SCREADLB(IRS,U,PAR,SCISTART)
c     ---------------------------------------
c
c       Reads the restart data for algebraic problems.
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'slidecont.h'
        INCLUDE 'auto.h'
        INCLUDE 'debug.h'
c
        DIMENSION U(*),PAR(*)
c
        INTEGER SCISTART,FID
        LOGICAL FOUND,EOFFID
c       LOGICAL LOP
c
c       Open solution file (fort.3)
        FID=3
c       DO FID=2,100
c         INQUIRE (UNIT=FID,OPENED=LOP)
c         IF (.NOT.LOP) GOTO 10
c       END DO
c       WRITE(*,520)
c       STOP
c
c 10    OPEN (FID,FILE='fort.3',STATUS='unknown')
c
c       Find label
        CALL SCFINDLB(FID,IRS,ITP,FOUND)
        IF (.NOT.FOUND) THEN
          WRITE(*,530)SCIRS
          STOP
        END IF
c
c       Read header
        READ(FID,*)IBRR,NTOTR,ITPR,LABR,NFPRR,ISWR,NTPLRS,NAR,
     +    NSKIPR,N1,N2,NPARR
        NDIM=NAR-1
c
c       Read state
        IF (SCISTART.EQ.-1) THEN
c
c         Start from u(0)
          READ(FID,*)T,(U(I),I=1,NDIM)
          NSKIP=(NDIM/7+1)*(NTPLRS-1)+(NICP-1)/20+1+(NICP-1)/7+1+ 
     +      ((NDIM-1)/7+1)*NTPLRS 
          CALL SCSKIP(FID,NSKIP,EOFFID)
c
        ELSE IF (SCISTART.EQ.-2) THEN
c
c         Start from u(1)
          NSKIP=(NDIM/7+1)*(NTPLRS-1)
          CALL SCSKIP(FID,NSKIP,EOFFID)
          READ(FID,*)T,(U(I),I=1,NDIM)
          NSKIP=(NICP-1)/20+1+(NICP-1)/7+1+((NDIM-1)/7+1)*NTPLRS 
          CALL SCSKIP(FID,NSKIP,EOFFID)
c
        ELSE IF (SCISTART.EQ.-3) THEN
c
c         Start from v(1)
          NSKIP=(NDIM/7+1)*(NTPLRS-1)
          CALL SCSKIP(FID,NSKIP,EOFFID)
          READ(FID,*)T,(U(I),I=1,NDIM)
          DO K=1,NDIM/2
            U(K)=U(K+NDIM/2)
          END DO
          NSKIP=(NICP-1)/20+1+(NICP-1)/7+1+((NDIM-1)/7+1)*NTPLRS 
          CALL SCSKIP(FID,NSKIP,EOFFID)
c
        ELSE
c
c         Start from an algebraic computation
          READ(FID,*)T,(U(I),I=1,NDIM)
c
        END IF

c       Read parameters
        IF (NPARR.GT.NPARX) THEN
          NPARR=NPARX
          WRITE(*,600)NPARR
        END IF
        READ(FID,*)(PAR(I),I=1,NPARR)
c
c       Close solution file (fort.3)
c       CLOSE(FID,STATUS='KEEP')
c
      RETURN
      END
c
c
c     -----------------------------------
      SUBROUTINE SCSKIP(FID,NSKIP,EOFFID)
c     -----------------------------------
c
c       Skips the specified number of lines on unit FID.
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'debug.h'
c
        INTEGER FID
        LOGICAL EOFFID
c
        EOFFID=.FALSE.
        DO I=1,NSKIP
          READ(FID,100,END=10)
        END DO
        RETURN
  10    EOFFID=.TRUE.
c
 100    FORMAT(1X)
c
      RETURN
      END
c

