C----------------------------------------------------------------------
C----------------------------------------------------------------------
C                               sclib1.f
C----------------------------------------------------------------------
C----------------------------------------------------------------------
C
C     SlideCont
C
C     An AUTO97 toolbox for sliding bifurcation analysis
C
C     BY  Fabio Dercole & Yuri Kuznetsov
C
C
C     VERSION 2.0 (last revision 2/2005)
C
C----------------------------------------------------------------------
C
C     ******************************
C     *                            *
C     *  Standard AUTO97 routines  *
C     *                            *
C     ******************************
C
C----------------------------------------------------------------------
C
c     ------------------------------------------------
      SUBROUTINE FUNC(NDIM,U,ICP,PAR,IJAC,F,DFDU,DFDP)
c     ------------------------------------------------
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'auto.h'
        INCLUDE 'slidecont.h'
        INCLUDE 'debug.h'
c
        DIMENSION U(NDIM),ICP(*),PAR(*)
        DIMENSION F(NDIM),DFDU(NDIM,NDIM),DFDP(NDIM,*)
c
c       Common block reference
c       ----------------------
        LOGICAL DEFINED
        INTEGER SCNDIM,SCIPS,SCIRS,SCISTART
        INTEGER SCNICP,SCICP(SCNPARX)
        INTEGER SCNPSI,SCIPSI(SCNPSIX)
        INTEGER SCNFIXED,SCIFIXED(SCNPSIX)
        INTEGER SCIDIFF,SCNUZR
        LOGICAL SCFPSI(SCNPSIX),SCFFIXED(SCNPSIX)
c
        COMMON /PROB/ DEFINED
        COMMON /PROB/ SCNDIM,SCIPS,SCIRS,SCISTART
        COMMON /PROB/ SCNICP,SCICP
        COMMON /PROB/ SCNPSI,SCIPSI
        COMMON /PROB/ SCNFIXED,SCIFIXED
        COMMON /PROB/ SCIDIFF,SCNUZR
        COMMON /PROB/ SCFPSI,SCFFIXED
c
c       Functions references
c       --------------------
        INTEGER SCFJ
c
c       Vector fields
c       -------------
        DOUBLE PRECISION FI(SCNDIMX)
        DOUBLE PRECISION DFIDX(SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DFIDP(SCNDIMX,SCNPARX)
        DOUBLE PRECISION DFIDXDX(SCNDIMX,SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DFIDXDP(SCNDIMX,SCNDIMX,SCNPARX)
c
        DOUBLE PRECISION FJ(SCNDIMX)
        DOUBLE PRECISION DFJDX(SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DFJDP(SCNDIMX,SCNPARX)
        DOUBLE PRECISION DFJDXDX(SCNDIMX,SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DFJDXDP(SCNDIMX,SCNDIMX,SCNPARX)
c
        DOUBLE PRECISION FS(SCNDIMX)
        DOUBLE PRECISION DFSDX(SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DFSDP(SCNDIMX,SCNPARX)
        DOUBLE PRECISION LAMBDA,LAMBDA1,LAMBDA2
        DOUBLE PRECISION DLAMBDADX(SCNDIMX)
        DOUBLE PRECISION DLAMBDADP(SCNPARX)
c
c       Zero initialization
        DATA FI/SCNDIMX*0.0/
        DATA DFIDX/SCNDIMX2*0.0/
        DATA DFIDP/SCNDIMXNPARX*0.0/
        DATA DFIDXDX/SCNDIMX3*0.0/
        DATA DFIDXDP/SCNDIMX2NPARX*0.0/
c
        DATA FJ/SCNDIMX*0.0/
        DATA DFJDX/SCNDIMX2*0.0/
        DATA DFJDP/SCNDIMXNPARX*0.0/
        DATA DFJDXDX/SCNDIMX3*0.0/
        DATA DFJDXDP/SCNDIMX2NPARX*0.0/
c
        DATA FS/SCNDIMX*0.0/
        DATA DFSDX/SCNDIMX2*0.0/
        DATA DFSDP/SCNDIMXNPARX*0.0/
        DATA DLAMBDADX/SCNDIMX*0.0/
        DATA DLAMBDADP/SCNPARX*0.0/
c
c       Discontinuity boundary
c       ----------------------
        DOUBLE PRECISION H
        DOUBLE PRECISION DHDX(SCNDIMX)
        DOUBLE PRECISION DHDP(SCNPARX)
        DOUBLE PRECISION DHDXDX(SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DHDXDP(SCNDIMX,SCNPARX)
c
c       Zero initialization
        DATA H/0.0/
        DATA DHDX/SCNDIMX*0.0/
        DATA DHDP/SCNPARX*0.0/
        DATA DHDXDX/SCNDIMX2*0.0/
        DATA DHDXDP/SCNDIMXNPARX*0.0/
c
c       Other local variables
c       ---------------------
        INTEGER IOVRPAR
        DOUBLE PRECISION X(SCNDIMX)
        INTEGER TMPIDIFF
c
c       Debug
        IF (DEBUG) THEN
          IF (DBGLEV.GE.DBGL1) THEN
            PRINT *,'enter in FUNC(NDIM=',NDIM,', IJAC=',IJAC,')'
            PRINT *,'U='
            CALL DARRPRN1(U,SCNDIM,1,NDIM)
            PRINT *,'ICP='
            CALL IARRPRN1(ICP,SCNICP,1,SCNICP)
            PRINT *,'PAR='
            CALL DARRPRN1(PAR,SCNPARX,1,SCNPARX)
          END IF
        END IF
c
c       Common block initialization
c       ---------------------------
        IF (.NOT.DEFINED) THEN
          CALL SCPROB
        END IF
c
c       Vector field selection
c       ----------------------
        IF (SCIPS.LT.200) THEN
          I=1
        ELSE IF (SCIPS.LT.300) THEN
          I=2
        ELSE
          I=3
        END IF

c       Debug
        IF (DEBUG) THEN
          IF (DBGLEV.GE.DBGL1) THEN
            PRINT *,'I=',I
          END IF
        END IF
c
c       DFDU/DFDP zero initialization
c       -----------------------------
        IF (IJAC.GT.0) THEN
          DO K=1,NDIM
            DO L=1,NDIM
              DFDU(K,L)=0.0D0
            END DO
          END DO
        END IF
        IF (IJAC.GT.1) THEN
          DO K=1,NDIM
            DO L=1,SCNPARX
              DFDP(K,L)=0.0D0
            END DO
          END DO
          DO K=1,NDIM
            DO L=SCIPARTS,SCIPARTJ
              DFDP(K,L)=0.0D0
            END DO
          END DO
        END IF
c
c       Problem type selection
c       ----------------------
        IF (SCIPS.LT.100) THEN
c
c         AUTO97 problem type
c         -------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'AUTO97 problem type'
            END IF
          END IF
c
c         Jacobian flag
          IF (IJAC.GE.0) THEN
            TMPIDIFF=1
          ELSE
            TMPIDIFF=0
          END IF
 
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,TMPIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
c
        ELSE IF ((SCIPS.EQ.T3D1).OR.(SCIPS.EQ.T3D2)) THEN
c
c         Tangent point (SCNDIM=3)
c         ------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Tangent point (SCNDIM=3)'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary computation
          CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +      H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H=',H
              PRINT *,'DHDX='
              CALL DARRPRN1(DHDX,SCNDIM,1,SCNDIM)
              PRINT *,'DHDP='
              CALL DARRPRN1(DHDP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          F(1)=H
          F(2)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          F(3)=U(1)-PAR(SCIPAR0+1)
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DFDU(1,K)=DHDX(K)
            DFDU(2,K)=DSCPR2(DFIDX,SCNDIM,SCNDIM,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI,SCNDIM,1,1,1,1,
     +        DHDXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
          END DO
          DFDU(3,1)=1.0D0
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DFDP(1,K)=DHDP(K)
            DFDP(2,K)=DSCPR2(DFIDP,SCNDIM,SCNPARX,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI,SCNDIM,1,1,1,1,
     +        DHDXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
          END DO
          DFDP(1,SCIPAR0+1)=0.0D0
          DFDP(2,SCIPAR0+1)=0.0D0
          DFDP(3,SCIPAR0+1)=-1.0D0
c
        ELSE IF ((SCIPS.EQ.TAN1).OR.(SCIPS.EQ.TAN2)) THEN
c
c         Tangent point (SCNDIM=2)
c         ------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Tangent point (SCNDIM=2)'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary computation
          CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +      H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H=',H
              PRINT *,'DHDX='
              CALL DARRPRN1(DHDX,SCNDIM,1,SCNDIM)
              PRINT *,'DHDP='
              CALL DARRPRN1(DHDP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          F(1)=H
          F(2)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DFDU(1,K)=DHDX(K)
            DFDU(2,K)=DSCPR2(DFIDX,SCNDIM,SCNDIM,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI,SCNDIM,1,1,1,1,
     +        DHDXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DFDP(1,K)=DHDP(K)
            DFDP(2,K)=DSCPR2(DFIDP,SCNDIM,SCNPARX,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI,SCNDIM,1,1,1,1,
     +        DHDXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
          END DO
c
        ELSE IF ((SCIPS.EQ.EQL1).OR.(SCIPS.EQ.EQL2)) THEN
c
c         Equilibrium
c         -----------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Equilibrium'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
c
        ELSE IF ((SCIPS.EQ.BEQ1).OR.(SCIPS.EQ.BEQ2)) THEN
c
c         Boundary equilibrium
c         --------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Boundary equilibrium'
            END IF
          END IF
c
c         State-parameter overspecification
          IOVRPAR=2
          PAR(SCICP(IOVRPAR))=U(SCNDIM+1)         
c         
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary computation
          CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +      H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H=',H
              PRINT *,'DHDX='
              CALL DARRPRN1(DHDX,SCNDIM,1,SCNDIM)
              PRINT *,'DHDP='
              CALL DARRPRN1(DHDP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
          CALL DARRCPY1(H,1,1,F,NDIM,SCNDIM+1,1)
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,SCICP(IOVRPAR),
     +      DFDU,NDIM,NDIM,1,SCNDIM+1,SCNDIM,1)
          CALL DARRCPY2(DHDX,1,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,SCNDIM+1,1,1,SCNDIM)
          CALL DARRCPY2(DHDP,1,SCNPARX,1,SCICP(IOVRPAR),
     +      DFDU,NDIM,NDIM,SCNDIM+1,SCNDIM+1,1,1)
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
          CALL DARRCPY2(DHDP,1,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,SCNDIM+1,1,1,SCNPARX)
c
        ELSE IF (SCIPS.EQ.BND) THEN
c
c         Discontinuity boundary (SCNDIM=2)
c         ---------------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Discontinuity boundary (SCNDIM=2)'
            END IF
          END IF
c
c         Discontinuity boundary computation
          CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +      H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H=',H
              PRINT *,'DHDX='
              CALL DARRPRN1(DHDX,SCNDIM,1,SCNDIM)
              PRINT *,'DHDP='
              CALL DARRPRN1(DHDP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          F(1)=H
          F(2)=U(1)-PAR(SCIPAR0+1)
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DFDU(1,1)=DHDX(1)
          DFDU(1,2)=DHDX(2)
          DFDU(2,1)=1.0D0
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DHDP,1,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,1,SCNPARX)
          DFDP(1,SCIPAR0+1)=0.0D0
          DFDP(2,SCIPAR0+1)=-1.0D0
c
        ELSE IF (SCIPS.EQ.PEQ) THEN
c
c         Pseudo-equilibrium
c         ------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Pseudo-equilibrium'
            END IF
          END IF
c
c         Vector field 1 computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,1)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',1,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',1,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',1,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field 2 computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,2)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',2,'='
              CALL DARRPRN1(FJ,SCNDIM,1,SCNDIM)
              PRINT *,'DF',2,'DX='
              CALL DARRPRN2(DFJDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',2,'DP='
              CALL DARRPRN2(DFJDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary computation
          CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +      H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H=',H
              PRINT *,'DHDX='
              CALL DARRPRN1(DHDX,SCNDIM,1,SCNDIM)
              PRINT *,'DHDP='
              CALL DARRPRN1(DHDP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          DO K=1,SCNDIM
            F(K)=U(SCNDIM+1)*FI(K)+U(SCNDIM+2)*FJ(K)
          END DO
          F(SCNDIM+1)=H
          F(SCNDIM+2)=U(SCNDIM+1)+U(SCNDIM+2)-1.0D0
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=U(SCNDIM+1)*DARR2(DFIDX,SCNDIM,SCNDIM,K,L)+
     +          U(SCNDIM+2)*DARR2(DFJDX,SCNDIM,SCNDIM,K,L)
            END DO
          END DO
          DO K=1,SCNDIM
            DFDU(K,SCNDIM+1)=FI(K)
            DFDU(K,SCNDIM+2)=FJ(K)
            DFDU(SCNDIM+1,K)=DHDX(K)
          END DO
          DO K=1,2
            DFDU(SCNDIM+2,SCNDIM+K)=1.0D0
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=U(SCNDIM+1)*DARR2(DFIDP,SCNDIM,SCNPARX,K,L)+
     +          U(SCNDIM+2)*DARR2(DFJDP,SCNDIM,SCNPARX,K,L)
            END DO
          END DO
          DO K=1,SCNPARX
            DFDP(SCNDIM+1,K)=DHDP(K)
          END DO
c
        ELSE IF (SCIPS.EQ.CTG) THEN
c
c         Collision of tangent points (SCNDIM=2)
c         --------------------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Collision of tangent points (SCNDIM=2)'
            END IF
          END IF
c
c         State-parameter overspecification
          IOVRPAR=2
          PAR(SCICP(IOVRPAR))=U(SCNDIM+1)         
c         
c         Vector field 1 computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,1)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',1,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field 2 computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,2)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',2,'='
              CALL DARRPRN1(FJ,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFJDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFJDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary computation
          CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +      H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H=',H
              PRINT *,'DHDX='
              CALL DARRPRN1(DHDX,SCNDIM,1,SCNDIM)
              PRINT *,'DHDP='
              CALL DARRPRN1(DHDP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          F(1)=H
          F(2)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          F(3)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DFDU(1,K)=DHDX(K)
            DFDU(2,K)=DSCPR2(DFIDX,SCNDIM,SCNDIM,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI,SCNDIM,1,1,1,1,
     +        DHDXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
            DFDU(3,K)=DSCPR2(DFJDX,SCNDIM,SCNDIM,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FJ,SCNDIM,1,1,1,1,
     +        DHDXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
          END DO
          DFDU(1,3)=DHDP(SCICP(IOVRPAR))
          DFDU(2,3)=DSCPR2(DFIDP,SCNDIM,SCNPARX,1,SCICP(IOVRPAR),1,
     +      DHDX,SCNDIM,1,1,1,1,SCNDIM)+DSCPR2(FI,SCNDIM,1,1,1,1,
     +      DHDXDP,SCNDIM,SCNPARX,1,SCICP(IOVRPAR),1,SCNDIM)
          DFDU(3,3)=DSCPR2(DFJDP,SCNDIM,SCNPARX,1,SCICP(IOVRPAR),1,
     +      DHDX,SCNDIM,1,1,1,1,SCNDIM)+DSCPR2(FJ,SCNDIM,1,1,1,1,
     +      DHDXDP,SCNDIM,SCNPARX,1,SCICP(IOVRPAR),1,SCNDIM)
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DFDP(1,K)=DHDP(K)
            DFDP(2,K)=DSCPR2(DFIDP,SCNDIM,SCNPARX,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI,SCNDIM,1,1,1,1,
     +        DHDXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
            DFDP(3,K)=DSCPR2(DFJDP,SCNDIM,SCNPARX,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FJ,SCNDIM,1,1,1,1,
     +        DHDXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
          END DO
c
        ELSE IF ((SCIPS.EQ.CYC1).OR.(SCIPS.EQ.CYC2)) THEN
c
c         Cycle
c         -----
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Cycle'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
c
        ELSE IF ((SCIPS.EQ.TGB1).OR.(SCIPS.EQ.TGB2)) THEN
c
c         Connection tangent point - discontinuity boundary (SCNDIM=2)
c         ------------------------------------------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Connection tangent point - discontinuity' //
     +          'boundary (SCNDIM=2)'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
          END DO
c
        ELSE IF ((SCIPS.EQ.TCB1).OR.(SCIPS.EQ.TCB2)) THEN
c
c         Connection tg. pt - crossing - disc. boundary (SCNDIM=2)
c         --------------------------------------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Connection tg. pt - crossing - ' //
     +          'disc. boundary (SCNDIM=2)'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation
          DO K=1,SCNDIM
            X(K)=U(SCNDIM+K)
          END DO
          CALL SCFUNC(SCNDIM,X,PAR,SCIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ,SCNDIM,1,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DX='
              CALL DARRPRN2(DFJDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DP='
              CALL DARRPRN2(DFJDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
          CALL DARRCPY1(FJ,SCNDIM,1,F,NDIM,SCNDIM+1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
            F(SCNDIM+K)=PAR(SCIPARTJ)*F(SCNDIM+K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
          CALL DARRCPY2(DFJDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,SCNDIM+1,SCNDIM+1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
              DFDU(SCNDIM+K,SCNDIM+L)=PAR(SCIPARTJ)*
     +          DFDU(SCNDIM+K,SCNDIM+L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
          CALL DARRCPY2(DFJDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,SCNDIM+1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
              DFDP(SCNDIM+K,L)=PAR(SCIPARTJ)*DFDP(SCNDIM+K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
            DFDP(SCNDIM+K,SCIPARTJ)=FJ(K)
          END DO
c
        ELSE IF ((SCIPS.EQ.PEB1).OR.(SCIPS.EQ.PEB2)) THEN
c
c         Connection pseudo-equilibrium - discontinuity boundary
c         ------------------------------------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Connection pseudo-equilibrium - ' //
     +          'discontinuity boundary'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
          END DO
c
        ELSE IF ((SCIPS.EQ.PCB1).OR.(SCIPS.EQ.PCB2)) THEN
c
c         Connection pseudo-eq. - crossing - discontinuity boundary
c         ---------------------------------------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Connection pseudo-eq. - crossing - ' //
     +          'discontinuity boundary'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation
          DO K=1,SCNDIM
            X(K)=U(SCNDIM+K)
          END DO
          CALL SCFUNC(SCNDIM,X,PAR,SCIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ,SCNDIM,1,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DX='
              CALL DARRPRN2(DFJDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DP='
              CALL DARRPRN2(DFJDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
          CALL DARRCPY1(FJ,SCNDIM,1,F,NDIM,SCNDIM+1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
            F(SCNDIM+K)=PAR(SCIPARTJ)*F(SCNDIM+K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
          CALL DARRCPY2(DFJDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,SCNDIM+1,SCNDIM+1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
              DFDU(SCNDIM+K,SCNDIM+L)=PAR(SCIPARTJ)*
     +          DFDU(SCNDIM+K,SCNDIM+L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
          CALL DARRCPY2(DFJDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,SCNDIM+1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
              DFDP(SCNDIM+K,L)=PAR(SCIPARTJ)*DFDP(SCNDIM+K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
            DFDP(SCNDIM+K,SCIPARTJ)=FJ(K)
          END DO
c
        ELSE IF ((SCIPS.EQ.TCH1).OR.(SCIPS.EQ.TCH2)) THEN
c
c         Grazing
c         -------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Grazing'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
          END DO
c
        ELSE IF ((SCIPS.EQ.TGT1).OR.(SCIPS.EQ.TGT2)) THEN
c
c         Connection tg. pt - tg. pt of the same v.f. (SCNDIM=2)
c         ------------------------------------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Connection tg. pt - tg. pt of the' //
     +          'same v.f. (SCNDIM=2)'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
          END DO
c
       ELSE IF ((SCIPS.EQ.TCT1).OR.(SCIPS.EQ.TCT2)) THEN
c
c         Connection tg. pt - cros. - tg. pt same v.f. (SCNDIM=2)
c         -------------------------------------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Connection tg. pt - cros. - tg. pt ' //
     +          'same v.f. (SCNDIM=2)'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation
          DO K=1,SCNDIM
            X(K)=U(SCNDIM+K)
          END DO
          CALL SCFUNC(SCNDIM,X,PAR,SCIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ,SCNDIM,1,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DX='
              CALL DARRPRN2(DFJDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DP='
              CALL DARRPRN2(DFJDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
          CALL DARRCPY1(FJ,SCNDIM,1,F,NDIM,SCNDIM+1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
            F(SCNDIM+K)=PAR(SCIPARTJ)*F(SCNDIM+K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
          CALL DARRCPY2(DFJDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,SCNDIM+1,SCNDIM+1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
              DFDU(SCNDIM+K,SCNDIM+L)=PAR(SCIPARTJ)*
     +          DFDU(SCNDIM+K,SCNDIM+L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
          CALL DARRCPY2(DFJDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,SCNDIM+1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
              DFDP(SCNDIM+K,L)=PAR(SCIPARTJ)*DFDP(SCNDIM+K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
            DFDP(SCNDIM+K,SCIPARTJ)=FJ(K)
          END DO
c
        ELSE IF ((SCIPS.EQ.TTD1).OR.(SCIPS.EQ.TTD2)) THEN
c
c         Connection tg. pt - tg. pt of different v.f. (SCNDIM=2)
c         -------------------------------------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Connection tg. pt - tg. pt of ' //
     +          'different v.f. (SCNDIM=2)'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
          END DO
c
        ELSE IF ((SCIPS.EQ.TCD1).OR.(SCIPS.EQ.TCD2)) THEN
c
c         Connection tg. pt - cros. - tg. pt different v.f. (SCNDIM=2)
c         ------------------------------------------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Connection tg. pt - cros. - tg. pt' //
     +          'different v.f. (SCNDIM=2)'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation
          DO K=1,SCNDIM
            X(K)=U(SCNDIM+K)
          END DO
          CALL SCFUNC(SCNDIM,X,PAR,SCIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ,SCNDIM,1,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DX='
              CALL DARRPRN2(DFJDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DP='
              CALL DARRPRN2(DFJDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
          CALL DARRCPY1(FJ,SCNDIM,1,F,NDIM,SCNDIM+1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
            F(SCNDIM+K)=PAR(SCIPARTJ)*F(SCNDIM+K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
          CALL DARRCPY2(DFJDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,SCNDIM+1,SCNDIM+1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
              DFDU(SCNDIM+K,SCNDIM+L)=PAR(SCIPARTJ)*
     +          DFDU(SCNDIM+K,SCNDIM+L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
          CALL DARRCPY2(DFJDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,SCNDIM+1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
              DFDP(SCNDIM+K,L)=PAR(SCIPARTJ)*DFDP(SCNDIM+K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
            DFDP(SCNDIM+K,SCIPARTJ)=FJ(K)
          END DO
c
        ELSE IF ((SCIPS.EQ.TGP1).OR.(SCIPS.EQ.TGP2)) THEN
c
c         Connection tangent point - pseudo-equilibrium
c         ---------------------------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Connection tangent point - pseudo-equilibrium'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
          END DO
c
        ELSE IF ((SCIPS.EQ.TCP1).OR.(SCIPS.EQ.TCP2)) THEN
c
c         Connection tangent point - crossing - pseudo-equilibrium
c         --------------------------------------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Connection tangent point - crossing - ' //
     +          'pseudo-equilibrium'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation
          DO K=1,SCNDIM
            X(K)=U(SCNDIM+K)
          END DO
          CALL SCFUNC(SCNDIM,X,PAR,SCIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ,SCNDIM,1,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DX='
              CALL DARRPRN2(DFJDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DP='
              CALL DARRPRN2(DFJDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
          CALL DARRCPY1(FJ,SCNDIM,1,F,NDIM,SCNDIM+1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
            F(SCNDIM+K)=PAR(SCIPARTJ)*F(SCNDIM+K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
          CALL DARRCPY2(DFJDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,SCNDIM+1,SCNDIM+1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
              DFDU(SCNDIM+K,SCNDIM+L)=PAR(SCIPARTJ)*
     +          DFDU(SCNDIM+K,SCNDIM+L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
          CALL DARRCPY2(DFJDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,SCNDIM+1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
              DFDP(SCNDIM+K,L)=PAR(SCIPARTJ)*DFDP(SCNDIM+K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
            DFDP(SCNDIM+K,SCIPARTJ)=FJ(K)
          END DO
c
        ELSE IF ((SCIPS.EQ.TGS1).OR.(SCIPS.EQ.TGS2)) THEN
c
c         Connection tangent point - saddle (SCNDIM=2)
c         --------------------------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Connection tangent point - saddle (SCNDIM=2)'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
          END DO
c
        ELSE IF ((SCIPS.EQ.TCS1).OR.(SCIPS.EQ.TCS2)) THEN
c
c         Connection tangent point - crossing - saddle (SCNDIM=2)
c         -------------------------------------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Connection tangent point - crossing - ' //
     +          'saddle (SCNDIM=2)'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation
          DO K=1,SCNDIM
            X(K)=U(SCNDIM+K)
          END DO
          CALL SCFUNC(SCNDIM,X,PAR,SCIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ,SCNDIM,1,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DX='
              CALL DARRPRN2(DFJDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DP='
              CALL DARRPRN2(DFJDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
          CALL DARRCPY1(FJ,SCNDIM,1,F,NDIM,SCNDIM+1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
            F(SCNDIM+K)=PAR(SCIPARTJ)*F(SCNDIM+K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
          CALL DARRCPY2(DFJDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,SCNDIM+1,SCNDIM+1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
              DFDU(SCNDIM+K,SCNDIM+L)=PAR(SCIPARTJ)*
     +          DFDU(SCNDIM+K,SCNDIM+L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
          CALL DARRCPY2(DFJDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,SCNDIM+1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
              DFDP(SCNDIM+K,L)=PAR(SCIPARTJ)*DFDP(SCNDIM+K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
            DFDP(SCNDIM+K,SCIPARTJ)=FJ(K)
          END DO
c
        ELSE IF ((SCIPS.EQ.PET1).OR.(SCIPS.EQ.PET2)) THEN
c
c         Connection pseudo-eq. - tangent point of the same v.f.
c         ------------------------------------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Connection pseudo-eq. - tangent point of ' //
     +          'the same v.f.'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
          END DO
c
        ELSE IF ((SCIPS.EQ.PCT1).OR.(SCIPS.EQ.PCT2)) THEN
c
c         Connection pseudo-eq. - crossing - tg. pt of the same v.f.
c         ----------------------------------------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Connection pseudo-eq. - crossing - tg. pt' //
     +          'of the same v.f.'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation
          DO K=1,SCNDIM
            X(K)=U(SCNDIM+K)
          END DO
          CALL SCFUNC(SCNDIM,X,PAR,SCIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ,SCNDIM,1,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DX='
              CALL DARRPRN2(DFJDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DP='
              CALL DARRPRN2(DFJDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
          CALL DARRCPY1(FJ,SCNDIM,1,F,NDIM,SCNDIM+1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
            F(SCNDIM+K)=PAR(SCIPARTJ)*F(SCNDIM+K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
          CALL DARRCPY2(DFJDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,SCNDIM+1,SCNDIM+1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
              DFDU(SCNDIM+K,SCNDIM+L)=PAR(SCIPARTJ)*
     +          DFDU(SCNDIM+K,SCNDIM+L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
          CALL DARRCPY2(DFJDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,SCNDIM+1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
              DFDP(SCNDIM+K,L)=PAR(SCIPARTJ)*DFDP(SCNDIM+K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
            DFDP(SCNDIM+K,SCIPARTJ)=FJ(K)
          END DO
c
        ELSE IF ((SCIPS.EQ.PTD1).OR.(SCIPS.EQ.PTD2)) THEN
c
c         Connection pseudo-eq. - tangent point of different v.f.
c         -------------------------------------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Connection pseudo-eq. - tangent point of ' //
     +          'different v.f.'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
          END DO
c
        ELSE IF ((SCIPS.EQ.PCD1).OR.(SCIPS.EQ.PCD2)) THEN
c
c         Connection pseudo-eq. - crossing - tg. pt of different v.f.
c         -----------------------------------------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Connection pseudo-eq. - crossing - tg. pt' //
     +          'of different v.f.'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation
          DO K=1,SCNDIM
            X(K)=U(SCNDIM+K)
          END DO
          CALL SCFUNC(SCNDIM,X,PAR,SCIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ,SCNDIM,1,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DX='
              CALL DARRPRN2(DFJDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DP='
              CALL DARRPRN2(DFJDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
          CALL DARRCPY1(FJ,SCNDIM,1,F,NDIM,SCNDIM+1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
            F(SCNDIM+K)=PAR(SCIPARTJ)*F(SCNDIM+K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
          CALL DARRCPY2(DFJDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,SCNDIM+1,SCNDIM+1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
              DFDU(SCNDIM+K,SCNDIM+L)=PAR(SCIPARTJ)*
     +          DFDU(SCNDIM+K,SCNDIM+L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
          CALL DARRCPY2(DFJDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,SCNDIM+1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
              DFDP(SCNDIM+K,L)=PAR(SCIPARTJ)*DFDP(SCNDIM+K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
            DFDP(SCNDIM+K,SCIPARTJ)=FJ(K)
          END DO
c
        ELSE IF ((SCIPS.EQ.PEP1).OR.(SCIPS.EQ.PEP2)) THEN
c
c         Connection pseudo-eq. - pseudo-eq. (SCNDIM=2)
c         ---------------------------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Connection pseudo-eq. - ' //
     +          'pseudo-eq. (SCNDIM=2)'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
          END DO
c
        ELSE IF ((SCIPS.EQ.PCP1).OR.(SCIPS.EQ.PCP2)) THEN
c
c         Connection pseudo-eq. - crossing - pseudo-eq. (SCNDIM=2)
c         --------------------------------------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Connection pseudo-eq. - crossing - ' //
     +          'pseudo-eq. (SCNDIM=2)'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation
          DO K=1,SCNDIM
            X(K)=U(SCNDIM+K)
          END DO
          CALL SCFUNC(SCNDIM,X,PAR,SCIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ,SCNDIM,1,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DX='
              CALL DARRPRN2(DFJDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DP='
              CALL DARRPRN2(DFJDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
          CALL DARRCPY1(FJ,SCNDIM,1,F,NDIM,SCNDIM+1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
            F(SCNDIM+K)=PAR(SCIPARTJ)*F(SCNDIM+K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
          CALL DARRCPY2(DFJDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,SCNDIM+1,SCNDIM+1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
              DFDU(SCNDIM+K,SCNDIM+L)=PAR(SCIPARTJ)*
     +          DFDU(SCNDIM+K,SCNDIM+L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
          CALL DARRCPY2(DFJDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,SCNDIM+1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
              DFDP(SCNDIM+K,L)=PAR(SCIPARTJ)*DFDP(SCNDIM+K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
            DFDP(SCNDIM+K,SCIPARTJ)=FJ(K)
          END DO
c
        ELSE IF ((SCIPS.EQ.PES1).OR.(SCIPS.EQ.PES2)) THEN
c
c         Connection pseudo-equilibrium - saddle
c         --------------------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Connection pseudo-equilibrium - saddle'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
          END DO
c
        ELSE IF ((SCIPS.EQ.PCS1).OR.(SCIPS.EQ.PCS2)) THEN
c
c         Connection pseudo-equilibrium - crossing - saddle
c         -------------------------------------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Connection pseudo-equilibrium - crossing - ' //
     +          'saddle'
            END IF
          END IF
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation
          DO K=1,SCNDIM
            X(K)=U(SCNDIM+K)
          END DO
          CALL SCFUNC(SCNDIM,X,PAR,SCIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ,SCNDIM,1,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DX='
              CALL DARRPRN2(DFJDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DP='
              CALL DARRPRN2(DFJDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
          CALL DARRCPY1(FJ,SCNDIM,1,F,NDIM,SCNDIM+1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
            F(SCNDIM+K)=PAR(SCIPARTJ)*F(SCNDIM+K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
          CALL DARRCPY2(DFJDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,SCNDIM+1,SCNDIM+1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
              DFDU(SCNDIM+K,SCNDIM+L)=PAR(SCIPARTJ)*
     +          DFDU(SCNDIM+K,SCNDIM+L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
          CALL DARRCPY2(DFJDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,SCNDIM+1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
              DFDP(SCNDIM+K,L)=PAR(SCIPARTJ)*DFDP(SCNDIM+K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
            DFDP(SCNDIM+K,SCIPARTJ)=FJ(K)
          END DO
c
        ELSE IF ((SCIPS.EQ.SCY1).OR.(SCIPS.EQ.SCY2)) THEN
c
c         Sliding cycle
c         -------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Sliding cycle'
            END IF
          END IF
c
c         Sliding point
          DO K=1,SCNDIM
            X(K)=U(SCNDIM+K)
          END DO
c
c         Vector field 1 computation
          CALL SCFUNC(SCNDIM,X,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,1)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',1,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',1,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',1,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field 2 computation
          CALL SCFUNC(SCNDIM,X,PAR,SCIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,2)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',2,'='
              CALL DARRPRN1(FJ,SCNDIM,1,SCNDIM)
              PRINT *,'DF',2,'DX='
              CALL DARRPRN2(DFJDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',2,'DP='
              CALL DARRPRN2(DFJDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary computation
          CALL SCBOUND(SCNDIM,X,PAR,SCIDIFF,
     +      H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H=',H
              PRINT *,'DHDX='
              CALL DARRPRN1(DHDX,SCNDIM,1,SCNDIM)
              PRINT *,'DHDP='
              CALL DARRPRN1(DHDP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Sliding vector field
          LAMBDA1=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          LAMBDA2=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          LAMBDA=LAMBDA2/(LAMBDA2-LAMBDA1)
          DO K=1,SCNDIM
            FS(K)=LAMBDA*FI(K)+(1.0D0-LAMBDA)*FJ(K)
c    +        -H*DHDX(K)
          END DO
c
          DO K=1,SCNDIM
            DLAMBDADX(K)=(DSCPR2(DFJDX,SCNDIM,SCNDIM,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FJ,SCNDIM,1,1,1,1,
     +        DHDXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM))/(LAMBDA2-LAMBDA1)-
     +        LAMBDA2*(DSCPR2(DFJDX,SCNDIM,SCNDIM,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FJ,SCNDIM,1,1,1,1,
     +        DHDXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)-
     +        DSCPR2(DFIDX,SCNDIM,SCNDIM,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)-
     +        DSCPR2(FI,SCNDIM,1,1,1,1,
     +        DHDXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM))/(LAMBDA2-LAMBDA1)**2
            DO L=1,SCNDIM
              DFSDX(L,K)=DLAMBDADX(K)*FI(L)+LAMBDA*DFIDX(L,K)-
     +          DLAMBDADX(K)*FJ(L)+(1.0D0-LAMBDA)*DFJDX(L,K)
c    +          -DHDX(K)*DHDX(L)-H*DHDXDX(L,K)
            END DO
          END DO
c
          DO K=1,SCNPARX
            DLAMBDADP(K)=(DSCPR2(DFJDP,SCNDIM,SCNPARX,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FJ,SCNDIM,1,1,1,1,
     +        DHDXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM))/(LAMBDA2-LAMBDA1)-
     +        LAMBDA2*(DSCPR2(DFJDP,SCNDIM,SCNPARX,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FJ,SCNDIM,1,1,1,1,
     +        DHDXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)-
     +        DSCPR2(DFIDP,SCNDIM,SCNPARX,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)-
     +        DSCPR2(FI,SCNDIM,1,1,1,1,
     +        DHDXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM))/(LAMBDA2-LAMBDA1)**2
            DO L=1,SCNDIM
              DFSDP(L,K)=DLAMBDADP(K)*FI(L)+LAMBDA*DFIDP(L,K)-
     +          DLAMBDADP(K)*FJ(L)+(1.0D0-LAMBDA)*DFJDP(L,K)
c    +          -DHDP(K)*DHDX(L)-H*DHDXDP(L,K)
            END DO
          END DO
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
          CALL DARRCPY1(FS,SCNDIM,1,F,NDIM,SCNDIM+1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
            F(SCNDIM+K)=PAR(SCIPARTS)*F(SCNDIM+K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
          CALL DARRCPY2(DFSDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,SCNDIM+1,SCNDIM+1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
              DFDU(SCNDIM+K,SCNDIM+L)=PAR(SCIPARTS)*
     +          DFDU(SCNDIM+K,SCNDIM+L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
          CALL DARRCPY2(DFSDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,SCNDIM+1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
              DFDP(SCNDIM+K,L)=PAR(SCIPARTS)*DFDP(SCNDIM+K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
            DFDP(SCNDIM+K,SCIPARTS)=FS(K)
          END DO
c
        ELSE IF ((SCIPS.EQ.SCC1).OR.(SCIPS.EQ.SCC2)) THEN
c
c         Sliding-crossing cycle
c         ----------------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Sliding-crossing cycle'
            END IF
          END IF
c
c         Sliding point
          DO K=1,SCNDIM
            X(K)=U(2*SCNDIM+K)
          END DO
c
c         Vector field 1 computation
          CALL SCFUNC(SCNDIM,X,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,1)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',1,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',1,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',1,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field 2 computation
          CALL SCFUNC(SCNDIM,X,PAR,SCIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,2)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',2,'='
              CALL DARRPRN1(FJ,SCNDIM,1,SCNDIM)
              PRINT *,'DF',2,'DX='
              CALL DARRPRN2(DFJDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',2,'DP='
              CALL DARRPRN2(DFJDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary computation
          CALL SCBOUND(SCNDIM,X,PAR,SCIDIFF,
     +      H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H=',H
              PRINT *,'DHDX='
              CALL DARRPRN1(DHDX,SCNDIM,1,SCNDIM)
              PRINT *,'DHDP='
              CALL DARRPRN1(DHDP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Sliding vector field
          LAMBDA1=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          LAMBDA2=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          LAMBDA=LAMBDA2/(LAMBDA2-LAMBDA1)
          DO K=1,SCNDIM
            FS(K)=LAMBDA*FI(K)+(1.0D0-LAMBDA)*FJ(K)
c    +        -H*DHDX(K)
          END DO
c
          DO K=1,SCNDIM
            DLAMBDADX(K)=(DSCPR2(DFJDX,SCNDIM,SCNDIM,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FJ,SCNDIM,1,1,1,1,
     +        DHDXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM))/(LAMBDA2-LAMBDA1)-
     +        LAMBDA2*(DSCPR2(DFJDX,SCNDIM,SCNDIM,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FJ,SCNDIM,1,1,1,1,
     +        DHDXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)-
     +        DSCPR2(DFIDX,SCNDIM,SCNDIM,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)-
     +        DSCPR2(FI,SCNDIM,1,1,1,1,
     +        DHDXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM))/(LAMBDA2-LAMBDA1)**2
            DO L=1,SCNDIM
              DFSDX(L,K)=DLAMBDADX(K)*FI(L)+LAMBDA*DFIDX(L,K)-
     +          DLAMBDADX(K)*FJ(L)+(1.0D0-LAMBDA)*DFJDX(L,K)
c    +          -DHDX(K)*DHDX(L)-H*DHDXDX(L,K)
            END DO
          END DO
c
          DO K=1,SCNPARX
            DLAMBDADP(K)=(DSCPR2(DFJDP,SCNDIM,SCNPARX,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FJ,SCNDIM,1,1,1,1,
     +        DHDXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM))/(LAMBDA2-LAMBDA1)-
     +        LAMBDA2*(DSCPR2(DFJDP,SCNDIM,SCNPARX,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FJ,SCNDIM,1,1,1,1,
     +        DHDXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)-
     +        DSCPR2(DFIDP,SCNDIM,SCNPARX,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)-
     +        DSCPR2(FI,SCNDIM,1,1,1,1,
     +        DHDXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM))/(LAMBDA2-LAMBDA1)**2
            DO L=1,SCNDIM
              DFSDP(L,K)=DLAMBDADP(K)*FI(L)+LAMBDA*DFIDP(L,K)-
     +          DLAMBDADP(K)*FJ(L)+(1.0D0-LAMBDA)*DFJDP(L,K)
c    +          -DHDP(K)*DHDX(L)-H*DHDXDP(L,K)
            END DO
          END DO
c
c         Vector field I computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation
          DO K=1,SCNDIM
            X(K)=U(SCNDIM+K)
          END DO
          CALL SCFUNC(SCNDIM,X,PAR,SCIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ,SCNDIM,1,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DX='
              CALL DARRPRN2(DFJDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DP='
              CALL DARRPRN2(DFJDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
          CALL DARRCPY1(FJ,SCNDIM,1,F,NDIM,SCNDIM+1,SCNDIM)
          CALL DARRCPY1(FS,SCNDIM,1,F,NDIM,2*SCNDIM+1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
            F(SCNDIM+K)=PAR(SCIPARTJ)*F(SCNDIM+K)
            F(2*SCNDIM+K)=PAR(SCIPARTS)*F(2*SCNDIM+K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
          CALL DARRCPY2(DFJDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,SCNDIM+1,SCNDIM+1,SCNDIM,SCNDIM)
          CALL DARRCPY2(DFSDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,2*SCNDIM+1,2*SCNDIM+1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
              DFDU(SCNDIM+K,SCNDIM+L)=PAR(SCIPARTJ)*
     +          DFDU(SCNDIM+K,SCNDIM+L)
              DFDU(2*SCNDIM+K,2*SCNDIM+L)=PAR(SCIPARTS)*
     +          DFDU(2*SCNDIM+K,2*SCNDIM+L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
          CALL DARRCPY2(DFJDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,SCNDIM+1,1,SCNDIM,SCNPARX)
          CALL DARRCPY2(DFSDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,2*SCNDIM+1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
              DFDP(SCNDIM+K,L)=PAR(SCIPARTJ)*DFDP(SCNDIM+K,L)
              DFDP(2*SCNDIM+K,L)=PAR(SCIPARTS)*DFDP(2*SCNDIM+K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
            DFDP(SCNDIM+K,SCIPARTJ)=FJ(K)
            DFDP(2*SCNDIM+K,SCIPARTS)=FS(K)
          END DO
c
        ELSE IF (SCIPS.EQ.CCY) THEN
c
c         Crossing cycle
c         --------------
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'Crossing cycle'
            END IF
          END IF
c
c         Vector field 1 computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,1)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',1,'='
              CALL DARRPRN1(FI,SCNDIM,1,SCNDIM)
              PRINT *,'DF',1,'DX='
              CALL DARRPRN2(DFIDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',1,'DP='
              CALL DARRPRN2(DFIDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field 2 computation
          DO K=1,SCNDIM
            X(K)=U(SCNDIM+K)
          END DO
          CALL SCFUNC(SCNDIM,X,PAR,SCIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,2)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',2,'='
              CALL DARRPRN1(FJ,SCNDIM,1,SCNDIM)
              PRINT *,'DF',2,'DX='
              CALL DARRPRN2(DFJDX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',2,'DP='
              CALL DARRPRN2(DFJDP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Auto97 right-hand side
          CALL DARRCPY1(FI,SCNDIM,1,F,NDIM,1,SCNDIM)
          CALL DARRCPY1(FJ,SCNDIM,1,F,NDIM,SCNDIM+1,SCNDIM)
c
          DO K=1,SCNDIM
            F(K)=PAR(SCIPARTI)*F(K)
            F(SCNDIM+K)=PAR(SCIPARTJ)*F(SCNDIM+K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          CALL DARRCPY2(DFIDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,1,1,SCNDIM,SCNDIM)
          CALL DARRCPY2(DFJDX,SCNDIM,SCNDIM,1,1,
     +      DFDU,NDIM,NDIM,SCNDIM+1,SCNDIM+1,SCNDIM,SCNDIM)
c
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DFDU(K,L)=PAR(SCIPARTI)*DFDU(K,L)
              DFDU(SCNDIM+K,SCNDIM+L)=PAR(SCIPARTJ)*
     +          DFDU(SCNDIM+K,SCNDIM+L)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          CALL DARRCPY2(DFIDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
          CALL DARRCPY2(DFJDP,SCNDIM,SCNPARX,1,1,
     +      DFDP,NDIM,SCNPARX,SCNDIM+1,1,SCNDIM,SCNPARX)
c
          DO K=1,SCNDIM
            DO L=1,SCNPARX
              DFDP(K,L)=PAR(SCIPARTI)*DFDP(K,L)
              DFDP(SCNDIM+K,L)=PAR(SCIPARTJ)*DFDP(SCNDIM+K,L)
            END DO
            DFDP(K,SCIPARTI)=FI(K)
            DFDP(SCNDIM+K,SCIPARTJ)=FJ(K)
          END DO
c
        ELSE
          WRITE(*,510)SCIPS
          STOP
        END IF
c
c       Debug
  10    IF (DEBUG) THEN
          IF (DBGLEV.GE.DBGL1) THEN
            PRINT *,'F='
            CALL DARRPRN1(F,NDIM,1,NDIM)
            PRINT *,'DFDU='
            CALL DARRPRN2(DFDU,NDIM,NDIM,1,1,NDIM,NDIM)
            PRINT *,'DFDP='
            CALL DARRPRN2(DFDP,NDIM,SCNPARX,1,1,NDIM,SCNPARX)
            PRINT *,'exit from FUNC'
          END IF
        END IF
c
      RETURN
      END
c
c
c     ------------------------------
      SUBROUTINE STPNT(NDIM,U,PAR,T)
c     ------------------------------
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'slidecont.h'
        INCLUDE 'debug.h'
c
        DIMENSION U(NDIM),PAR(*)
c
c       Common block reference
c       ----------------------
        LOGICAL DEFINED
        INTEGER SCNDIM,SCIPS,SCIRS,SCISTART
        INTEGER SCNICP,SCICP(SCNPARX)
        INTEGER SCNPSI,SCIPSI(SCNPSIX)
        INTEGER SCNFIXED,SCIFIXED(SCNPSIX)
        INTEGER SCIDIFF,SCNUZR
        LOGICAL SCFPSI(SCNPSIX),SCFFIXED(SCNPSIX)
c
        COMMON /PROB/ DEFINED
        COMMON /PROB/ SCNDIM,SCIPS,SCIRS,SCISTART
        COMMON /PROB/ SCNICP,SCICP
        COMMON /PROB/ SCNPSI,SCIPSI
        COMMON /PROB/ SCNFIXED,SCIFIXED
        COMMON /PROB/ SCIDIFF,SCNUZR
        COMMON /PROB/ SCFPSI,SCFFIXED
c
c       Functions references
c       --------------------
        INTEGER SCFJ
c
c       Vector fields
c       -------------
        DOUBLE PRECISION FI(SCNDIMX)
        DOUBLE PRECISION DFIDX(SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DFIDP(SCNDIMX,SCNPARX)
        DOUBLE PRECISION DFIDXDX(SCNDIMX,SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DFIDXDP(SCNDIMX,SCNDIMX,SCNPARX)
c
        DOUBLE PRECISION FJ(SCNDIMX)
        DOUBLE PRECISION DFJDX(SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DFJDP(SCNDIMX,SCNPARX)
        DOUBLE PRECISION DFJDXDX(SCNDIMX,SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DFJDXDP(SCNDIMX,SCNDIMX,SCNPARX)
c
c       Zero initialization
        DATA FI/SCNDIMX*0.0/
        DATA DFIDX/SCNDIMX2*0.0/
        DATA DFIDP/SCNDIMXNPARX*0.0/
        DATA DFIDXDX/SCNDIMX3*0.0/
        DATA DFIDXDP/SCNDIMX2NPARX*0.0/
c
        DATA FJ/SCNDIMX*0.0/
        DATA DFJDX/SCNDIMX2*0.0/
        DATA DFJDP/SCNDIMXNPARX*0.0/
        DATA DFJDXDX/SCNDIMX3*0.0/
        DATA DFJDXDP/SCNDIMX2NPARX*0.0/
c
c       Discontinuity boundary
c       ----------------------
        DOUBLE PRECISION H
        DOUBLE PRECISION DHDX(SCNDIMX)
        DOUBLE PRECISION DHDP(SCNPARX)
        DOUBLE PRECISION DHDXDX(SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DHDXDP(SCNDIMX,SCNPARX)
c
c       Zero initialization
        DATA H/0.0/
        DATA DHDX/SCNDIMX*0.0/
        DATA DHDP/SCNPARX*0.0/
        DATA DHDXDX/SCNDIMX2*0.0/
        DATA DHDXDP/SCNDIMXNPARX*0.0/
c
c       Other variables
c       ---------------
        DOUBLE PRECISION RR(SCNDIMX),RI(SCNDIMX),VRET(SCNDIMX,SCNDIMX)
        DOUBLE PRECISION U0(SCNDIMX)
        DOUBLE PRECISION U1(SCNDIMX)
        DOUBLE PRECISION FIMODL
        DOUBLE PRECISION FJMODL
        DOUBLE PRECISION FIMODR
        DOUBLE PRECISION FJMODR
        INTEGER TMPIDIFF
        INTEGER IOVRPAR
c
c       Debug
        IF (DEBUG) THEN
          IF (DBGLEV.GE.DBGL2) THEN
            PRINT *,'enter in STPNT'
          END IF
        END IF
c
c       Common block initialization
c       ---------------------------
        IF (.NOT.DEFINED) THEN
          CALL SCPROB
        END IF
c
c       Vector field selection
c       ----------------------
        IF (SCIPS.LT.200) THEN
          I=1
        ELSE IF (SCIPS.LT.300) THEN
          I=2
        ELSE
          I=3
        END IF
c
c       Problem type selection
c       ----------------------
        IF (SCIPS.LT.100) THEN
c
c         AUTO97 problem type
c         -------------------
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
        ELSE IF ((SCIPS.EQ.T3D1).OR.(SCIPS.EQ.T3D2)) THEN
c
c         Tangent point (SCNDIM=3)
c         ------------------------
          IF (SCIRS.GT.0) THEN
            CALL SCREADLB(SCIRS,U,PAR,SCISTART)
          ELSE
            CALL SCSTPNT(SCNDIM,U,PAR,T,I)
          END IF
c
c         State-parameter overspecification
          PAR(SCIPAR0+1)=U(1)
c
        ELSE IF ((SCIPS.EQ.TAN1).OR.(SCIPS.EQ.TAN2)) THEN
c
c         Tangent point (SCNDIM=2)
c         ------------------------
          IF (SCIRS.GT.0) THEN
            CALL SCREADLB(SCIRS,U,PAR,SCISTART)
          ELSE
            CALL SCSTPNT(SCNDIM,U,PAR,T,I)
          END IF
c
        ELSE IF ((SCIPS.EQ.EQL1).OR.(SCIPS.EQ.EQL2)) THEN
c
c         Equilibrium
c         -----------
          IF (SCIRS.GT.0) THEN
            CALL SCREADLB(SCIRS,U,PAR,SCISTART)
          ELSE
            CALL SCSTPNT(SCNDIM,U,PAR,T,I)
          END IF
c
        ELSE IF ((SCIPS.EQ.BEQ1).OR.(SCIPS.EQ.BEQ2)) THEN
c
c         Boundary equilibrium
c         --------------------
          IF (SCIRS.GT.0) THEN
            CALL SCREADLB(SCIRS,U,PAR,SCISTART)
          ELSE
            CALL SCSTPNT(SCNDIM,U,PAR,T,I)
          END IF
c
c         State-parameter overspecification
          IOVRPAR=2
          U(SCNDIM+1)=PAR(SCICP(IOVRPAR))
c
        ELSE IF (SCIPS.EQ.BND) THEN
c
c         Discontinuity boundary (SCNDIM=2)
c         ---------------------------------
          IF (SCIRS.GT.0) THEN
            CALL SCREADLB(SCIRS,U,PAR,SCISTART)
          ELSE
            CALL SCSTPNT(SCNDIM,U,PAR,T,I)
          END IF
c
c         State-parameter overspecification
          PAR(SCIPAR0+1)=U(1)
c
        ELSE IF (SCIPS.EQ.PEQ) THEN
c
c         Pseudo-equilibrium
c         ------------------
          IF (SCIRS.GT.0) THEN
            CALL SCREADLB(SCIRS,U,PAR,SCISTART)
          ELSE
            CALL SCSTPNT(SCNDIM,U,PAR,T,I)
          END IF
c
c         Vector field 1 computation
          TMPIDIFF=0
          CALL SCFUNC(SCNDIM,U,PAR,TMPIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,1)
c
c         Vector field 2 computation
          CALL SCFUNC(SCNDIM,U,PAR,TMPIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,2)
c
c         Vector field 1,2 norm computation
          FIMODL=DSQRT(DSCPR1(FI,SCNDIM,1,FI,SCNDIM,1,SCNDIM))
          FJMODL=DSQRT(DSCPR1(FJ,SCNDIM,1,FJ,SCNDIM,1,SCNDIM))
c
c         State-parameter overspecification
          U(SCNDIM+1)=FJMODL/(FIMODL+FJMODL)
          U(SCNDIM+2)=FIMODL/(FIMODL+FJMODL)
c
        ELSE IF (SCIPS.EQ.CTG) THEN
c
c         Collision of tangent points (SCNDIM=2)
c         --------------------------------------
          IF (SCIRS.GT.0) THEN
            CALL SCREADLB(SCIRS,U,PAR,SCISTART)
          ELSE
            CALL SCSTPNT(SCNDIM,U,PAR,T,I)
          END IF
c
c         State-parameter overspecification
          IOVRPAR=2
          U(SCNDIM+1)=PAR(SCICP(IOVRPAR))
c
        ELSE IF ((SCIPS.EQ.CYC1).OR.(SCIPS.EQ.CYC2)) THEN
c
c         Cycle
c         -----
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
        ELSE IF ((SCIPS.EQ.TGB1).OR.(SCIPS.EQ.TGB2)) THEN
c
c         Connection tangent point - discontinuity boundary (SCNDIM=2)
c         ------------------------------------------------------------
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
        ELSE IF ((SCIPS.EQ.TCB1).OR.(SCIPS.EQ.TCB2)) THEN
c
c         Connection tg. pt - crossing - disc. boundary (SCNDIM=2)
c         --------------------------------------------------------
c
c         Initialize vector field J
          CALL SCSTPNT(SCNDIM,U,PAR,T,SCFJ(I))
          DO K=1,SCNDIM
            U(K+SCNDIM)=U(K)
          END DO
c
c         Initialize vector field I
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
        ELSE IF ((SCIPS.EQ.PEB1).OR.(SCIPS.EQ.PEB2)) THEN
c
c         Connection pseudo-equilibrium - discontinuity boundary
c         ------------------------------------------------------
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
c         Left boundary value
          DO K=1,SCNDIM
            U0(K)=PAR(SCIPAR0+4+K)
          END DO
c
c         Vector field I computation at left boundary value
          TMPIDIFF=0
          CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Vector field J computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Vector field I,J norm computation
          FIMODL=DSQRT(DSCPR1(FI,SCNDIM,1,FI,SCNDIM,1,SCNDIM))
          FJMODL=DSQRT(DSCPR1(FJ,SCNDIM,1,FJ,SCNDIM,1,SCNDIM))
c
c         State-parameter overspecification
          PAR(SCIPAR0+1)=FJMODL/(FIMODL+FJMODL)
          PAR(SCIPAR0+2)=FIMODL/(FIMODL+FJMODL)
c
        ELSE IF ((SCIPS.EQ.PCB1).OR.(SCIPS.EQ.PCB2)) THEN
c
c         Connection pseudo-eq. - crossing - discontinuity boundary
c         ---------------------------------------------------------
c
c         Initialize vector field J
          CALL SCSTPNT(SCNDIM,U,PAR,T,SCFJ(I))
          DO K=1,SCNDIM
            U(K+SCNDIM)=U(K)
          END DO
c
c         Initialize vector field I
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
c         Left boundary value
          DO K=1,SCNDIM
            U0(K)=PAR(SCIPAR0+4+K)
          END DO
c
c         Vector field I computation at left boundary value
          TMPIDIFF=0
          CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Vector field J computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Vector field I,J norm computation
          FIMODL=DSQRT(DSCPR1(FI,SCNDIM,1,FI,SCNDIM,1,SCNDIM))
          FJMODL=DSQRT(DSCPR1(FJ,SCNDIM,1,FJ,SCNDIM,1,SCNDIM))
c
c         State-parameter overspecification
          PAR(SCIPAR0+1)=FJMODL/(FIMODL+FJMODL)
          PAR(SCIPAR0+2)=FIMODL/(FIMODL+FJMODL)
c
        ELSE IF ((SCIPS.EQ.TCH1).OR.(SCIPS.EQ.TCH2)) THEN
c
c         Grazing
c         -------
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
        ELSE IF ((SCIPS.EQ.TGT1).OR.(SCIPS.EQ.TGT2)) THEN
c
c         Connection tg. pt - tg. pt of the same v.f. (SCNDIM=2)
c         ------------------------------------------------------
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
        ELSE IF ((SCIPS.EQ.TCT1).OR.(SCIPS.EQ.TCT2)) THEN
c
c         Connection tg. pt - cros. - tg. pt same v.f. (SCNDIM=2)
c         -------------------------------------------------------
c
c         Initialize vector field J
          CALL SCSTPNT(SCNDIM,U,PAR,T,SCFJ(I))
          DO K=1,SCNDIM
            U(K+SCNDIM)=U(K)
          END DO
c
c         Initialize vector field I
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
        ELSE IF ((SCIPS.EQ.TTD1).OR.(SCIPS.EQ.TTD2)) THEN
c
c         Connection tg. pt - tg. pt of different v.f. (SCNDIM=2)
c         -------------------------------------------------------
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
        ELSE IF ((SCIPS.EQ.TCD1).OR.(SCIPS.EQ.TCD2)) THEN
c
c         Connection tg. pt - cros. - tg. pt different v.f. (SCNDIM=2)
c         ------------------------------------------------------------
c
c         Initialize vector field J
          CALL SCSTPNT(SCNDIM,U,PAR,T,SCFJ(I))
          DO K=1,SCNDIM
            U(K+SCNDIM)=U(K)
          END DO
c
c         Initialize vector field I
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
        ELSE IF ((SCIPS.EQ.TGP1).OR.(SCIPS.EQ.TGP2)) THEN
c
c         Connection tangent point - pseudo-equilibrium
c         ---------------------------------------------
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
c         Right boundary value
          DO K=1,SCNDIM
            U1(K)=PAR(SCIPAR0+4+SCNDIM+K)
          END DO
c
c         Vector field I computation at right boundary value
          TMPIDIFF=0
          CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Vector field J computation at right boundary value
          CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Vector field I,J norm computation
          FIMODR=DSQRT(DSCPR1(FI,SCNDIM,1,FI,SCNDIM,1,SCNDIM))
          FJMODR=DSQRT(DSCPR1(FJ,SCNDIM,1,FJ,SCNDIM,1,SCNDIM))
c
c         State-parameter overspecification
          PAR(SCIPAR0+1)=FJMODR/(FIMODR+FJMODR)
          PAR(SCIPAR0+2)=FIMODR/(FIMODR+FJMODR)
c
        ELSE IF ((SCIPS.EQ.TCP1).OR.(SCIPS.EQ.TCP2)) THEN
c
c         Connection tangent point - crossing - pseudo-equilibrium
c         --------------------------------------------------------
c
c         Initialize vector field J
          CALL SCSTPNT(SCNDIM,U,PAR,T,SCFJ(I))
          DO K=1,SCNDIM
            U(K+SCNDIM)=U(K)
          END DO
c
c         Initialize vector field I
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
c         Right boundary value
          DO K=1,SCNDIM
            U1(K)=PAR(SCIPAR0+4+SCNDIM+K)
          END DO
c
c         Vector field I computation at right boundary value
          TMPIDIFF=0
          CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Vector field J computation at right boundary value
          CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Vector field I,J norm computation
          FIMODR=DSQRT(DSCPR1(FI,SCNDIM,1,FI,SCNDIM,1,SCNDIM))
          FJMODR=DSQRT(DSCPR1(FJ,SCNDIM,1,FJ,SCNDIM,1,SCNDIM))
c
c         State-parameter overspecification
          PAR(SCIPAR0+1)=FJMODR/(FIMODR+FJMODR)
          PAR(SCIPAR0+2)=FIMODR/(FIMODR+FJMODR)
c
        ELSE IF ((SCIPS.EQ.TGS1).OR.(SCIPS.EQ.TGS2)) THEN
c
c         Connection tangent point - saddle (SCNDIM=2)
c         --------------------------------------------
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
c         Right boundary value
          DO K=1,SCNDIM
            U1(K)=PAR(SCIPAR0+4+SCNDIM+K)
          END DO
c
c         Vector field I computation at the saddle
          CALL SCFUNC(SCNDIM,U1,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Eigenvalues and eigenvectors computation
          CALL SCEIG(DFIDX,SCNDIM,1,1,RR,RI,VRET)
c
c         Extra parameters initialization
          DO K=1,SCNDIM
            PAR(SCIPAR0+4+K)=U1(K)
          END DO
          CALL DARRCPY2(VRET,SCNDIM,SCNDIM,1,SCNDIM,
     +      PAR,SCIPSI0,1,SCIPAR0+5+SCNDIM,1,SCNDIM,1)
          PAR(SCIPAR0+5+2*SCNDIM)=RR(SCNDIM)
c
        ELSE IF ((SCIPS.EQ.TCS1).OR.(SCIPS.EQ.TCS2)) THEN
c
c         Connection tangent point - crossing - saddle (SCNDIM=2)
c         -------------------------------------------------------
c
c         Initialize vector field J
          CALL SCSTPNT(SCNDIM,U,PAR,T,SCFJ(I))
          DO K=1,SCNDIM
            U(K+SCNDIM)=U(K)
          END DO
c
c         Initialize vector field I
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
c         Right boundary value
          DO K=1,SCNDIM
            U1(K)=PAR(SCIPAR0+4+SCNDIM+K)
          END DO
c
c         Vector field J computation at the saddle
          CALL SCFUNC(SCNDIM,U1,PAR,SCIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Eigenvalues and eigenvectors computation
          CALL SCEIG(DFJDX,SCNDIM,1,1,RR,RI,VRET)
c
c         Extra parameters initialization
          DO K=1,SCNDIM
            PAR(SCIPAR0+4+K)=U1(K)
          END DO
          CALL DARRCPY2(VRET,SCNDIM,SCNDIM,1,SCNDIM,
     +      PAR,SCIPSI0,1,SCIPAR0+5+SCNDIM,1,SCNDIM,1)
          PAR(SCIPAR0+5+2*SCNDIM)=RR(SCNDIM)
c
        ELSE IF ((SCIPS.EQ.PET1).OR.(SCIPS.EQ.PET2)) THEN
c
c         Connection pseudo-eq. - tangent point of the same v.f.
c         ------------------------------------------------------
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
c         Left boundary value
          DO K=1,SCNDIM
            U0(K)=PAR(SCIPAR0+4+K)
          END DO
c
c         Vector field I computation at left boundary value
          TMPIDIFF=0
          CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Vector field J computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Vector field I,J norm computation
          FIMODL=DSQRT(DSCPR1(FI,SCNDIM,1,FI,SCNDIM,1,SCNDIM))
          FJMODL=DSQRT(DSCPR1(FJ,SCNDIM,1,FJ,SCNDIM,1,SCNDIM))
c
c         State-parameter overspecification
          PAR(SCIPAR0+1)=FJMODL/(FIMODL+FJMODL)
          PAR(SCIPAR0+2)=FIMODL/(FIMODL+FJMODL)
c
        ELSE IF ((SCIPS.EQ.PCT1).OR.(SCIPS.EQ.PCT2)) THEN
c
c         Connection pseudo-eq. - crossing - tg. pt of the same v.f.
c         ----------------------------------------------------------
c
c         Initialize vector field J
          CALL SCSTPNT(SCNDIM,U,PAR,T,SCFJ(I))
          DO K=1,SCNDIM
            U(K+SCNDIM)=U(K)
          END DO
c
c         Initialize vector field I
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
c         Left boundary value
          DO K=1,SCNDIM
            U0(K)=PAR(SCIPAR0+4+K)
          END DO
c
c         Vector field I computation at left boundary value
          TMPIDIFF=0
          CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Vector field J computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Vector field I,J norm computation
          FIMODL=DSQRT(DSCPR1(FI,SCNDIM,1,FI,SCNDIM,1,SCNDIM))
          FJMODL=DSQRT(DSCPR1(FJ,SCNDIM,1,FJ,SCNDIM,1,SCNDIM))
c
c         State-parameter overspecification
          PAR(SCIPAR0+1)=FJMODL/(FIMODL+FJMODL)
          PAR(SCIPAR0+2)=FIMODL/(FIMODL+FJMODL)
c
        ELSE IF ((SCIPS.EQ.PTD1).OR.(SCIPS.EQ.PTD2)) THEN
c
c         Connection pseudo-eq. - tangent point of different v.f.
c         -------------------------------------------------------
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
c         Left boundary value
          DO K=1,SCNDIM
            U0(K)=PAR(SCIPAR0+4+K)
          END DO
c
c         Vector field I computation at left boundary value
          TMPIDIFF=0
          CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Vector field J computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Vector field I,J norm computation
          FIMODL=DSQRT(DSCPR1(FI,SCNDIM,1,FI,SCNDIM,1,SCNDIM))
          FJMODL=DSQRT(DSCPR1(FJ,SCNDIM,1,FJ,SCNDIM,1,SCNDIM))
c
c         State-parameter overspecification
          PAR(SCIPAR0+1)=FJMODL/(FIMODL+FJMODL)
          PAR(SCIPAR0+2)=FIMODL/(FIMODL+FJMODL)
c
        ELSE IF ((SCIPS.EQ.PCD1).OR.(SCIPS.EQ.PCD2)) THEN
c
c         Connection pseudo-eq. - crossing - tg. pt of different v.f.
c         -----------------------------------------------------------
c
c         Initialize vector field J
          CALL SCSTPNT(SCNDIM,U,PAR,T,SCFJ(I))
          DO K=1,SCNDIM
            U(K+SCNDIM)=U(K)
          END DO
c
c         Initialize vector field I
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
c         Left boundary value
          DO K=1,SCNDIM
            U0(K)=PAR(SCIPAR0+4+K)
          END DO
c
c         Vector field I computation at left boundary value
          TMPIDIFF=0
          CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Vector field J computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Vector field I,J norm computation
          FIMODL=DSQRT(DSCPR1(FI,SCNDIM,1,FI,SCNDIM,1,SCNDIM))
          FJMODL=DSQRT(DSCPR1(FJ,SCNDIM,1,FJ,SCNDIM,1,SCNDIM))
c
c         State-parameter overspecification
          PAR(SCIPAR0+1)=FJMODL/(FIMODL+FJMODL)
          PAR(SCIPAR0+2)=FIMODL/(FIMODL+FJMODL)
c
        ELSE IF ((SCIPS.EQ.PEP1).OR.(SCIPS.EQ.PEP2)) THEN
c
c         Connection pseudo-eq. - pseudo-eq. (SCNDIM=2)
c         ---------------------------------------------
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
c         Left and right boundary values
          DO K=1,SCNDIM
            U0(K)=PAR(SCIPAR0+4+K)
            U1(K)=PAR(SCIPAR0+4+SCNDIM+K)
          END DO
c
c         Vector field I computation at left boundary value
          TMPIDIFF=0
          CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Vector field J computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Vector field I,J norm computation
          FIMODL=DSQRT(DSCPR1(FI,SCNDIM,1,FI,SCNDIM,1,SCNDIM))
          FJMODL=DSQRT(DSCPR1(FJ,SCNDIM,1,FJ,SCNDIM,1,SCNDIM))
c
c         Vector field I computation at right boundary value
          CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Vector field J computation at right boundary value
          CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Vector field I,J norm computation
          FIMODR=DSQRT(DSCPR1(FI,SCNDIM,1,FI,SCNDIM,1,SCNDIM))
          FJMODR=DSQRT(DSCPR1(FJ,SCNDIM,1,FJ,SCNDIM,1,SCNDIM))
c
c         State-parameter overspecification
          PAR(SCIPAR0+1)=FJMODL/(FIMODL+FJMODL)
          PAR(SCIPAR0+2)=FIMODL/(FIMODL+FJMODL)
          PAR(SCIPAR0+3)=FJMODR/(FIMODR+FJMODR)
          PAR(SCIPAR0+4)=FIMODR/(FIMODR+FJMODR)
c
        ELSE IF ((SCIPS.EQ.PCP1).OR.(SCIPS.EQ.PCP2)) THEN
c
c         Connection pseudo-eq. - crossing - pseudo-eq. (SCNDIM=2)
c         --------------------------------------------------------
c
c         Initialize vector field J
          CALL SCSTPNT(SCNDIM,U,PAR,T,SCFJ(I))
          DO K=1,SCNDIM
            U(K+SCNDIM)=U(K)
          END DO
c
c         Initialize vector field I
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
c         Left and right boundary values
          DO K=1,SCNDIM
            U0(K)=PAR(SCIPAR0+4+K)
            U1(K)=PAR(SCIPAR0+4+SCNDIM+K)
          END DO
c
c         Vector field I computation at left boundary value
          TMPIDIFF=0
          CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Vector field J computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Vector field I,J norm computation
          FIMODL=DSQRT(DSCPR1(FI,SCNDIM,1,FI,SCNDIM,1,SCNDIM))
          FJMODL=DSQRT(DSCPR1(FJ,SCNDIM,1,FJ,SCNDIM,1,SCNDIM))
c
c         Vector field I computation at right boundary value
          CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Vector field J computation at right boundary value
          CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Vector field I,J norm computation
          FIMODR=DSQRT(DSCPR1(FI,SCNDIM,1,FI,SCNDIM,1,SCNDIM))
          FJMODR=DSQRT(DSCPR1(FJ,SCNDIM,1,FJ,SCNDIM,1,SCNDIM))
c
c         State-parameter overspecification
          PAR(SCIPAR0+1)=FJMODL/(FIMODL+FJMODL)
          PAR(SCIPAR0+2)=FIMODL/(FIMODL+FJMODL)
          PAR(SCIPAR0+3)=FJMODR/(FIMODR+FJMODR)
          PAR(SCIPAR0+4)=FIMODR/(FIMODR+FJMODR)
c
        ELSE IF ((SCIPS.EQ.PES1).OR.(SCIPS.EQ.PES2)) THEN
c
c         Connection pseudo-equilibrium - saddle
c         --------------------------------------
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
c         Left and right boundary values
          DO K=1,SCNDIM
            U0(K)=PAR(SCIPAR0+4+K)
            U1(K)=PAR(SCIPAR0+4+SCNDIM+K)
          END DO
c
c         Vector field I computation at left boundary value
          TMPIDIFF=0
          CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Vector field J computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Vector field I,J norm computation
          FIMODL=DSQRT(DSCPR1(FI,SCNDIM,1,FI,SCNDIM,1,SCNDIM))
          FJMODL=DSQRT(DSCPR1(FJ,SCNDIM,1,FJ,SCNDIM,1,SCNDIM))
c
c         State-parameter overspecification
          PAR(SCIPAR0+1)=FJMODL/(FIMODL+FJMODL)
          PAR(SCIPAR0+2)=FIMODL/(FIMODL+FJMODL)
c
c         Vector field I computation at the saddle
          CALL SCFUNC(SCNDIM,U1,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Eigenvalues and eigenvectors computation
          CALL SCEIG(DFIDX,SCNDIM,1,1,RR,RI,VRET)
c
c         Extra parameters initialization
          DO K=1,SCNDIM
            PAR(SCIPAR0+4+K)=U1(K)
          END DO
          CALL DARRCPY2(VRET,SCNDIM,SCNDIM,1,SCNDIM,
     +      PAR,SCIPSI0,1,SCIPAR0+5+SCNDIM,1,SCNDIM,1)
          PAR(SCIPAR0+5+2*SCNDIM)=RR(SCNDIM)
c
        ELSE IF ((SCIPS.EQ.PCS1).OR.(SCIPS.EQ.PCS2)) THEN
c
c         Connection pseudo-equilibrium - crossing - saddle
c         -------------------------------------------------
c
c         Initialize vector field J
          CALL SCSTPNT(SCNDIM,U,PAR,T,SCFJ(I))
          DO K=1,SCNDIM
            U(K+SCNDIM)=U(K)
          END DO
c
c         Initialize vector field I
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
c         Left and right boundary values
          DO K=1,SCNDIM
            U0(K)=PAR(SCIPAR0+4+K)
            U1(K)=PAR(SCIPAR0+4+SCNDIM+K)
          END DO
c
c         Vector field I computation at left boundary value
          TMPIDIFF=0
          CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Vector field J computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Vector field I,J norm computation
          FIMODL=DSQRT(DSCPR1(FI,SCNDIM,1,FI,SCNDIM,1,SCNDIM))
          FJMODL=DSQRT(DSCPR1(FJ,SCNDIM,1,FJ,SCNDIM,1,SCNDIM))
c
c         State-parameter overspecification
          PAR(SCIPAR0+1)=FJMODL/(FIMODL+FJMODL)
          PAR(SCIPAR0+2)=FIMODL/(FIMODL+FJMODL)
c
c         Vector field J computation at the saddle
          CALL SCFUNC(SCNDIM,U1,PAR,SCIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Eigenvalues and eigenvectors computation
          CALL SCEIG(DFJDX,SCNDIM,1,1,RR,RI,VRET)
c
c         Extra parameters initialization
          DO K=1,SCNDIM
            PAR(SCIPAR0+4+K)=U1(K)
          END DO
          CALL DARRCPY2(VRET,SCNDIM,SCNDIM,1,SCNDIM,
     +      PAR,SCIPSI0,1,SCIPAR0+5+SCNDIM,1,SCNDIM,1)
          PAR(SCIPAR0+5+2*SCNDIM)=RR(SCNDIM)
c
        ELSE IF ((SCIPS.EQ.SCY1).OR.(SCIPS.EQ.SCY2)) THEN
c
c         Sliding cycle
c         -------------
c
c         Initialize sliding vector field
          CALL SCSTPNT(SCNDIM,U,PAR,T,3)
          DO K=1,SCNDIM
            U(K+SCNDIM)=U(K)
          END DO
c
c         Initialize vector field I
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
        ELSE IF ((SCIPS.EQ.SCC1).OR.(SCIPS.EQ.SCC2)) THEN
c
c         Sliding-crossing cycle
c         ----------------------
c
c         Initialize sliding vector field
          CALL SCSTPNT(SCNDIM,U,PAR,T,3)
          DO K=1,SCNDIM
            U(K+2*SCNDIM)=U(K)
          END DO
c
c         Initialize vector field J
          CALL SCSTPNT(SCNDIM,U,PAR,T,SCFJ(I))
          DO K=1,SCNDIM
            U(K+SCNDIM)=U(K)
          END DO
c
c         Initialize vector field I
          CALL SCSTPNT(SCNDIM,U,PAR,T,I)
c
        ELSE IF (SCIPS.EQ.CCY) THEN
c
c         Crossing cycle
c         --------------
c
c         Initialize vector field 2
          CALL SCSTPNT(SCNDIM,U,PAR,T,2)
          DO K=1,SCNDIM
            U(K+SCNDIM)=U(K)
          END DO
c
c         Initialize vector field 1
          CALL SCSTPNT(SCNDIM,U,PAR,T,1)
c
        ELSE
          WRITE(*,510)SCIPS
          STOP
        END IF
c
c       Debug
        IF (DEBUG) THEN
          IF (DBGLEV.GE.DBGL2) THEN
            PRINT *,'exit from STPNT'
          END IF
        END IF
c
      RETURN
      END
c
c
c     ---------------------------
      SUBROUTINE PVLS(NDIM,U,PAR)
c     ---------------------------
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'slidecont.h'
        INCLUDE 'debug.h'
c
        DIMENSION U(NDIM),PAR(*)
c
c       Common block reference
c       ----------------------
        LOGICAL DEFINED
        INTEGER SCNDIM,SCIPS,SCIRS,SCISTART
        INTEGER SCNICP,SCICP(SCNPARX)
        INTEGER SCNPSI,SCIPSI(SCNPSIX)
        INTEGER SCNFIXED,SCIFIXED(SCNPSIX)
        INTEGER SCIDIFF,SCNUZR
        LOGICAL SCFPSI(SCNPSIX),SCFFIXED(SCNPSIX)
c
        COMMON /PROB/ DEFINED
        COMMON /PROB/ SCNDIM,SCIPS,SCIRS,SCISTART
        COMMON /PROB/ SCNICP,SCICP
        COMMON /PROB/ SCNPSI,SCIPSI
        COMMON /PROB/ SCNFIXED,SCIFIXED
        COMMON /PROB/ SCIDIFF,SCNUZR
        COMMON /PROB/ SCFPSI,SCFFIXED
c
c       Functions references
c       --------------------
        INTEGER SCFJ
        INTEGER SCIHOPF
        INTEGER SCIFOLD
        INTEGER SCNSTAB
c
c       Vector fields
c       -------------
        DOUBLE PRECISION FI(SCNDIMX)
        DOUBLE PRECISION DFIDX(SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DFIDP(SCNDIMX,SCNPARX)
        DOUBLE PRECISION DFIDXDX(SCNDIMX,SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DFIDXDP(SCNDIMX,SCNDIMX,SCNPARX)
c
        DOUBLE PRECISION FJ(SCNDIMX)
        DOUBLE PRECISION DFJDX(SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DFJDP(SCNDIMX,SCNPARX)
        DOUBLE PRECISION DFJDXDX(SCNDIMX,SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DFJDXDP(SCNDIMX,SCNDIMX,SCNPARX)
c
c       Zero initialization
        DATA FI/SCNDIMX*0.0/
        DATA DFIDX/SCNDIMX2*0.0/
        DATA DFIDP/SCNDIMXNPARX*0.0/
        DATA DFIDXDX/SCNDIMX3*0.0/
        DATA DFIDXDP/SCNDIMX2NPARX*0.0/
c
        DATA FJ/SCNDIMX*0.0/
        DATA DFJDX/SCNDIMX2*0.0/
        DATA DFJDP/SCNDIMXNPARX*0.0/
        DATA DFJDXDX/SCNDIMX3*0.0/
        DATA DFJDXDP/SCNDIMX2NPARX*0.0/
c
c       Discontinuity boundary
c       ----------------------
        DOUBLE PRECISION H
        DOUBLE PRECISION DHDX(SCNDIMX)
        DOUBLE PRECISION DHDP(SCNPARX)
        DOUBLE PRECISION DHDXDX(SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DHDXDP(SCNDIMX,SCNPARX)
c
c       Zero initialization
        DATA H/0.0/
        DATA DHDX/SCNDIMX*0.0/
        DATA DHDP/SCNPARX*0.0/
        DATA DHDXDX/SCNDIMX2*0.0/
        DATA DHDXDP/SCNDIMXNPARX*0.0/
c
c       Discontinuity boundary tangent vector (SCNDIM=2)
c       ------------------------------------------------
        DOUBLE PRECISION DHDXR(SCNDIMX)
c
c       Zero initialization
        DATA DHDXR/SCNDIMX*0.0/
c
c       Other local variables
c       ---------------------
        DOUBLE PRECISION U0(SCNDIMX),U1(SCNDIMX),U2(SCNDIMX)
        DOUBLE PRECISION RR(SCNDIMX),RI(SCNDIMX),VRET(SCNDIMX,SCNDIMX)
        COMPLEX EV
        INTEGER TMPIDIFF
        INTEGER IOVRPAR
c
c       Debug
        IF (DEBUG) THEN
          IF (DBGLEV.GE.DBGL2) THEN
            PRINT *,'enter in PVLS'
            PRINT *,'PAR='
            CALL DARRPRN1(PAR,12,1,12)
          END IF
        END IF
c
c       Common block initialization
c       ---------------------------
        IF (.NOT.DEFINED) THEN
          CALL SCPROB
        END IF
c
c       Vector field selection
c       ----------------------
        IF (SCIPS.LT.200) THEN
          I=1
        ELSE IF (SCIPS.LT.300) THEN
          I=2
        ELSE
          I=3
        END IF
c
c       Problem type selection
c       ----------------------
        IF (SCIPS.LT.100) THEN
c
c         AUTO97 problem type
c         -------------------
c         Dummy step
c
        ELSE IF ((SCIPS.EQ.T3D1).OR.(SCIPS.EQ.T3D2)) THEN
c
c         Tangent point (SCNDIM=3)
c         ------------------------
c
c         Test function 1: Tangent point of vector field J
c         ------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Vector fields J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+1)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
        ELSE IF ((SCIPS.EQ.TAN1).OR.(SCIPS.EQ.TAN2)) THEN
c
c         Tangent point (SCNDIM=2)
c         ------------------------
c
c         Test function 1: Boundary equilibrium of vector field I
c         -------------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Vector fields I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+1)=DSCPR1(FI,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 2: Tangent point of vector field J
c         ------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Vector fields J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+2)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
        ELSE IF ((SCIPS.EQ.EQL1).OR.(SCIPS.EQ.EQL2)) THEN
c
c         Equilibrium
c         -----------
c
c         Test function 1: Boundary equilibrium of vector field I
c         -------------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Discontinuity boundary computation
            TMPIDIFF=0
            CALL SCBOUND(SCNDIM,U,PAR,TMPIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+1)=H
          END IF
c
        ELSE IF ((SCIPS.EQ.BEQ1).OR.(SCIPS.EQ.BEQ2)) THEN
c
c         Boundary equilibrium
c         --------------------
c
c         State-parameter overspecification
c         ---------------------------------
          IOVRPAR=2
          PAR(SCICP(IOVRPAR))=U(SCNDIM+1)         
c
c         Vector fields computation
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Eigenvalues computation
          CALL SCEIG(DFIDX,SCNDIM,1,0,RR,RI,VRET)
c
c         Write eigenvalues in fort.9 (d.xxx)
          IFOLD=SCIFOLD(RR,RI,SCNDIM)
          TFFOLD=0.0D0
          IF (IFOLD.GT.0) TFFOLD=RR(IFOLD)
          IHOPF=SCIHOPF(RR,RI,SCNDIM)
          TFHOPF=0.0D0
          IF (IHOPF.GT.0) TFHOPF=RR(IHOPF)
          NSTAB=SCNSTAB(RR,RI,SCNDIM)
          WRITE(9,700)TFFOLD
          WRITE(9,710)TFHOPF
          WRITE(9,720)NSTAB
          DO K=1,SCNDIM
            EV=CMPLX(RR(K),RI(K))
            WRITE(9,730)K,EV
          END DO
c
c         Test function 1: Hopf
c         ---------------------
          IF (SCFPSI(1)) THEN
c
c           Test function
            PAR(SCIPSI0+1)=TFHOPF
          END IF
c
c         Test function 2: Fold/BR
c         ------------------------
          IF (SCFPSI(2)) THEN
c
c           Test function
            PAR(SCIPSI0+2)=TFFOLD
          END IF
c
c         Test function 3: Tangent point of vector field J
c         ------------------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Vector fields J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+3)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
        ELSE IF (SCIPS.EQ.BND) THEN
c
c         Discontinuity boundary (SCNDIM=2)
c         ---------------------------------
c
c         Test function 1: Tangent point of vector field 1
c         ------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Vector fields 1 computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,1)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+1)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 2: Tangent point of vector field 2
c         ------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Vector fields 2 computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,2)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+2)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 3: Pseudo-equilibrium
c         -----------------------------------
c
          IF (SCFPSI(3)) THEN
c
c           Vector fields 1 computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,1)
c
c           Vector fields 2 computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,2)

c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Normal component of vector fields
            FIN=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
            FJN=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
c
c           Test function
            PAR(SCIPSI0+3)=DHDXR(1)*(FJN*FI(1)-FIN*FJ(1))+
     +        DHDXR(2)*(FJN*FI(2)-FIN*FJ(2))
          END IF
c
        ELSE IF (SCIPS.EQ.PEQ) THEN
c
c         Pseudo-equilibrium
c         ------------------
c
c         Test function 1: Boundary equilibrium of vector field 1
c         -------------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Test function
            PAR(SCIPSI0+1)=U(SCNDIM+2)
          END IF
c
c         Test function 2: Boundary equilibrium of vector field 2
c         -------------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Test function
            PAR(SCIPSI0+2)=U(SCNDIM+1)
          END IF
c
c         Test function 3: Singular sliding point
c         ---------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Vector fields 1 computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,1)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+3)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
        ELSE IF (SCIPS.EQ.CTG) THEN
c
c         Collision of tangent points (SCNDIM=2)
c         --------------------------------------
c
c         Test function 1: Boundary equilibrium of vector field 1
c         -------------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Vector fields 1 computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,1)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+1)=DSCPR1(FI,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 2: Boundary equilibrium of vector field 2
c         -------------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Vector fields 2 computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,2)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+2)=DSCPR1(FJ,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
        ELSE IF ((SCIPS.EQ.CYC1).OR.(SCIPS.EQ.CYC2)) THEN
c
c         Cycle
c         -----
c
c         Period overspecification
c         ------------------------
          PAR(SCIPARTI)=PAR(SCIPARTC)
c
c         Test function 1: Grazing
c         ------------------------
          IF (SCFPSI(1)) THEN
c
c           Test function
            PAR(SCIPSI0+1)=SCGRAZ(SCNDIM,PAR,U,I)
          END IF
c
        ELSE IF ((SCIPS.EQ.TGB1).OR.(SCIPS.EQ.TGB2)) THEN
c
c         Connection tangent point - discontinuity boundary (SCNDIM=2)
c         ------------------------------------------------------------
c
c         Period overspecification
c         ------------------------
          PAR(SCIPARTC)=PAR(SCIPARTI)
c
c         Left and right boundary values
c         ------------------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
            U1(K)=SCGETP('BV1',0,K,U)
          END DO
c
c         Test function 1: Boundary equilibrium of v.f. I at l.b.v.
c         ---------------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+1)=DSCPR1(FI,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 2: Tangent point of v.f. J at l.b.v.
c         --------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+2)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 3: Tangent point of v.f. I at r.b.v.
c         --------------------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+3)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 4: Tangent point of v.f. J at r.b.v.
c         --------------------------------------------------
          IF (SCFPSI(4)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+4)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 5: Pseudo-equilibrium at r.b.v.
c         ---------------------------------------------
          IF (SCFPSI(5)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Normal component of vector fields
            FIN=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
            FJN=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
c
c           Test function
            PAR(SCIPSI0+5)=DHDXR(1)*(FJN*FI(1)-FIN*FJ(1))+
     +        DHDXR(2)*(FJN*FI(2)-FIN*FJ(2))
          END IF
c
        ELSE IF ((SCIPS.EQ.TCB1).OR.(SCIPS.EQ.TCB2)) THEN
c
c         Connection tg. pt - crossing - disc. boundary (SCNDIM=2)
c         --------------------------------------------------------
c
c         Left and right boundary values
c         ------------------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
            U1(K)=SCGETP('BV1',0,K,U)
            U2(K)=SCGETP('BV1',0,SCNDIM+K,U)
          END DO
c
c         Test function 1: Boundary eq. of v.f. I at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+1)=DSCPR1(FI,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 2: Tang. point of v.f. J at l.b.v.
c         ------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+2)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 3: Tang. point of v.f. I at r.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+3)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 4: Tang. point of v.f. J at r.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(4)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+4)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 5: Tang. point of v.f. I at r.b.v.
c         ------------------------------------------------
          IF (SCFPSI(5)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U2,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U2,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+5)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 6: Tang. point of v.f. J at r.b.v.
c         ------------------------------------------------
          IF (SCFPSI(6)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U2,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U2,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+6)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 7: Pseudo-equilibrium at r.b.v.
c         ---------------------------------------------
          IF (SCFPSI(7)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U2,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U2,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U2,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Normal component of vector fields
            FIN=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
            FJN=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
c
c           Test function
            PAR(SCIPSI0+7)=DHDXR(1)*(FJN*FI(1)-FIN*FJ(1))+
     +        DHDXR(2)*(FJN*FI(2)-FIN*FJ(2))
          END IF
c
        ELSE IF ((SCIPS.EQ.PEB1).OR.(SCIPS.EQ.PEB2)) THEN
c
c         Connection pseudo-equilibrium - discontinuity boundary
c         ------------------------------------------------------
c
c         Period overspecification
c         ------------------------
          PAR(SCIPARTC)=PAR(SCIPARTI)
c
c         Left and right boundary values
c         ------------------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
            U1(K)=SCGETP('BV1',0,K,U)
          END DO
c
c         Test function 1: Boundary eq. of v.f. I at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Test function
            PAR(SCIPSI0+1)=PAR(SCIPAR0+2)
          END IF
c
c         Test function 2: Boundary eq. of v.f. J at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Test function
            PAR(SCIPSI0+2)=PAR(SCIPAR0+1)
          END IF
c
c         Test function 3: Singular sliding point at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+3)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 4: Tangent point of v.f. I at r.b.v.
c         --------------------------------------------------
          IF (SCFPSI(4)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+4)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 5: Tangent point of v.f. J at r.b.v.
c         --------------------------------------------------
          IF (SCFPSI(5)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+5)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 6: Pseudo-equilibrium at r.b.v. (SCNDIM=2)
c         --------------------------------------------------------
          IF (SCFPSI(6)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Normal component of vector fields
            FIN=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
            FJN=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
c
c           Test function
            PAR(SCIPSI0+6)=DHDXR(1)*(FJN*FI(1)-FIN*FJ(1))+
     +        DHDXR(2)*(FJN*FI(2)-FIN*FJ(2))
          END IF
c
        ELSE IF ((SCIPS.EQ.PCB1).OR.(SCIPS.EQ.PCB2)) THEN
c
c         Connection pseudo-eq. - crossing - discontinuity boundary
c         ---------------------------------------------------------
c
c         Left and right boundary values
c         ------------------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
            U1(K)=SCGETP('BV1',0,K,U)
            U2(K)=SCGETP('BV1',0,SCNDIM+K,U)
          END DO
c
c         Test function 1: Boundary eq. of v.f. I at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Test function
            PAR(SCIPSI0+1)=PAR(SCIPAR0+2)
          END IF
c
c         Test function 2: Boundary eq. of v.f. J at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Test function
            PAR(SCIPSI0+2)=PAR(SCIPAR0+1)
          END IF
c
c         Test function 3: Singular sliding point at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+3)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 4: Tang. point of v.f. I at r.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(4)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+4)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 5: Tang. point of v.f. J at r.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(5)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+5)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 6: Tang. point of v.f. I at r.b.v.
c         ------------------------------------------------
          IF (SCFPSI(6)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U2,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U2,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+6)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 7: Tang. point of v.f. J at r.b.v.
c         ------------------------------------------------
          IF (SCFPSI(7)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U2,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U2,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+7)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 8: Pseudo-eq. at r.b.v. (SCNDIM=2)
c         ------------------------------------------------
          IF (SCFPSI(8)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U2,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U2,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U2,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Normal component of vector fields
            FIN=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
            FJN=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
c
c           Test function
            PAR(SCIPSI0+8)=DHDXR(1)*(FJN*FI(1)-FIN*FJ(1))+
     +        DHDXR(2)*(FJN*FI(2)-FIN*FJ(2))
          END IF
c
        ELSE IF ((SCIPS.EQ.TCH1).OR.(SCIPS.EQ.TCH2)) THEN
c
c         Grazing
c         -------
c
c         Period overspecification
c         ------------------------
          PAR(SCIPARTC)=PAR(SCIPARTI)
c
c         Left boundary value
c         -------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
          END DO
c
c         Test function 1: Boundary eq. of v.f. I at l.b.v. (SCNDIM=2)
c         ------------------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+1)=DSCPR1(FI,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 2: Tangent point of v.f. J at l.b.v.
c         --------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+2)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
        ELSE IF ((SCIPS.EQ.TGT1).OR.(SCIPS.EQ.TGT2)) THEN
c
c         Connection tg. pt - tg. pt of the same v.f. (SCNDIM=2)
c         ------------------------------------------------------
c
c         Period overspecification
c         ------------------------
          PAR(SCIPARTC)=PAR(SCIPARTI)
c
c         Left and right boundary values
c         ------------------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
            U1(K)=SCGETP('BV1',0,K,U)
          END DO
c
c         Test function 1: Boundary equilibrium of v.f. I at l.b.v.
c         ---------------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+1)=DSCPR1(FI,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 2: Tangent point of v.f. J at l.b.v.
c         --------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+2)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 3: Boundary equilibrium of v.f. I at r.b.v.
c         ---------------------------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+3)=DSCPR1(FI,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 4: Tangent point of v.f. J at r.b.v.
c         --------------------------------------------------
          IF (SCFPSI(4)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+4)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
        ELSE IF ((SCIPS.EQ.TCT1).OR.(SCIPS.EQ.TCT2)) THEN
c
c         Connection tg. pt - cros. - tg. pt same v.f. (SCNDIM=2)
c         -------------------------------------------------------
c
c         Left and right boundary values
c         ------------------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
            U1(K)=SCGETP('BV1',0,K,U)
            U2(K)=SCGETP('BV1',0,SCNDIM+K,U)
          END DO
c
c         Test function 1: Boundary eq. of v.f. I at l.b.v. of v.f. I
c         -----------------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+1)=DSCPR1(FI,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 2: Tang. point of v.f. J at l.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+2)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 3: Tang. point of v.f. I at r.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+3)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 4: Tang. point of v.f. J at r.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(4)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+4)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 5: Boundary eq. of v.f. I at r.b.v. of v.f. J
c         -----------------------------------------------------------
          IF (SCFPSI(5)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U2,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U2,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+5)=DSCPR1(FI,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 6: Tang. point of v.f. J at r.b.v. of v.f. J
c         ----------------------------------------------------------
          IF (SCFPSI(6)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U2,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U2,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+6)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
        ELSE IF ((SCIPS.EQ.TTD1).OR.(SCIPS.EQ.TTD2)) THEN
c
c         Connection tg. pt - tg. pt of different v.f. (SCNDIM=2)
c         -------------------------------------------------------
c
c         Period overspecification
c         ------------------------
          PAR(SCIPARTC)=PAR(SCIPARTI)
c
c         Left and right boundary values
c         ------------------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
            U1(K)=SCGETP('BV1',0,K,U)
          END DO
c
c         Test function 1: Boundary equilibrium of v.f. I at l.b.v.
c         ---------------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+1)=DSCPR1(FI,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 2: Tangent point of v.f. J at l.b.v.
c         --------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+2)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 3: Boundary equilibrium of v.f. J at r.b.v.
c         ---------------------------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+3)=DSCPR1(FJ,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 4: Tangent point of v.f. I at r.b.v.
c         --------------------------------------------------
          IF (SCFPSI(4)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+4)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
        ELSE IF ((SCIPS.EQ.TCD1).OR.(SCIPS.EQ.TCD2)) THEN
c
c         Connection tg. pt - cros. - tg. pt different v.f. (SCNDIM=2)
c         ------------------------------------------------------------
c
c         Left and right boundary values
c         ------------------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
            U1(K)=SCGETP('BV1',0,K,U)
            U2(K)=SCGETP('BV1',0,SCNDIM+K,U)
          END DO
c
c         Test function 1: Boundary eq. of v.f. I at l.b.v. of v.f. I
c         -----------------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+1)=DSCPR1(FI,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 2: Tang. point of v.f. J at l.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+2)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 3: Tang. point of v.f. I at r.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+3)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 4: Tang. point of v.f. J at r.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(4)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+4)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 5: Boundary eq. of v.f. J at r.b.v. of v.f. J
c         -----------------------------------------------------------
          IF (SCFPSI(5)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U2,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U2,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+5)=DSCPR1(FJ,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 6: Tang. point of v.f. I at r.b.v. of v.f. J
c         ----------------------------------------------------------
          IF (SCFPSI(6)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U2,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U2,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+6)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
        ELSE IF ((SCIPS.EQ.TGP1).OR.(SCIPS.EQ.TGP2)) THEN
c
c         Connection tangent point - pseudo-equilibrium
c         ---------------------------------------------
c
c         Period overspecification
c         ------------------------
          PAR(SCIPARTC)=PAR(SCIPARTI)
c
c         Left and right boundary values
c         ------------------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
            U1(K)=SCGETP('BV1',0,K,U)
          END DO
c
c         Test function 1: Boundary eq. of v.f. I at l.b.v. (SCNDIM=2)
c         ------------------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+1)=DSCPR1(FI,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 2: Tangent point of v.f. J at l.b.v.
c         --------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+2)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 3: Boundary eq. of v.f. I at r.b.v.
c         -------------------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Test function
            PAR(SCIPSI0+3)=PAR(SCIPAR0+2)
          END IF
c
c         Test function 4: Boundary eq. of v.f. J at r.b.v.
c         -------------------------------------------------
          IF (SCFPSI(4)) THEN
c
c           Test function
            PAR(SCIPSI0+4)=PAR(SCIPAR0+1)
          END IF
c
c         Test function 5: Singular sliding point at r.b.v.
c         -------------------------------------------------
          IF (SCFPSI(5)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+5)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
        ELSE IF ((SCIPS.EQ.TCP1).OR.(SCIPS.EQ.TCP2)) THEN
c
c         Connection tangent point - crossing - pseudo-equilibrium
c         --------------------------------------------------------
c
c         Left and right boundary values
c         ------------------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
            U1(K)=SCGETP('BV1',0,K,U)
            U2(K)=SCGETP('BV1',0,SCNDIM+K,U)
          END DO
c
c         Test function 1: Boundary eq. of v.f. I at l.b.v. (SCNDIM=2)
c         ------------------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+1)=DSCPR1(FI,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 2: Tang. point of v.f. J at l.b.v.
c         ------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+2)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 3: Tang. point of v.f. I at r.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+3)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 4: Tang. point of v.f. J at r.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(4)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+4)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 5: Boundary eq. of v.f. I at r.b.v.
c         -------------------------------------------------
          IF (SCFPSI(5)) THEN
c
c           Test function
            PAR(SCIPSI0+5)=PAR(SCIPAR0+2)
          END IF
c
c         Test function 6: Boundary eq. of v.f. J at r.b.v.
c         -------------------------------------------------
          IF (SCFPSI(6)) THEN
c
c           Test function
            PAR(SCIPSI0+6)=PAR(SCIPAR0+1)
          END IF
c
c         Test function 7: Singular sliding point at r.b.v.
c         -------------------------------------------------
          IF (SCFPSI(7)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U2,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U2,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+7)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
        ELSE IF ((SCIPS.EQ.TGS1).OR.(SCIPS.EQ.TGS2)) THEN
c
c         Connection tangent point - saddle (SCNDIM=2)
c         --------------------------------------------
c
c         Left and right boundary values
c         ------------------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
            U1(K)=PAR(SCIPAR0+4+K)
          END DO
c
c         Vector fields computation at saddle
          CALL SCFUNC(SCNDIM,U1,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Eigenvalues computation
          CALL SCEIG(DFIDX,SCNDIM,1,0,RR,RI,VRET)
c
c         Write eigenvalues in fort.9 (d.xxx)
          IFOLD=SCIFOLD(RR,RI,SCNDIM)
          TFFOLD=0.0D0
          IF (IFOLD.GT.0) TFFOLD=RR(IFOLD)
          IHOPF=SCIHOPF(RR,RI,SCNDIM)
          TFHOPF=0.0D0
          IF (IHOPF.GT.0) TFHOPF=RR(IHOPF)
          NSTAB=SCNSTAB(RR,RI,SCNDIM)
          WRITE(9,700)TFFOLD
          WRITE(9,710)TFHOPF
          WRITE(9,720)NSTAB
          DO K=1,SCNDIM
            EV=CMPLX(RR(K),RI(K))
            WRITE(9,730)K,EV
          END DO
c
c         Test function 1: Boundary equilibrium of v.f. I at l.b.v.
c         ---------------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+1)=DSCPR1(FI,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 2: Tangent point of v.f. J at l.b.v.
c         --------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+2)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 3: Boundary saddle
c         --------------------------------
          IF (SCFPSI(3)) THEN
c
c           Discontinuity boundary computation
            TMPIDIFF=0
            CALL SCBOUND(SCNDIM,U1,PAR,TMPIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+3)=H
          END IF
c
c         Test function 4: Saddle Fold/BR
c         -------------------------------
          IF (SCFPSI(4)) THEN
c
c           Test function
            PAR(SCIPSI0+4)=TFFOLD
          END IF
c
        ELSE IF ((SCIPS.EQ.TCS1).OR.(SCIPS.EQ.TCS2)) THEN
c
c         Connection tangent point - crossing - saddle (SCNDIM=2)
c         -------------------------------------------------------
c
c         Left and right boundary values
c         ------------------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
            U1(K)=SCGETP('BV1',0,K,U)
            U2(K)=PAR(SCIPAR0+4+K)
          END DO
c
c         Vector fields computation at saddle
          CALL SCFUNC(SCNDIM,U2,PAR,SCIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Eigenvalues computation
          CALL SCEIG(DFJDX,SCNDIM,1,0,RR,RI,VRET)
c
c         Write eigenvalues in fort.9 (d.xxx)
          IFOLD=SCIFOLD(RR,RI,SCNDIM)
          TFFOLD=0.0D0
          IF (IFOLD.GT.0) TFFOLD=RR(IFOLD)
          IHOPF=SCIHOPF(RR,RI,SCNDIM)
          TFHOPF=0.0D0
          IF (IHOPF.GT.0) TFHOPF=RR(IHOPF)
          NSTAB=SCNSTAB(RR,RI,SCNDIM)
          WRITE(9,700)TFFOLD
          WRITE(9,710)TFHOPF
          WRITE(9,720)NSTAB
          DO K=1,SCNDIM
            EV=CMPLX(RR(K),RI(K))
            WRITE(9,730)K,EV
          END DO
c
c         Test function 1: Boundary. eq. of v.f. I at l.b.v.
c         --------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+1)=DSCPR1(FI,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 2: Tang. point of v.f. J at l.b.v.
c         ------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+2)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 3: Tang. point of v.f. I at r.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+3)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 4: Tang. point of v.f. J at r.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(4)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+4)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 5: Boundary saddle
c         --------------------------------
          IF (SCFPSI(5)) THEN
c
c           Discontinuity boundary computation
            TMPIDIFF=0
            CALL SCBOUND(SCNDIM,U2,PAR,TMPIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+5)=H
          END IF
c
c         Test function 6: Saddle Fold/BR
c         -------------------------------
          IF (SCFPSI(6)) THEN
c
c           Test function
            PAR(SCIPSI0+6)=TFFOLD
          END IF
c
        ELSE IF ((SCIPS.EQ.PET1).OR.(SCIPS.EQ.PET2)) THEN
c
c         Connection pseudo-eq. - tangent point of the same v.f.
c         ------------------------------------------------------
c
c         Period overspecification
c         ------------------------
          PAR(SCIPARTC)=PAR(SCIPARTI)
c
c         Left and right boundary values
c         ------------------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
            U1(K)=SCGETP('BV1',0,K,U)
          END DO
c
c         Test function 1: Boundary eq. of v.f. I at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Test function
            PAR(SCIPSI0+1)=PAR(SCIPAR0+2)
          END IF
c
c         Test function 2: Boundary eq. of v.f. J at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Test function
            PAR(SCIPSI0+2)=PAR(SCIPAR0+1)
          END IF
c
c         Test function 3: Singular sliding point at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+3)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 4: Boundary eq. of v.f. I at r.b.v. (SCNDIM=2)
c         ------------------------------------------------------------
          IF (SCFPSI(4)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+4)=DSCPR1(FI,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 5: Tangent point of v.f. J at r.b.v.
c         --------------------------------------------------
          IF (SCFPSI(5)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+5)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
        ELSE IF ((SCIPS.EQ.PCT1).OR.(SCIPS.EQ.PCT2)) THEN
c
c         Connection pseudo-eq. - crossing - tg. pt of the same v.f.
c         ----------------------------------------------------------
c
c         Left and right boundary values
c         ------------------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
            U1(K)=SCGETP('BV1',0,K,U)
            U2(K)=SCGETP('BV1',0,SCNDIM+K,U)
          END DO
c
c         Test function 1: Boundary eq. of v.f. I at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Test function
            PAR(SCIPSI0+1)=PAR(SCIPAR0+2)
          END IF
c
c         Test function 2: Boundary eq. of v.f. J at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Test function
            PAR(SCIPSI0+2)=PAR(SCIPAR0+1)
          END IF
c
c         Test function 3: Singular sliding point at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+3)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 4: Tang. point of v.f. I at r.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(4)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+4)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 5: Tang. point of v.f. J at r.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(5)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+5)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 6: Boundary eq. of v.f. I at r.b.v. (SCNDIM=2)
c         ------------------------------------------------------------
          IF (SCFPSI(6)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U2,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U2,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+6)=DSCPR1(FI,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 7: Tang. point of v.f. J at r.b.v.
c         ------------------------------------------------
          IF (SCFPSI(7)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U2,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U2,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+7)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
        ELSE IF ((SCIPS.EQ.PTD1).OR.(SCIPS.EQ.PTD2)) THEN
c
c         Connection pseudo-eq. - tangent point of different v.f.
c         -------------------------------------------------------
c
c         Period overspecification
c         ------------------------
          PAR(SCIPARTC)=PAR(SCIPARTI)
c
c         Left and right boundary values
c         ------------------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
            U1(K)=SCGETP('BV1',0,K,U)
          END DO
c
c         Test function 1: Boundary eq. of v.f. I at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Test function
            PAR(SCIPSI0+1)=PAR(SCIPAR0+2)
          END IF
c
c         Test function 2: Boundary eq. of v.f. J at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Test function
            PAR(SCIPSI0+2)=PAR(SCIPAR0+1)
          END IF
c
c         Test function 3: Singular sliding point at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+3)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 4: Boundary eq. of v.f. J at r.b.v. (SCNDIM=2)
c         ------------------------------------------------------------
          IF (SCFPSI(4)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+4)=DSCPR1(FJ,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 5: Tangent point of v.f. I at r.b.v.
c         --------------------------------------------------
          IF (SCFPSI(5)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+5)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
        ELSE IF ((SCIPS.EQ.PCD1).OR.(SCIPS.EQ.PCD2)) THEN
c
c         Connection pseudo-eq. - crossing - tg. pt of different v.f.
c         -----------------------------------------------------------
c
c         Left and right boundary values
c         ------------------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
            U1(K)=SCGETP('BV1',0,K,U)
            U2(K)=SCGETP('BV1',0,SCNDIM+K,U)
          END DO
c
c         Test function 1: Boundary eq. of v.f. I at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Test function
            PAR(SCIPSI0+1)=PAR(SCIPAR0+2)
          END IF
c
c         Test function 2: Boundary eq. of v.f. J at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Test function
            PAR(SCIPSI0+2)=PAR(SCIPAR0+1)
          END IF
c
c         Test function 3: Singular sliding point at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+3)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 4: Tang. point of v.f. I at r.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(4)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+4)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 5: Tang. point of v.f. J at r.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(5)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+5)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 6: Boundary eq. of v.f. J at r.b.v. (SCNDIM=2)
c         ------------------------------------------------------------
          IF (SCFPSI(6)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U2,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U2,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+6)=DSCPR1(FJ,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 7: Tang. point of v.f. I at r.b.v.
c         ------------------------------------------------
          IF (SCFPSI(7)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U2,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U2,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+7)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
        ELSE IF ((SCIPS.EQ.PEP1).OR.(SCIPS.EQ.PEP2)) THEN
c
c         Connection pseudo-eq. - pseudo-eq. (SCNDIM=2)
c         ---------------------------------------------
c
c         Period overspecification
c         ------------------------
          PAR(SCIPARTC)=PAR(SCIPARTI)
c
c         Left and right boundary values
c         ------------------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
            U1(K)=SCGETP('BV1',0,K,U)
          END DO
c
c         Test function 1: Boundary eq. of v.f. I at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Test function
            PAR(SCIPSI0+1)=PAR(SCIPAR0+2)
          END IF
c
c         Test function 2: Boundary eq. of v.f. J at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Test function
            PAR(SCIPSI0+2)=PAR(SCIPAR0+1)
          END IF
c
c         Test function 3: Singular sliding point at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+3)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 4: Boundary eq. of v.f. I at r.b.v.
c         -------------------------------------------------
          IF (SCFPSI(4)) THEN
c
c           Test function
            PAR(SCIPSI0+4)=PAR(SCIPAR0+4)
          END IF
c
c         Test function 5: Boundary eq. of v.f. J at r.b.v.
c         -------------------------------------------------
          IF (SCFPSI(5)) THEN
c
c           Test function
            PAR(SCIPSI0+5)=PAR(SCIPAR0+3)
          END IF
c
c         Test function 6: Singular sliding point at r.b.v.
c         -------------------------------------------------
          IF (SCFPSI(6)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+6)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
        ELSE IF ((SCIPS.EQ.PCP1).OR.(SCIPS.EQ.PCP2)) THEN
c
c         Connection pseudo-eq. - crossing - pseudo-eq. (SCNDIM=2)
c         --------------------------------------------------------
c
c         Left and right boundary values
c         ------------------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
            U1(K)=SCGETP('BV1',0,K,U)
            U2(K)=SCGETP('BV1',0,SCNDIM+K,U)
          END DO
c
c         Test function 1: Boundary eq. of v.f. I at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Test function
            PAR(SCIPSI0+1)=PAR(SCIPAR0+2)
          END IF
c
c         Test function 2: Boundary eq. of v.f. J at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Test function
            PAR(SCIPSI0+2)=PAR(SCIPAR0+1)
          END IF
c
c         Test function 3: Singular sliding point at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+3)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 4: Tang. point of v.f. I at r.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(4)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+4)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 5: Tang. point of v.f. J at r.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(5)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+5)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 6: Boundary eq. of v.f. I at r.b.v.
c         -------------------------------------------------
          IF (SCFPSI(6)) THEN
c
c           Test function
            PAR(SCIPSI0+6)=PAR(SCIPAR0+4)
          END IF
c
c         Test function 7: Boundary eq. of v.f. J at r.b.v.
c         -------------------------------------------------
          IF (SCFPSI(7)) THEN
c
c           Test function
            PAR(SCIPSI0+7)=PAR(SCIPAR0+3)
          END IF
c
c         Test function 8: Singular sliding point at r.b.v.
c         -------------------------------------------------
          IF (SCFPSI(8)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U2,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U2,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+8)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
        ELSE IF ((SCIPS.EQ.PES1).OR.(SCIPS.EQ.PES2)) THEN
c
c         Connection pseudo-equilibrium - saddle
c         --------------------------------------
c
c         Left and right boundary values
c         ------------------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
            U1(K)=PAR(SCIPAR0+4+K)
          END DO
c
c         Vector fields computation at saddle
          CALL SCFUNC(SCNDIM,U1,PAR,SCIDIFF,FI,
     +      DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c         Eigenvalues computation
          CALL SCEIG(DFIDX,SCNDIM,1,0,RR,RI,VRET)
c
c         Write eigenvalues in fort.9 (d.xxx)
          IFOLD=SCIFOLD(RR,RI,SCNDIM)
          TFFOLD=0.0D0
          IF (IFOLD.GT.0) TFFOLD=RR(IFOLD)
          IHOPF=SCIHOPF(RR,RI,SCNDIM)
          TFHOPF=0.0D0
          IF (IHOPF.GT.0) TFHOPF=RR(IHOPF)
          NSTAB=SCNSTAB(RR,RI,SCNDIM)
          WRITE(9,700)TFFOLD
          WRITE(9,710)TFHOPF
          WRITE(9,720)NSTAB
          DO K=1,SCNDIM
            EV=CMPLX(RR(K),RI(K))
            WRITE(9,730)K,EV
          END DO
c
c         Test function 1: Boundary eq. of v.f. I at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Test function
            PAR(SCIPSI0+1)=PAR(SCIPAR0+2)
          END IF
c
c         Test function 2: Boundary eq. of v.f. J at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Test function
            PAR(SCIPSI0+2)=PAR(SCIPAR0+1)
          END IF
c
c         Test function 3: Singular sliding point at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+3)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 4: Boundary saddle
c         --------------------------------
          IF (SCFPSI(4)) THEN
c
c           Discontinuity boundary computation
            TMPIDIFF=0
            CALL SCBOUND(SCNDIM,U1,PAR,TMPIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+4)=H
          END IF
c
c         Test function 5: Saddle Hopf
c         ----------------------------
          IF (SCFPSI(5)) THEN
c
c           Test function
            PAR(SCIPSI0+5)=TFHOPF
          END IF
c
c         Test function 6: Saddle Fold/BR
c         -------------------------------
          IF (SCFPSI(6)) THEN
c
c           Test function
            PAR(SCIPSI0+6)=TFFOLD
          END IF
c
        ELSE IF ((SCIPS.EQ.PCS1).OR.(SCIPS.EQ.PCS2)) THEN
c
c         Connection pseudo-equilibrium - crossing - saddle
c         -------------------------------------------------
c
c         Left and right boundary values
c         ------------------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
            U1(K)=SCGETP('BV1',0,K,U)
            U2(K)=PAR(SCIPAR0+4+K)
          END DO
c
c         Vector fields computation at saddle
          CALL SCFUNC(SCNDIM,U2,PAR,SCIDIFF,FJ,
     +      DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c         Eigenvalues computation
          CALL SCEIG(DFJDX,SCNDIM,1,0,RR,RI,VRET)
c
c         Write eigenvalues in fort.9 (d.xxx)
          IFOLD=SCIFOLD(RR,RI,SCNDIM)
          TFFOLD=0.0D0
          IF (IFOLD.GT.0) TFFOLD=RR(IFOLD)
          IHOPF=SCIHOPF(RR,RI,SCNDIM)
          TFHOPF=0.0D0
          IF (IHOPF.GT.0) TFHOPF=RR(IHOPF)
          NSTAB=SCNSTAB(RR,RI,SCNDIM)
          WRITE(9,700)TFFOLD
          WRITE(9,710)TFHOPF
          WRITE(9,720)NSTAB
          DO K=1,SCNDIM
            EV=CMPLX(RR(K),RI(K))
            WRITE(9,730)K,EV
          END DO
c
c         Test function 1: Boundary eq. of v.f. I at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Test function
            PAR(SCIPSI0+1)=PAR(SCIPAR0+2)
          END IF
c
c         Test function 2: Boundary eq. of v.f. J at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Test function
            PAR(SCIPSI0+2)=PAR(SCIPAR0+1)
          END IF
c
c         Test function 3: Singular sliding point at l.b.v.
c         -------------------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+3)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 4: Tang. point of v.f. I at r.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(4)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+4)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 5: Tang. point of v.f. J at r.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(5)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+5)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 6: Boundary saddle
c         --------------------------------
          IF (SCFPSI(6)) THEN
c
c           Discontinuity boundary computation
            TMPIDIFF=0
            CALL SCBOUND(SCNDIM,U2,PAR,TMPIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+6)=H
          END IF
c
c         Test function 7: Saddle Hopf
c         ----------------------------
          IF (SCFPSI(7)) THEN
c
c           Test function
            PAR(SCIPSI0+7)=TFHOPF
          END IF
c
c         Test function 8: Saddle Fold/BR
c         -------------------------------
          IF (SCFPSI(8)) THEN
c
c           Test function
            PAR(SCIPSI0+8)=TFFOLD
          END IF
c
        ELSE IF ((SCIPS.EQ.SCY1).OR.(SCIPS.EQ.SCY2)) THEN
c
c         Sliding cycle
c         -------------
c
c         Left and right boundary values
c         ------------------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
            U1(K)=SCGETP('BV1',0,K,U)
          END DO
c
c         Test function 1: Boundary eq. of v.f. I at l.b.v. (SCNDIM=2)
c         ------------------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+1)=DSCPR1(FI,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 2: Tangent point of v.f. J at l.b.v.
c         --------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+2)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 3: Tangent point of v.f. I at r.b.v.
c         --------------------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+3)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 4: Tangent point of v.f. J at r.b.v.
c         --------------------------------------------------
          IF (SCFPSI(4)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+4)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 5: Pseudo-equilibrium at r.b.v. (SCNDIM=2)
c         --------------------------------------------------------
          IF (SCFPSI(5)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Normal component of vector fields
            FIN=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
            FJN=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
c
c           Test function
            PAR(SCIPSI0+5)=DHDXR(1)*(FJN*FI(1)-FIN*FJ(1))+
     +        DHDXR(2)*(FJN*FI(2)-FIN*FJ(2))
          END IF
c
        ELSE IF ((SCIPS.EQ.SCC1).OR.(SCIPS.EQ.SCC2)) THEN
c
c         Sliding-crossing cycle
c         ----------------------
c
c         Left and right boundary values
c         ------------------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
            U1(K)=SCGETP('BV1',0,K,U)
            U2(K)=SCGETP('BV1',0,SCNDIM+K,U)
          END DO
c
c         Test function 1: Boundary eq. of v.f. I at l.b.v. (SCNDIM=2)
c         ------------------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Test function
            PAR(SCIPSI0+1)=DSCPR1(FI,SCNDIM,1,DHDXR,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 2: Tang. point of v.f. J at l.b.v.
c         ------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+2)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 3: Tang. point of v.f. I at r.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+3)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 4: Tang. point of v.f. J at r.b.v. of v.f. I
c         ----------------------------------------------------------
          IF (SCFPSI(4)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+4)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 5: Tang. point of v.f. I at r.b.v.
c         ------------------------------------------------
          IF (SCFPSI(5)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U2,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U2,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+5)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 6: Tang. point of v.f. J at r.b.v.
c         ------------------------------------------------
          IF (SCFPSI(6)) THEN
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U2,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U2,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+6)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 7: Pseudo-equilibrium at r.b.v. (SCNDIM=2)
c         --------------------------------------------------------
          IF (SCFPSI(7)) THEN
c
c           Vector field I computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U2,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,I)
c
c           Vector field J computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U2,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,SCFJ(I))
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U2,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Tangent vector to the discontinuity boundary
            DHDXR(1)=-DHDX(2)
            DHDXR(2)=DHDX(1)
c
c           Normal component of vector fields
            FIN=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
            FJN=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
c
c           Test function
            PAR(SCIPSI0+7)=DHDXR(1)*(FJN*FI(1)-FIN*FJ(1))+
     +        DHDXR(2)*(FJN*FI(2)-FIN*FJ(2))
          END IF
c
        ELSE IF (SCIPS.EQ.CCY) THEN
c
c         Crossing cycle
c         --------------
c
c         Left and right boundary values
c         ------------------------------
          DO K=1,SCNDIM
            U0(K)=SCGETP('BV0',0,K,U)
            U1(K)=SCGETP('BV1',0,K,U)
          END DO
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'U0='
              CALL DARRPRN1(U0,SCNDIM,1,SCNDIM)
              PRINT *,'U1='
              CALL DARRPRN1(U1,SCNDIM,1,SCNDIM)
            END IF
          END IF
c
c         Test function 1: Tang. point of v.f. 1 at l.b.v. of v.f. 1
c         ----------------------------------------------------------
          IF (SCFPSI(1)) THEN
c
c           Vector field 1 computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,1)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+1)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 2: Tang. point of v.f. 2 at l.b.v. of v.f. 1
c         ----------------------------------------------------------
          IF (SCFPSI(2)) THEN
c
c           Vector field 2 computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U0,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,2)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+2)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 3: Tang. point of v.f. 1 at l.b.v. of v.f. 2
c         ----------------------------------------------------------
          IF (SCFPSI(3)) THEN
c
c           Vector field 1 computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FI,
     +        DFIDX,DFIDP,DFIDXDX,DFIDXDP,1)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+3)=DSCPR1(FI,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
c         Test function 4: Tang. point of v.f. 2 at l.b.v. of v.f. 2
c         ----------------------------------------------------------
          IF (SCFPSI(4)) THEN
c
c           Vector field 2 computation
            TMPIDIFF=0
            CALL SCFUNC(SCNDIM,U1,PAR,TMPIDIFF,FJ,
     +        DFJDX,DFJDP,DFJDXDX,DFJDXDP,2)
c
c           Discontinuity boundary computation
            CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +        H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c           Test function
            PAR(SCIPSI0+4)=DSCPR1(FJ,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          END IF
c
        ELSE
          WRITE(*,510)SCIPS
          STOP
        END IF
c
c       User defined functions
c       ----------------------
        CALL SCPVLS(SCNDIM,U,PAR,I)
c
c       Debug
        IF (DEBUG) THEN
          IF (DBGLEV.GE.DBGL2) THEN
            PRINT *,'exit from PVLS'
          END IF
        END IF
c
      RETURN
      END
c
c
c     ---------------------------------------------------
      SUBROUTINE BCND(NDIM,PAR,ICP,NBC,U0,U1,FB,IJAC,DBC)
c     ---------------------------------------------------
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'slidecont.h'
        INCLUDE 'debug.h'
c
        DIMENSION PAR(*),ICP(*),U0(NDIM),U1(NDIM),FB(NBC),DBC(NBC,*)
c
c       Common block reference
c       ----------------------
        LOGICAL DEFINED
        INTEGER SCNDIM,SCIPS,SCIRS,SCISTART
        INTEGER SCNICP,SCICP(SCNPARX)
        INTEGER SCNPSI,SCIPSI(SCNPSIX)
        INTEGER SCNFIXED,SCIFIXED(SCNPSIX)
        INTEGER SCIDIFF,SCNUZR
        LOGICAL SCFPSI(SCNPSIX),SCFFIXED(SCNPSIX)
c
        COMMON /PROB/ DEFINED
        COMMON /PROB/ SCNDIM,SCIPS,SCIRS,SCISTART
        COMMON /PROB/ SCNICP,SCICP
        COMMON /PROB/ SCNPSI,SCIPSI
        COMMON /PROB/ SCNFIXED,SCIFIXED
        COMMON /PROB/ SCIDIFF,SCNUZR
        COMMON /PROB/ SCFPSI,SCFFIXED
c
c       Functions references
c       --------------------
        INTEGER SCFJ
c
c       Vector fields
c       -------------
        DOUBLE PRECISION FI0(SCNDIMX)
        DOUBLE PRECISION DFI0DX(SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DFI0DP(SCNDIMX,SCNPARX)
        DOUBLE PRECISION DFI0DXDX(SCNDIMX,SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DFI0DXDP(SCNDIMX,SCNDIMX,SCNPARX)
c
        DOUBLE PRECISION FI1(SCNDIMX)
        DOUBLE PRECISION DFI1DX(SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DFI1DP(SCNDIMX,SCNPARX)
        DOUBLE PRECISION DFI1DXDX(SCNDIMX,SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DFI1DXDP(SCNDIMX,SCNDIMX,SCNPARX)
c
        DOUBLE PRECISION FJ0(SCNDIMX)
        DOUBLE PRECISION DFJ0DX(SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DFJ0DP(SCNDIMX,SCNPARX)
        DOUBLE PRECISION DFJ0DXDX(SCNDIMX,SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DFJ0DXDP(SCNDIMX,SCNDIMX,SCNPARX)
c
        DOUBLE PRECISION FJ1(SCNDIMX)
        DOUBLE PRECISION DFJ1DX(SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DFJ1DP(SCNDIMX,SCNPARX)
        DOUBLE PRECISION DFJ1DXDX(SCNDIMX,SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DFJ1DXDP(SCNDIMX,SCNDIMX,SCNPARX)
c
c       Zero initialization
        DATA FI0/SCNDIMX*0.0/
        DATA DFI0DX/SCNDIMX2*0.0/
        DATA DFI0DP/SCNDIMXNPARX*0.0/
        DATA DFI0DXDX/SCNDIMX3*0.0/
        DATA DFI0DXDP/SCNDIMX2NPARX*0.0/
c
        DATA FI1/SCNDIMX*0.0/
        DATA DFI1DX/SCNDIMX2*0.0/
        DATA DFI1DP/SCNDIMXNPARX*0.0/
        DATA DFI1DXDX/SCNDIMX3*0.0/
        DATA DFI1DXDP/SCNDIMX2NPARX*0.0/
c
        DATA FJ0/SCNDIMX*0.0/
        DATA DFJ0DX/SCNDIMX2*0.0/
        DATA DFJ0DP/SCNDIMXNPARX*0.0/
        DATA DFJ0DXDX/SCNDIMX3*0.0/
        DATA DFJ0DXDP/SCNDIMX2NPARX*0.0/
c
        DATA FJ1/SCNDIMX*0.0/
        DATA DFJ1DX/SCNDIMX2*0.0/
        DATA DFJ1DP/SCNDIMXNPARX*0.0/
        DATA DFJ1DXDX/SCNDIMX3*0.0/
        DATA DFJ1DXDP/SCNDIMX2NPARX*0.0/
c
c       Discontinuity boundary
c       ----------------------
        DOUBLE PRECISION H
        DOUBLE PRECISION DHDX(SCNDIMX)
        DOUBLE PRECISION DHDP(SCNPARX)
        DOUBLE PRECISION DHDXDX(SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DHDXDP(SCNDIMX,SCNPARX)
c
        DOUBLE PRECISION H0
        DOUBLE PRECISION DH0DX(SCNDIMX)
        DOUBLE PRECISION DH0DP(SCNPARX)
        DOUBLE PRECISION DH0DXDX(SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DH0DXDP(SCNDIMX,SCNPARX)
c
        DOUBLE PRECISION H1
        DOUBLE PRECISION DH1DX(SCNDIMX)
        DOUBLE PRECISION DH1DP(SCNPARX)
        DOUBLE PRECISION DH1DXDX(SCNDIMX,SCNDIMX)
        DOUBLE PRECISION DH1DXDP(SCNDIMX,SCNPARX)
c
c       Zero initialization
        DATA H/0.0/
        DATA DHDX/SCNDIMX*0.0/
        DATA DHDP/SCNPARX*0.0/
        DATA DHDXDX/SCNDIMX2*0.0/
        DATA DHDXDP/SCNDIMXNPARX*0.0/
c
        DATA H0/0.0/
        DATA DH0DX/SCNDIMX*0.0/
        DATA DH0DP/SCNPARX*0.0/
        DATA DH0DXDX/SCNDIMX2*0.0/
        DATA DH0DXDP/SCNDIMXNPARX*0.0/
c
        DATA H1/0.0/
        DATA DH1DX/SCNDIMX*0.0/
        DATA DH1DP/SCNPARX*0.0/
        DATA DH1DXDX/SCNDIMX2*0.0/
        DATA DH1DXDP/SCNDIMXNPARX*0.0/
c
c       Other variables
c       ---------------
        DOUBLE PRECISION U(SCNDIMX)
        DOUBLE PRECISION S(SCNDIMX)
        DOUBLE PRECISION W(SCNDIMX)
        DOUBLE PRECISION NU
c
c       Debug
        IF (DEBUG) THEN
          IF (DBGLEV.GE.DBGL1) THEN
            PRINT *,'enter in BCND(NBC=',NBC,')'
            PRINT *,'U0='
            CALL DARRPRN1(U0,SCNDIM,1,NDIM)
            PRINT *,'U1='
            CALL DARRPRN1(U1,SCNDIM,1,NDIM)
          END IF
        END IF
c
c       Common block initialization
c       ---------------------------
        IF (.NOT.DEFINED) THEN
          CALL SCPROB
        END IF
c
c       Vector field selection
c       ----------------------
        IF (SCIPS.LT.200) THEN
          I=1
        ELSE IF (SCIPS.LT.300) THEN
          I=2
        ELSE
          I=3
        END IF
c
c       DBC zero initialization
c       -----------------------
        IF (IJAC.GT.0) THEN
          DO K=1,NBC
            DO L=1,2*NDIM
              DBC(K,L)=0.0D0
            END DO
          END DO
        END IF
        IF (IJAC.GT.1) THEN
          DO K=1,NBC
            DO L=1,SCNPARX
              DBC(K,2*NDIM+L)=0.0D0
            END DO
          END DO
          DO K=1,NBC
            DO L=SCIPARTS,SCIPARTJ
              DBC(K,2*NDIM+L)=0.0D0
            END DO
          END DO
        END IF
c
c       Problem type selection
c       ----------------------
        IF (SCIPS.LT.100) THEN
c
c         AUTO97 problem type
c         -------------------
          CALL SCBCND(NDIM,PAR,ICP,NBC,U0,U1,FB,IJAC,DBC)
c
c       ELSE IF ((SCIPS.EQ.T3D1).OR.(SCIPS.EQ.T3D2)) THEN
c
c         Tangent point (SCNDIM=3)
c         ------------------------
c         Dummy step
c
c       ELSE IF ((SCIPS.EQ.TAN1).OR.(SCIPS.EQ.TAN2)) THEN
c
c         Tangent point (SCNDIM=2)
c         ------------------------
c         Dummy step
c
c       ELSE IF ((SCIPS.EQ.EQL1).OR.(SCIPS.EQ.EQL2)) THEN
c
c         Equilibrium
c         -----------
c         Dummy step
c
c       ELSE IF ((SCIPS.EQ.BEQ1).OR.(SCIPS.EQ.BEQ2)) THEN
c
c         Boundary equilibrium
c         --------------------
c         Dummy step
c
c       ELSE IF (SCIPS.EQ.BND) THEN
c
c         Discontinuity boundary (SCNDIM=2)
c         ---------------------------------
c         Dummy step
c
c       ELSE IF (SCIPS.EQ.PEQ) THEN
c
c         Pseudo-equilibrium
c         ------------------
c         Dummy step
c
c       ELSE IF (SCIPS.EQ.CTG) THEN
c
c         Collision of tangent points (SCNDIM=2)
c         --------------------------------------
c         Dummy step
c
c       ELSE IF ((SCIPS.EQ.CYC1).OR.(SCIPS.EQ.CYC2)) THEN
c
c         Cycle
c         -----
c         Dummy step
c
        ELSE IF ((SCIPS.EQ.TGB1).OR.(SCIPS.EQ.TGB2)) THEN
c
c         Connection tangent point - discontinuity boundary (SCNDIM=2)
c         ------------------------------------------------------------
c
c         Vector field I computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FI0,
     +      DFI0DX,DFI0DP,DFI0DXDX,DFI0DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at right boundary value
          CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +      H1,DH1DX,DH1DP,DH1DXDX,DH1DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H1=',H1
              PRINT *,'DH1DX='
              CALL DARRPRN1(DH1DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DH1DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          FB(2)=DSCPR1(FI0,SCNDIM,1,DH0DX,SCNDIM,1,SCNDIM)
          FB(3)=H1
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)         
            DBC(2,K)=DSCPR2(DFI0DX,SCNDIM,SCNDIM,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
            DBC(3,SCNDIM+K)=DH1DX(K)
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,2*SCNDIM+K)=DH0DP(K)         
            DBC(2,2*SCNDIM+K)=DSCPR2(DFI0DP,SCNDIM,SCNPARX,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
            DBC(3,2*SCNDIM+K)=DH1DP(K)         
          END DO
c
        ELSE IF ((SCIPS.EQ.TCB1).OR.(SCIPS.EQ.TCB2)) THEN
c
c         Connection tg. pt - crossing - disc. boundary (SCNDIM=2)
c         --------------------------------------------------------
c
c         Right boundary value
          DO K=1,SCNDIM
            U(K)=U1(SCNDIM+K)
          END DO
c
c         Vector field I computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FI0,
     +      DFI0DX,DFI0DP,DFI0DXDX,DFI0DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at r.b.v. of vector field I
          CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +      H1,DH1DX,DH1DP,DH1DXDX,DH1DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H1=',H1
              PRINT *,'DH1DX='
              CALL DARRPRN1(DH1DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DH1DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at r.b.v. of vector field J
          CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +      H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H=',H
              PRINT *,'DHDX='
              CALL DARRPRN1(DHDX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DHDP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          FB(2)=DSCPR1(FI0,SCNDIM,1,DH0DX,SCNDIM,1,SCNDIM)
          FB(3)=H1
          FB(4)=H
          DO K=1,SCNDIM
            FB(4+K)=U0(SCNDIM+K)-U1(K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)         
            DBC(2,K)=DSCPR2(DFI0DX,SCNDIM,SCNDIM,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
            DBC(3,2*SCNDIM+K)=DH1DX(K)
            DBC(4,3*SCNDIM+K)=DHDX(K)
            DBC(4+K,SCNDIM+K)=1.0D0
            DBC(4+K,2*SCNDIM+K)=-1.0D0
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,4*SCNDIM+K)=DH0DP(K)         
            DBC(2,4*SCNDIM+K)=DSCPR2(DFI0DP,SCNDIM,SCNPARX,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
            DBC(3,4*SCNDIM+K)=DH1DP(K)         
            DBC(4,4*SCNDIM+K)=DHDP(K)         
          END DO
c
        ELSE IF ((SCIPS.EQ.PEB1).OR.(SCIPS.EQ.PEB2)) THEN
c
c         Connection pseudo-equilibrium - discontinuity boundary
c         ------------------------------------------------------
c
c         Vector field I computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FI0,
     +      DFI0DX,DFI0DP,DFI0DXDX,DFI0DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FJ0,
     +      DFJ0DX,DFJ0DP,DFJ0DXDX,DFJ0DXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFJ0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFJ0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at right boundary value
          CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +      H1,DH1DX,DH1DP,DH1DXDX,DH1DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H1=',H1
              PRINT *,'DH1DX='
              CALL DARRPRN1(DH1DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DH1DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          DO K=1,SCNDIM
            FB(K+1)=PAR(SCIPAR0+1)*FI0(K)+PAR(SCIPAR0+2)*FJ0(K)
          END DO
          FB(SCNDIM+2)=PAR(SCIPAR0+1)+PAR(SCIPAR0+2)-1.0D0
          FB(SCNDIM+3)=H1
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)
            DO L=1,SCNDIM
              DBC(L+1,K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI0DX,SCNDIM,SCNDIM,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ0DX,SCNDIM,SCNDIM,L,K)
            END DO
            DBC(SCNDIM+3,SCNDIM+K)=DH1DX(K)
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,2*SCNDIM+K)=DH0DP(K)         
            DO L=1,SCNDIM
              DBC(L+1,2*SCNDIM+K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI0DP,SCNDIM,SCNPARX,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ0DP,SCNDIM,SCNPARX,L,K)
            END DO
            DBC(SCNDIM+3,2*SCNDIM+K)=DH1DP(K)         
          END DO
          DO K=1,SCNDIM
            DBC(K+1,2*SCNDIM+SCIPAR0+1)=FI0(K)
            DBC(K+1,2*SCNDIM+SCIPAR0+2)=FJ0(K)
          END DO
          DBC(1,2*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(1,2*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(SCNDIM+2,2*SCNDIM+SCIPAR0+1)=1.0D0
          DBC(SCNDIM+2,2*SCNDIM+SCIPAR0+2)=1.0D0
          DBC(SCNDIM+3,2*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(SCNDIM+3,2*SCNDIM+SCIPAR0+2)=0.0D0
c
        ELSE IF ((SCIPS.EQ.PCB1).OR.(SCIPS.EQ.PCB2)) THEN
c
c         Connection pseudo-eq. - crossing - discontinuity boundary
c         ---------------------------------------------------------
c
c         Right boundary value
          DO K=1,SCNDIM
            U(K)=U1(SCNDIM+K)
          END DO
c
c         Vector field I computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FI0,
     +      DFI0DX,DFI0DP,DFI0DXDX,DFI0DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FJ0,
     +      DFJ0DX,DFJ0DP,DFJ0DXDX,DFJ0DXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFJ0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFJ0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at r.b.v. of vector field I
          CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +      H1,DH1DX,DH1DP,DH1DXDX,DH1DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H1=',H1
              PRINT *,'DH1DX='
              CALL DARRPRN1(DH1DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DH1DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at r.b.v. of vector field J
          CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +      H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H=',H
              PRINT *,'DHDX='
              CALL DARRPRN1(DHDX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DHDP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          DO K=1,SCNDIM
            FB(K+1)=PAR(SCIPAR0+1)*FI0(K)+PAR(SCIPAR0+2)*FJ0(K)
          END DO
          FB(SCNDIM+2)=PAR(SCIPAR0+1)+PAR(SCIPAR0+2)-1.0D0
          FB(SCNDIM+3)=H1
          FB(SCNDIM+4)=H
          DO K=1,SCNDIM
            FB(SCNDIM+4+K)=U0(SCNDIM+K)-U1(K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)
            DO L=1,SCNDIM
              DBC(L+1,K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI0DX,SCNDIM,SCNDIM,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ0DX,SCNDIM,SCNDIM,L,K)
            END DO
            DBC(SCNDIM+3,2*SCNDIM+K)=DH1DX(K)
            DBC(SCNDIM+4,3*SCNDIM+K)=DHDX(K)
            DBC(SCNDIM+4+K,SCNDIM+K)=1.0D0
            DBC(SCNDIM+4+K,2*SCNDIM+K)=-1.0D0
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,4*SCNDIM+K)=DH0DP(K)         
            DO L=1,SCNDIM
              DBC(L+1,4*SCNDIM+K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI0DP,SCNDIM,SCNPARX,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ0DP,SCNDIM,SCNPARX,L,K)
            END DO
            DBC(SCNDIM+3,4*SCNDIM+K)=DH1DP(K)
            DBC(SCNDIM+4,4*SCNDIM+K)=DHDP(K)
          END DO
          DO K=1,SCNDIM
            DBC(K+1,4*SCNDIM+SCIPAR0+1)=FI0(K)
            DBC(K+1,4*SCNDIM+SCIPAR0+2)=FJ0(K)
            DBC(SCNDIM+4+K,4*SCNDIM+SCIPAR0+1)=0.0D0
            DBC(SCNDIM+4+K,4*SCNDIM+SCIPAR0+2)=0.0D0
          END DO
          DBC(1,4*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(1,4*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(SCNDIM+2,4*SCNDIM+SCIPAR0+1)=1.0D0
          DBC(SCNDIM+2,4*SCNDIM+SCIPAR0+2)=1.0D0
          DBC(SCNDIM+3,4*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(SCNDIM+3,4*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(SCNDIM+4,4*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(SCNDIM+4,4*SCNDIM+SCIPAR0+2)=0.0D0
c
        ELSE IF ((SCIPS.EQ.TCH1).OR.(SCIPS.EQ.TCH2)) THEN
c
c         Grazing
c         -------
c
c         Vector field I computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FI0,
     +      DFI0DX,DFI0DP,DFI0DXDX,DFI0DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          FB(2)=DSCPR1(FI0,SCNDIM,1,DH0DX,SCNDIM,1,SCNDIM)
          DO K=1,SCNDIM
            FB(K+2)=U0(K)-U1(K)         
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)         
            DBC(2,K)=DSCPR2(DFI0DX,SCNDIM,SCNDIM,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
            DBC(K+2,K)=1.0D0         
            DBC(K+2,SCNDIM+K)=-1.0D0         
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,2*SCNDIM+K)=DH0DP(K)         
            DBC(2,2*SCNDIM+K)=DSCPR2(DFI0DP,SCNDIM,SCNPARX,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
          END DO
c
        ELSE IF ((SCIPS.EQ.TGT1).OR.(SCIPS.EQ.TGT2)) THEN
c
c         Connection tg. pt - tg. pt of the same v.f. (SCNDIM=2)
c         ------------------------------------------------------
c
c         Vector field I computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FI0,
     +      DFI0DX,DFI0DP,DFI0DXDX,DFI0DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field I computation at right boundary value
          CALL SCFUNC(SCNDIM,U1,PAR,SCIDIFF,FI1,
     +      DFI1DX,DFI1DP,DFI1DXDX,DFI1DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI1,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI1DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI1DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at right boundary value
          CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +      H1,DH1DX,DH1DP,DH1DXDX,DH1DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H1=',H1
              PRINT *,'DH1DX='
              CALL DARRPRN1(DH1DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DH1DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          FB(2)=DSCPR1(FI0,SCNDIM,1,DH0DX,SCNDIM,1,SCNDIM)
          FB(3)=H1
          FB(4)=DSCPR1(FI1,SCNDIM,1,DH1DX,SCNDIM,1,SCNDIM)
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)         
            DBC(2,K)=DSCPR2(DFI0DX,SCNDIM,SCNDIM,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
            DBC(3,SCNDIM+K)=DH1DX(K)
            DBC(4,SCNDIM+K)=DSCPR2(DFI1DX,SCNDIM,SCNDIM,1,K,1,
     +        DH1DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI1,SCNDIM,1,1,1,1,
     +        DH1DXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,2*SCNDIM+K)=DH0DP(K)         
            DBC(2,2*SCNDIM+K)=DSCPR2(DFI0DP,SCNDIM,SCNPARX,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
            DBC(3,2*SCNDIM+K)=DH1DP(K)         
            DBC(4,2*SCNDIM+K)=DSCPR2(DFI1DP,SCNDIM,SCNPARX,1,K,1,
     +        DH1DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI1,SCNDIM,1,1,1,1,
     +        DH1DXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
          END DO
c
        ELSE IF ((SCIPS.EQ.TCT1).OR.(SCIPS.EQ.TCT2)) THEN
c
c         Connection tg. pt - cros. - tg. pt same v.f. (SCNDIM=2)
c         -------------------------------------------------------
c
c         Right boundary value
          DO K=1,SCNDIM
            U(K)=U1(SCNDIM+K)
          END DO
c
c         Vector field I computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FI0,
     +      DFI0DX,DFI0DP,DFI0DXDX,DFI0DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field I computation at r.b.v. of vector field J
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI1,
     +      DFI1DX,DFI1DP,DFI1DXDX,DFI1DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI1,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI1DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI1DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at right boundary value
          CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +      H1,DH1DX,DH1DP,DH1DXDX,DH1DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H1=',H1
              PRINT *,'DH1DX='
              CALL DARRPRN1(DH1DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DH1DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at r.b.v. of vector field J
          CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +      H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H=',H
              PRINT *,'DHDX='
              CALL DARRPRN1(DHDX,SCNDIM,1,SCNDIM)
              PRINT *,'DHDP='
              CALL DARRPRN1(DHDP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          FB(2)=DSCPR1(FI0,SCNDIM,1,DH0DX,SCNDIM,1,SCNDIM)
          FB(3)=H1
          FB(4)=H
          FB(5)=DSCPR1(FI1,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          DO K=1,SCNDIM
            FB(5+K)=U0(SCNDIM+K)-U1(K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)         
            DBC(2,K)=DSCPR2(DFI0DX,SCNDIM,SCNDIM,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
            DBC(3,2*SCNDIM+K)=DH1DX(K)
            DBC(4,3*SCNDIM+K)=DHDX(K)
            DBC(5,3*SCNDIM+K)=DSCPR2(DFI1DX,SCNDIM,SCNDIM,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI1,SCNDIM,1,1,1,1,
     +        DHDXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
            DBC(5+K,SCNDIM+K)=1.0D0
            DBC(5+K,2*SCNDIM+K)=-1.0D0
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,4*SCNDIM+K)=DH0DP(K)         
            DBC(2,4*SCNDIM+K)=DSCPR2(DFI0DP,SCNDIM,SCNPARX,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
            DBC(3,4*SCNDIM+K)=DH1DP(K)         
            DBC(4,4*SCNDIM+K)=DHDP(K)         
            DBC(5,4*SCNDIM+K)=DSCPR2(DFI1DP,SCNDIM,SCNPARX,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI1,SCNDIM,1,1,1,1,
     +        DHDXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
          END DO
c
        ELSE IF ((SCIPS.EQ.TTD1).OR.(SCIPS.EQ.TTD2)) THEN
c
c         Connection tg. pt - tg. pt of different v.f. (SCNDIM=2)
c         -------------------------------------------------------
c
c         Vector field I computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FI0,
     +      DFI0DX,DFI0DP,DFI0DXDX,DFI0DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation at right boundary value
          CALL SCFUNC(SCNDIM,U1,PAR,SCIDIFF,FJ1,
     +      DFJ1DX,DFJ1DP,DFJ1DXDX,DFJ1DXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ1,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFJ1DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFJ1DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at right boundary value
          CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +      H1,DH1DX,DH1DP,DH1DXDX,DH1DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H1=',H1
              PRINT *,'DH1DX='
              CALL DARRPRN1(DH1DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DH1DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          FB(2)=DSCPR1(FI0,SCNDIM,1,DH0DX,SCNDIM,1,SCNDIM)
          FB(3)=H1
          FB(4)=DSCPR1(FJ1,SCNDIM,1,DH1DX,SCNDIM,1,SCNDIM)
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)         
            DBC(2,K)=DSCPR2(DFI0DX,SCNDIM,SCNDIM,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
            DBC(3,SCNDIM+K)=DH1DX(K)
            DBC(4,SCNDIM+K)=DSCPR2(DFJ1DX,SCNDIM,SCNDIM,1,K,1,
     +        DH1DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FJ1,SCNDIM,1,1,1,1,
     +        DH1DXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,2*SCNDIM+K)=DH0DP(K)         
            DBC(2,2*SCNDIM+K)=DSCPR2(DFI0DP,SCNDIM,SCNPARX,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
            DBC(3,2*SCNDIM+K)=DH1DP(K)         
            DBC(4,2*SCNDIM+K)=DSCPR2(DFJ1DP,SCNDIM,SCNPARX,1,K,1,
     +        DH1DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FJ1,SCNDIM,1,1,1,1,
     +        DH1DXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
          END DO
c
        ELSE IF ((SCIPS.EQ.TCD1).OR.(SCIPS.EQ.TCD2)) THEN
c
c         Connection tg. pt - cros. - tg. pt different v.f. (SCNDIM=2)
c         ------------------------------------------------------------
c
c         Right boundary value
          DO K=1,SCNDIM
            U(K)=U1(SCNDIM+K)
          END DO
c
c         Vector field I computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FI0,
     +      DFI0DX,DFI0DP,DFI0DXDX,DFI0DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation at r.b.v. of vector field J
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FJ1,
     +      DFJ1DX,DFJ1DP,DFJ1DXDX,DFJ1DXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ1,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFJ1DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFJ1DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at right boundary value
          CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +      H1,DH1DX,DH1DP,DH1DXDX,DH1DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H1=',H1
              PRINT *,'DH1DX='
              CALL DARRPRN1(DH1DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DH1DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at r.b.v. of vector field J
          CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +      H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H=',H
              PRINT *,'DHDX='
              CALL DARRPRN1(DHDX,SCNDIM,1,SCNDIM)
              PRINT *,'DHDP='
              CALL DARRPRN1(DHDP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          FB(2)=DSCPR1(FI0,SCNDIM,1,DH0DX,SCNDIM,1,SCNDIM)
          FB(3)=H1
          FB(4)=H
          FB(5)=DSCPR1(FJ1,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          DO K=1,SCNDIM
            FB(5+K)=U0(SCNDIM+K)-U1(K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)         
            DBC(2,K)=DSCPR2(DFI0DX,SCNDIM,SCNDIM,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
            DBC(3,2*SCNDIM+K)=DH1DX(K)
            DBC(4,3*SCNDIM+K)=DHDX(K)
            DBC(5,3*SCNDIM+K)=DSCPR2(DFJ1DX,SCNDIM,SCNDIM,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FJ1,SCNDIM,1,1,1,1,
     +        DHDXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
            DBC(5+K,SCNDIM+K)=1.0D0
            DBC(5+K,2*SCNDIM+K)=-1.0D0
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,4*SCNDIM+K)=DH0DP(K)         
            DBC(2,4*SCNDIM+K)=DSCPR2(DFI0DP,SCNDIM,SCNPARX,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
            DBC(3,4*SCNDIM+K)=DH1DP(K)         
            DBC(4,4*SCNDIM+K)=DHDP(K)         
            DBC(5,4*SCNDIM+K)=DSCPR2(DFJ1DP,SCNDIM,SCNPARX,1,K,1,
     +        DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FJ1,SCNDIM,1,1,1,1,
     +        DHDXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
          END DO
c
        ELSE IF ((SCIPS.EQ.TGP1).OR.(SCIPS.EQ.TGP2)) THEN
c
c         Connection tangent point - pseudo-equilibrium
c         ---------------------------------------------
c
c         Vector field I computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FI0,
     +      DFI0DX,DFI0DP,DFI0DXDX,DFI0DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field I computation at right boundary value
          CALL SCFUNC(SCNDIM,U1,PAR,SCIDIFF,FI1,
     +      DFI1DX,DFI1DP,DFI1DXDX,DFI1DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI1,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI1DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI1DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation at right boundary value
          CALL SCFUNC(SCNDIM,U1,PAR,SCIDIFF,FJ1,
     +      DFJ1DX,DFJ1DP,DFJ1DXDX,DFJ1DXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ1,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFJ1DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFJ1DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at right boundary value
          CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +      H1,DH1DX,DH1DP,DH1DXDX,DH1DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H1=',H1
              PRINT *,'DH1DX='
              CALL DARRPRN1(DH1DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DH1DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          FB(2)=DSCPR1(FI0,SCNDIM,1,DH0DX,SCNDIM,1,SCNDIM)
          FB(3)=H1
          DO K=1,SCNDIM
            FB(K+3)=PAR(SCIPAR0+1)*FI1(K)+PAR(SCIPAR0+2)*FJ1(K)
          END DO
          FB(SCNDIM+4)=PAR(SCIPAR0+1)+PAR(SCIPAR0+2)-1.0D0
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)
            DBC(2,K)=DSCPR2(DFI0DX,SCNDIM,SCNDIM,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
            DBC(3,SCNDIM+K)=DH1DX(K)
            DO L=1,SCNDIM
              DBC(L+3,SCNDIM+K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI1DX,SCNDIM,SCNDIM,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ1DX,SCNDIM,SCNDIM,L,K)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,2*SCNDIM+K)=DH0DP(K)         
            DBC(2,2*SCNDIM+K)=DSCPR2(DFI0DP,SCNDIM,SCNPARX,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
            DBC(3,2*SCNDIM+K)=DH1DP(K)         
            DO L=1,SCNDIM
              DBC(L+3,2*SCNDIM+K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI1DP,SCNDIM,SCNPARX,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ1DP,SCNDIM,SCNPARX,L,K)
            END DO
          END DO
          DO K=1,SCNDIM
            DBC(K+3,2*SCNDIM+SCIPAR0+1)=FI1(K)
            DBC(K+3,2*SCNDIM+SCIPAR0+2)=FJ1(K)
          END DO
          DBC(1,2*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(1,2*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(2,2*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(2,2*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(3,2*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(3,2*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(SCNDIM+4,2*SCNDIM+SCIPAR0+1)=1.0D0
          DBC(SCNDIM+4,2*SCNDIM+SCIPAR0+2)=1.0D0
c
        ELSE IF ((SCIPS.EQ.TCP1).OR.(SCIPS.EQ.TCP2)) THEN
c
c         Connection tangent point - crossing - pseudo-equilibrium
c         --------------------------------------------------------
c
c         Right boundary value
          DO K=1,SCNDIM
            U(K)=U1(SCNDIM+K)
          END DO
c
c         Vector field I computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FI0,
     +      DFI0DX,DFI0DP,DFI0DXDX,DFI0DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field I computation at r.b.v. of vector field J
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI1,
     +      DFI1DX,DFI1DP,DFI1DXDX,DFI1DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI1,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI1DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI1DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation at at r.b.v. of vector field J
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FJ1,
     +      DFJ1DX,DFJ1DP,DFJ1DXDX,DFJ1DXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ1,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFJ1DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFJ1DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at r.b.v. of vector field I
          CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +      H1,DH1DX,DH1DP,DH1DXDX,DH1DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H1=',H1
              PRINT *,'DH1DX='
              CALL DARRPRN1(DH1DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DH1DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at r.b.v. of vector field J
          CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +      H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H=',H
              PRINT *,'DHDX='
              CALL DARRPRN1(DHDX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DHDP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          FB(2)=DSCPR1(FI0,SCNDIM,1,DH0DX,SCNDIM,1,SCNDIM)
          FB(3)=H1
          FB(4)=H
          DO K=1,SCNDIM
            FB(K+4)=PAR(SCIPAR0+1)*FI1(K)+PAR(SCIPAR0+2)*FJ1(K)
          END DO
          FB(SCNDIM+5)=PAR(SCIPAR0+1)+PAR(SCIPAR0+2)-1.0D0
          DO K=1,SCNDIM
            FB(SCNDIM+5+K)=U0(SCNDIM+K)-U1(K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)
            DBC(2,K)=DSCPR2(DFI0DX,SCNDIM,SCNDIM,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
            DBC(3,2*SCNDIM+K)=DH1DX(K)
            DBC(4,3*SCNDIM+K)=DHDX(K)
            DO L=1,SCNDIM
              DBC(L+4,3*SCNDIM+K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI1DX,SCNDIM,SCNDIM,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ1DX,SCNDIM,SCNDIM,L,K)
            END DO
            DBC(SCNDIM+5+K,SCNDIM+K)=1.0D0
            DBC(SCNDIM+5+K,2*SCNDIM+K)=-1.0D0
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,4*SCNDIM+K)=DH0DP(K)         
            DBC(2,4*SCNDIM+K)=DSCPR2(DFI0DP,SCNDIM,SCNPARX,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
            DBC(3,4*SCNDIM+K)=DH1DP(K)         
            DBC(4,4*SCNDIM+K)=DHDP(K)         
            DO L=1,SCNDIM
              DBC(L+4,4*SCNDIM+K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI1DP,SCNDIM,SCNPARX,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ1DP,SCNDIM,SCNPARX,L,K)
            END DO
          END DO
          DO K=1,SCNDIM
            DBC(K+4,4*SCNDIM+SCIPAR0+1)=FI1(K)
            DBC(K+4,4*SCNDIM+SCIPAR0+2)=FJ1(K)
            DBC(SCNDIM+5+K,4*SCNDIM+SCIPAR0+1)=0.0D0
            DBC(SCNDIM+5+K,4*SCNDIM+SCIPAR0+2)=0.0D0
          END DO
          DBC(1,4*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(1,4*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(2,4*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(2,4*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(3,4*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(3,4*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(4,4*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(4,4*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(SCNDIM+5,4*SCNDIM+SCIPAR0+1)=1.0D0
          DBC(SCNDIM+5,4*SCNDIM+SCIPAR0+2)=1.0D0
c
        ELSE IF ((SCIPS.EQ.TGS1).OR.(SCIPS.EQ.TGS2)) THEN
c
c         Connection tangent point - saddle (SCNDIM=2)
c         --------------------------------------------
c
c         Saddle
          DO K=1,SCNDIM
            S(K)=PAR(SCIPAR0+4+K)
          END DO
c
c         Unstable eigenvalue/eigenvector
          NU=PAR(SCIPAR0+5+2*SCNDIM)
          DO K=1,SCNDIM
            W(K)=PAR(SCIPAR0+4+SCNDIM+K)
          END DO
c
c         Vector field I computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FI0,
     +      DFI0DX,DFI0DP,DFI0DXDX,DFI0DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field I computation at the saddle
          CALL SCFUNC(SCNDIM,S,PAR,SCIDIFF,FI1,
     +      DFI1DX,DFI1DP,DFI1DXDX,DFI1DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI1,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI1DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI1DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          FB(2)=DSCPR1(FI0,SCNDIM,1,DH0DX,SCNDIM,1,SCNDIM)
          DO K=1,SCNDIM
            FB(K+2)=FI1(K)
            FB(K+SCNDIM+2)=DSCPR2(DFI1DX,SCNDIM,SCNDIM,1,K,1,
     +        W,SCNDIM,1,1,1,1,SCNDIM)-NU*W(K)
          END DO
          FB(2*SCNDIM+3)=DSCPR1(W,SCNDIM,1,W,SCNDIM,1,SCNDIM)-1.0D0
          FB(2*SCNDIM+4)=DSCPR1(W,SCNDIM,1,S,SCNDIM,1,SCNDIM)-
     +      DSCPR1(W,SCNDIM,1,U1,SCNDIM,1,SCNDIM)
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)
            DBC(2,K)=DSCPR2(DFI0DX,SCNDIM,SCNDIM,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
            DBC(2*SCNDIM+4,SCNDIM+K)=-W(K)
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,2*SCNDIM+K)=DH0DP(K)
            DBC(2,2*SCNDIM+K)=DSCPR2(DFI0DP,SCNDIM,SCNPARX,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
            DO L=1,SCNDIM
              DBC(L+2,2*SCNDIM+K)=DARR2(DFI1DP,SCNDIM,SCNPARX,L,K)
              DBC(L+SCNDIM+2,2*SCNDIM+K)=DSCPR3(DFI1DXDP,SCNDIM,
     +          SCNDIM,SCNPARX,1,L,K,1,W,SCNDIM,1,1,1,1,1,1,SCNDIM)
            END DO
          END DO
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DBC(L+2,2*SCNDIM+SCIPAR0+4+K)=DARR2(DFI1DX,SCNDIM,
     +          SCNDIM,L,K)
              DBC(L+2,2*SCNDIM+SCIPAR0+4+SCNDIM+K)=0.0D0
              DBC(L+SCNDIM+2,2*SCNDIM+SCIPAR0+4+K)=DSCPR3(
     +          DFI1DXDX,SCNDIM,SCNDIM,SCNDIM,1,L,K,1,
     +          W,SCNDIM,1,1,1,1,1,1,SCNDIM)
              DBC(L+SCNDIM+2,2*SCNDIM+SCIPAR0+4+SCNDIM+K)=DARR2(
     +          DFI1DX,SCNDIM,SCNDIM,K,L)
            END DO
            DBC(1,2*SCNDIM+SCIPAR0+4+K)=0.0D0
            DBC(1,2*SCNDIM+SCIPAR0+4+SCNDIM+K)=0.0D0
            DBC(2,2*SCNDIM+SCIPAR0+4+K)=0.0D0
            DBC(2,2*SCNDIM+SCIPAR0+4+SCNDIM+K)=0.0D0
            DBC(K+2,2*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
            DBC(K+SCNDIM+2,2*SCNDIM+SCIPAR0+4+SCNDIM+K)=
     +        DBC(K+SCNDIM+2,2*SCNDIM+SCIPAR0+4+SCNDIM+K)-NU
            DBC(K+SCNDIM+2,2*SCNDIM+SCIPAR0+5+2*SCNDIM)=-W(K)
            DBC(2*SCNDIM+3,2*SCNDIM+SCIPAR0+4+K)=0.0D0
            DBC(2*SCNDIM+3,2*SCNDIM+SCIPAR0+4+SCNDIM+K)=2.0D0*W(K)
            DBC(2*SCNDIM+4,2*SCNDIM+SCIPAR0+4+K)=W(K)
            DBC(2*SCNDIM+4,2*SCNDIM+SCIPAR0+4+SCNDIM+K)=S(K)-U1(K)
          END DO
          DBC(1,2*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
          DBC(2,2*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
          DBC(2*SCNDIM+3,2*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0
          DBC(2*SCNDIM+4,2*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
c
        ELSE IF ((SCIPS.EQ.TCS1).OR.(SCIPS.EQ.TCS2)) THEN
c
c         Connection tangent point - crossing - saddle (SCNDIM=2)
c         -------------------------------------------------------
c
c         Right boundary value
          DO K=1,SCNDIM
            U(K)=U1(SCNDIM+K)
          END DO
c
c         Saddle
          DO K=1,SCNDIM
            S(K)=PAR(SCIPAR0+4+K)
          END DO
c
c         Unstable eigenvalue/eigenvector
          NU=PAR(SCIPAR0+5+2*SCNDIM)
          DO K=1,SCNDIM
            W(K)=PAR(SCIPAR0+4+SCNDIM+K)
          END DO
c
c         Vector field I computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FI0,
     +      DFI0DX,DFI0DP,DFI0DXDX,DFI0DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation at the saddle
          CALL SCFUNC(SCNDIM,S,PAR,SCIDIFF,FJ1,
     +      DFJ1DX,DFJ1DP,DFJ1DXDX,DFJ1DXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ1,SCNDIM,1,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DX='
              CALL DARRPRN2(DFJ1DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DP='
              CALL DARRPRN2(DFJ1DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at r.b.v. of vector field I
          CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +      H1,DH1DX,DH1DP,DH1DXDX,DH1DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H1=',H1
              PRINT *,'DH1DX='
              CALL DARRPRN1(DH1DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DH1DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          FB(2)=DSCPR1(FI0,SCNDIM,1,DH0DX,SCNDIM,1,SCNDIM)
          FB(3)=H1
          DO K=1,SCNDIM
            FB(3+K)=U0(SCNDIM+K)-U1(K)
          END DO
          DO K=1,SCNDIM
            FB(SCNDIM+K+3)=FJ1(K)
            FB(2*SCNDIM+K+3)=DSCPR2(DFJ1DX,SCNDIM,SCNDIM,1,K,1,
     +        W,SCNDIM,1,1,1,1,SCNDIM)-NU*W(K)
          END DO
          FB(3*SCNDIM+4)=DSCPR1(W,SCNDIM,1,W,SCNDIM,1,SCNDIM)-1.0D0
          FB(3*SCNDIM+5)=DSCPR1(W,SCNDIM,1,S,SCNDIM,1,SCNDIM)-
     +      DSCPR1(W,SCNDIM,1,U,SCNDIM,1,SCNDIM)
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)         
            DBC(2,K)=DSCPR2(DFI0DX,SCNDIM,SCNDIM,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
            DBC(3,2*SCNDIM+K)=DH1DX(K)
            DBC(3+K,SCNDIM+K)=1.0D0
            DBC(3+K,2*SCNDIM+K)=-1.0D0
            DBC(3*SCNDIM+5,3*SCNDIM+K)=-W(K)
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,4*SCNDIM+K)=DH0DP(K)         
            DBC(2,4*SCNDIM+K)=DSCPR2(DFI0DP,SCNDIM,SCNPARX,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
            DBC(3,4*SCNDIM+K)=DH1DP(K)
            DO L=1,SCNDIM
              DBC(SCNDIM+L+3,4*SCNDIM+K)=DARR2(DFJ1DP,
     +          SCNDIM,SCNPARX,L,K)
              DBC(2*SCNDIM+L+3,4*SCNDIM+K)=DSCPR3(DFJ1DXDP,SCNDIM,
     +          SCNDIM,SCNPARX,1,L,K,1,W,SCNDIM,1,1,1,1,1,1,SCNDIM)
            END DO
          END DO
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DBC(L+3,4*SCNDIM+SCIPAR0+4+K)=0.0D0
              DBC(L+3,4*SCNDIM+SCIPAR0+4+SCNDIM+K)=0.0D0
              DBC(SCNDIM+L+3,4*SCNDIM+SCIPAR0+4+K)=DARR2(DFJ1DX,
     +          SCNDIM,SCNDIM,L,K)
              DBC(SCNDIM+L+3,4*SCNDIM+SCIPAR0+4+SCNDIM+K)=0.0D0
              DBC(2*SCNDIM+L+3,4*SCNDIM+SCIPAR0+4+K)=DSCPR3(
     +          DFJ1DXDX,SCNDIM,SCNDIM,SCNDIM,1,L,K,1,
     +          W,SCNDIM,1,1,1,1,1,1,SCNDIM)
              DBC(2*SCNDIM+L+3,4*SCNDIM+SCIPAR0+4+SCNDIM+K)=DARR2(
     +          DFJ1DX,SCNDIM,SCNDIM,K,L)
            END DO
            DBC(1,4*SCNDIM+SCIPAR0+4+K)=0.0D0
            DBC(1,4*SCNDIM+SCIPAR0+4+SCNDIM+K)=0.0D0
            DBC(2,4*SCNDIM+SCIPAR0+4+K)=0.0D0
            DBC(2,4*SCNDIM+SCIPAR0+4+SCNDIM+K)=0.0D0
            DBC(3,4*SCNDIM+SCIPAR0+4+K)=0.0D0
            DBC(3,4*SCNDIM+SCIPAR0+4+SCNDIM+K)=0.0D0
            DBC(K+3,4*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
            DBC(SCNDIM+K+3,4*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
            DBC(2*SCNDIM+K+3,4*SCNDIM+SCIPAR0+4+SCNDIM+K)=
     +        DBC(2*SCNDIM+K+3,4*SCNDIM+SCIPAR0+4+SCNDIM+K)-NU
            DBC(2*SCNDIM+K+3,4*SCNDIM+SCIPAR0+5+2*SCNDIM)=-W(K)
            DBC(3*SCNDIM+4,4*SCNDIM+SCIPAR0+4+K)=0.0D0
            DBC(3*SCNDIM+4,4*SCNDIM+SCIPAR0+4+SCNDIM+K)=2.0D0*W(K)
            DBC(3*SCNDIM+5,4*SCNDIM+SCIPAR0+4+K)=W(K)
            DBC(3*SCNDIM+5,4*SCNDIM+SCIPAR0+4+SCNDIM+K)=S(K)-U(K)
          END DO
          DBC(1,4*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
          DBC(2,4*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
          DBC(3,4*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
          DBC(3*SCNDIM+4,4*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
          DBC(3*SCNDIM+5,4*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
c
        ELSE IF ((SCIPS.EQ.PET1).OR.(SCIPS.EQ.PET2)) THEN
c
c         Connection pseudo-eq. - tangent point of the same v.f.
c         ------------------------------------------------------
c
c         Vector field I computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FI0,
     +      DFI0DX,DFI0DP,DFI0DXDX,DFI0DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FJ0,
     +      DFJ0DX,DFJ0DP,DFJ0DXDX,DFJ0DXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFJ0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFJ0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field I computation at right boundary value
          CALL SCFUNC(SCNDIM,U1,PAR,SCIDIFF,FI1,
     +      DFI1DX,DFI1DP,DFI1DXDX,DFI1DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI1,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI1DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI1DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at right boundary value
          CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +      H1,DH1DX,DH1DP,DH1DXDX,DH1DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H1=',H1
              PRINT *,'DH1DX='
              CALL DARRPRN1(DH1DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DH1DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          DO K=1,SCNDIM
            FB(K+1)=PAR(SCIPAR0+1)*FI0(K)+PAR(SCIPAR0+2)*FJ0(K)
          END DO
          FB(SCNDIM+2)=PAR(SCIPAR0+1)+PAR(SCIPAR0+2)-1.0D0
          FB(SCNDIM+3)=H1
          FB(SCNDIM+4)=DSCPR1(FI1,SCNDIM,1,DH1DX,SCNDIM,1,SCNDIM)
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)
            DO L=1,SCNDIM
              DBC(L+1,K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI0DX,SCNDIM,SCNDIM,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ0DX,SCNDIM,SCNDIM,L,K)
            END DO
            DBC(SCNDIM+3,SCNDIM+K)=DH1DX(K)
            DBC(SCNDIM+4,SCNDIM+K)=DSCPR2(DFI1DX,SCNDIM,SCNDIM,1,K,1,
     +        DH1DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI1,SCNDIM,1,1,1,1,
     +        DH1DXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,2*SCNDIM+K)=DH0DP(K)         
            DO L=1,SCNDIM
              DBC(L+1,2*SCNDIM+K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI0DP,SCNDIM,SCNPARX,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ0DP,SCNDIM,SCNPARX,L,K)
            END DO
            DBC(SCNDIM+3,2*SCNDIM+K)=DH1DP(K)         
            DBC(SCNDIM+4,2*SCNDIM+K)=DSCPR2(DFI1DP,SCNDIM,SCNPARX,
     +        1,K,1,DH1DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI1,SCNDIM,1,1,1,1,
     +        DH1DXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
          END DO
          DO K=1,SCNDIM
            DBC(K+1,2*SCNDIM+SCIPAR0+1)=FI0(K)
            DBC(K+1,2*SCNDIM+SCIPAR0+2)=FJ0(K)
          END DO
          DBC(1,2*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(1,2*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(SCNDIM+2,2*SCNDIM+SCIPAR0+1)=1.0D0
          DBC(SCNDIM+2,2*SCNDIM+SCIPAR0+2)=1.0D0
          DBC(SCNDIM+3,2*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(SCNDIM+3,2*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(SCNDIM+4,2*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(SCNDIM+4,2*SCNDIM+SCIPAR0+2)=0.0D0
c
        ELSE IF ((SCIPS.EQ.PCT1).OR.(SCIPS.EQ.PCT2)) THEN
c
c         Connection pseudo-eq. - crossing - tg. pt of the same v.f.
c         ----------------------------------------------------------
c
c         Right boundary value
          DO K=1,SCNDIM
            U(K)=U1(SCNDIM+K)
          END DO
c
c         Vector field I computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FI0,
     +      DFI0DX,DFI0DP,DFI0DXDX,DFI0DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FJ0,
     +      DFJ0DX,DFJ0DP,DFJ0DXDX,DFJ0DXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFJ0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFJ0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field I computation at r.b.v. of vector field J
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI1,
     +      DFI1DX,DFI1DP,DFI1DXDX,DFI1DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI1,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI1DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI1DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at r.b.v. of vector field I
          CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +      H1,DH1DX,DH1DP,DH1DXDX,DH1DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H1=',H1
              PRINT *,'DH1DX='
              CALL DARRPRN1(DH1DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DH1DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at r.b.v. of vector field J
          CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +      H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H=',H
              PRINT *,'DHDX='
              CALL DARRPRN1(DHDX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DHDP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          DO K=1,SCNDIM
            FB(K+1)=PAR(SCIPAR0+1)*FI0(K)+PAR(SCIPAR0+2)*FJ0(K)
          END DO
          FB(SCNDIM+2)=PAR(SCIPAR0+1)+PAR(SCIPAR0+2)-1.0D0
          FB(SCNDIM+3)=H1
          FB(SCNDIM+4)=H
          FB(SCNDIM+5)=DSCPR1(FI1,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          DO K=1,SCNDIM
            FB(SCNDIM+5+K)=U0(SCNDIM+K)-U1(K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)
            DO L=1,SCNDIM
              DBC(L+1,K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI0DX,SCNDIM,SCNDIM,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ0DX,SCNDIM,SCNDIM,L,K)
            END DO
            DBC(SCNDIM+3,2*SCNDIM+K)=DH1DX(K)
            DBC(SCNDIM+4,3*SCNDIM+K)=DHDX(K)
            DBC(SCNDIM+5,3*SCNDIM+K)=DSCPR2(DFI1DX,SCNDIM,SCNDIM,
     +        1,K,1,DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI1,SCNDIM,1,1,1,1,
     +        DHDXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
            DBC(SCNDIM+5+K,SCNDIM+K)=1.0D0
            DBC(SCNDIM+5+K,2*SCNDIM+K)=-1.0D0
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,4*SCNDIM+K)=DH0DP(K)         
            DO L=1,SCNDIM
              DBC(L+1,4*SCNDIM+K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI0DP,SCNDIM,SCNPARX,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ0DP,SCNDIM,SCNPARX,L,K)
            END DO
            DBC(SCNDIM+3,4*SCNDIM+K)=DH1DP(K)
            DBC(SCNDIM+4,4*SCNDIM+K)=DHDP(K)
            DBC(SCNDIM+5,4*SCNDIM+K)=DSCPR2(DFI1DP,SCNDIM,SCNPARX,
     +        1,K,1,DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI1,SCNDIM,1,1,1,1,
     +        DHDXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
          END DO
          DO K=1,SCNDIM
            DBC(K+1,4*SCNDIM+SCIPAR0+1)=FI0(K)
            DBC(K+1,4*SCNDIM+SCIPAR0+2)=FJ0(K)
            DBC(SCNDIM+5+K,4*SCNDIM+SCIPAR0+1)=0.0D0
            DBC(SCNDIM+5+K,4*SCNDIM+SCIPAR0+2)=0.0D0
          END DO
          DBC(1,4*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(1,4*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(SCNDIM+2,4*SCNDIM+SCIPAR0+1)=1.0D0
          DBC(SCNDIM+2,4*SCNDIM+SCIPAR0+2)=1.0D0
          DBC(SCNDIM+3,4*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(SCNDIM+3,4*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(SCNDIM+4,4*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(SCNDIM+4,4*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(SCNDIM+5,4*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(SCNDIM+5,4*SCNDIM+SCIPAR0+2)=0.0D0
c
        ELSE IF ((SCIPS.EQ.PTD1).OR.(SCIPS.EQ.PTD2)) THEN
c
c         Connection pseudo-eq. - tangent point of different v.f.
c         -------------------------------------------------------
c
c         Vector field I computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FI0,
     +      DFI0DX,DFI0DP,DFI0DXDX,DFI0DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FJ0,
     +      DFJ0DX,DFJ0DP,DFJ0DXDX,DFJ0DXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFJ0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFJ0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation at right boundary value
          CALL SCFUNC(SCNDIM,U1,PAR,SCIDIFF,FJ1,
     +      DFJ1DX,DFJ1DP,DFJ1DXDX,DFJ1DXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ1,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFJ1DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFJ1DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at right boundary value
          CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +      H1,DH1DX,DH1DP,DH1DXDX,DH1DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H1=',H1
              PRINT *,'DH1DX='
              CALL DARRPRN1(DH1DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DH1DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          DO K=1,SCNDIM
            FB(K+1)=PAR(SCIPAR0+1)*FI0(K)+PAR(SCIPAR0+2)*FJ0(K)
          END DO
          FB(SCNDIM+2)=PAR(SCIPAR0+1)+PAR(SCIPAR0+2)-1.0D0
          FB(SCNDIM+3)=H1
          FB(SCNDIM+4)=DSCPR1(FJ1,SCNDIM,1,DH1DX,SCNDIM,1,SCNDIM)
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)
            DO L=1,SCNDIM
              DBC(L+1,K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI0DX,SCNDIM,SCNDIM,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ0DX,SCNDIM,SCNDIM,L,K)
            END DO
            DBC(SCNDIM+3,SCNDIM+K)=DH1DX(K)
            DBC(SCNDIM+4,SCNDIM+K)=DSCPR2(DFJ1DX,SCNDIM,SCNDIM,1,K,1,
     +        DH1DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FJ1,SCNDIM,1,1,1,1,
     +        DH1DXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,2*SCNDIM+K)=DH0DP(K)         
            DO L=1,SCNDIM
              DBC(L+1,2*SCNDIM+K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI0DP,SCNDIM,SCNPARX,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ0DP,SCNDIM,SCNPARX,L,K)
            END DO
            DBC(SCNDIM+3,2*SCNDIM+K)=DH1DP(K)         
            DBC(SCNDIM+4,2*SCNDIM+K)=DSCPR2(DFJ1DP,SCNDIM,SCNPARX,
     +        1,K,1,DH1DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FJ1,SCNDIM,1,1,1,1,
     +        DH1DXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
          END DO
          DO K=1,SCNDIM
            DBC(K+1,2*SCNDIM+SCIPAR0+1)=FI0(K)
            DBC(K+1,2*SCNDIM+SCIPAR0+2)=FJ0(K)
          END DO
          DBC(1,2*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(1,2*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(SCNDIM+2,2*SCNDIM+SCIPAR0+1)=1.0D0
          DBC(SCNDIM+2,2*SCNDIM+SCIPAR0+2)=1.0D0
          DBC(SCNDIM+3,2*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(SCNDIM+3,2*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(SCNDIM+4,2*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(SCNDIM+4,2*SCNDIM+SCIPAR0+2)=0.0D0
c
        ELSE IF ((SCIPS.EQ.PCD1).OR.(SCIPS.EQ.PCD2)) THEN
c
c         Connection pseudo-eq. - crossing - tg. pt of different v.f.
c         -----------------------------------------------------------
c
c         Right boundary value
          DO K=1,SCNDIM
            U(K)=U1(SCNDIM+K)
          END DO
c
c         Vector field I computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FI0,
     +      DFI0DX,DFI0DP,DFI0DXDX,DFI0DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FJ0,
     +      DFJ0DX,DFJ0DP,DFJ0DXDX,DFJ0DXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFJ0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFJ0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation at r.b.v. of vector field J
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FJ1,
     +      DFJ1DX,DFJ1DP,DFJ1DXDX,DFJ1DXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ1,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFJ1DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFJ1DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at r.b.v. of vector field I
          CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +      H1,DH1DX,DH1DP,DH1DXDX,DH1DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H1=',H1
              PRINT *,'DH1DX='
              CALL DARRPRN1(DH1DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DH1DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at r.b.v. of vector field J
          CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +      H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H=',H
              PRINT *,'DHDX='
              CALL DARRPRN1(DHDX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DHDP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          DO K=1,SCNDIM
            FB(K+1)=PAR(SCIPAR0+1)*FI0(K)+PAR(SCIPAR0+2)*FJ0(K)
          END DO
          FB(SCNDIM+2)=PAR(SCIPAR0+1)+PAR(SCIPAR0+2)-1.0D0
          FB(SCNDIM+3)=H1
          FB(SCNDIM+4)=H
          FB(SCNDIM+5)=DSCPR1(FJ1,SCNDIM,1,DHDX,SCNDIM,1,SCNDIM)
          DO K=1,SCNDIM
            FB(SCNDIM+5+K)=U0(SCNDIM+K)-U1(K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)
            DO L=1,SCNDIM
              DBC(L+1,K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI0DX,SCNDIM,SCNDIM,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ0DX,SCNDIM,SCNDIM,L,K)
            END DO
            DBC(SCNDIM+3,2*SCNDIM+K)=DH1DX(K)
            DBC(SCNDIM+4,3*SCNDIM+K)=DHDX(K)
            DBC(SCNDIM+5,3*SCNDIM+K)=DSCPR2(DFJ1DX,SCNDIM,SCNDIM,
     +        1,K,1,DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FJ1,SCNDIM,1,1,1,1,
     +        DHDXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
            DBC(SCNDIM+5+K,SCNDIM+K)=1.0D0
            DBC(SCNDIM+5+K,2*SCNDIM+K)=-1.0D0
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,4*SCNDIM+K)=DH0DP(K)         
            DO L=1,SCNDIM
              DBC(L+1,4*SCNDIM+K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI0DP,SCNDIM,SCNPARX,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ0DP,SCNDIM,SCNPARX,L,K)
            END DO
            DBC(SCNDIM+3,4*SCNDIM+K)=DH1DP(K)
            DBC(SCNDIM+4,4*SCNDIM+K)=DHDP(K)
            DBC(SCNDIM+5,4*SCNDIM+K)=DSCPR2(DFJ1DP,SCNDIM,SCNPARX,
     +        1,K,1,DHDX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FJ1,SCNDIM,1,1,1,1,
     +        DHDXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
          END DO
          DO K=1,SCNDIM
            DBC(K+1,4*SCNDIM+SCIPAR0+1)=FI0(K)
            DBC(K+1,4*SCNDIM+SCIPAR0+2)=FJ0(K)
            DBC(SCNDIM+5+K,4*SCNDIM+SCIPAR0+1)=0.0D0
            DBC(SCNDIM+5+K,4*SCNDIM+SCIPAR0+2)=0.0D0
          END DO
          DBC(1,4*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(1,4*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(SCNDIM+2,4*SCNDIM+SCIPAR0+1)=1.0D0
          DBC(SCNDIM+2,4*SCNDIM+SCIPAR0+2)=1.0D0
          DBC(SCNDIM+3,4*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(SCNDIM+3,4*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(SCNDIM+4,4*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(SCNDIM+4,4*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(SCNDIM+5,4*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(SCNDIM+5,4*SCNDIM+SCIPAR0+2)=0.0D0
c
        ELSE IF ((SCIPS.EQ.PEP1).OR.(SCIPS.EQ.PEP2)) THEN
c
c         Connection pseudo-eq. - pseudo-eq. (SCNDIM=2)
c         ---------------------------------------------
c
c         Vector field I computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FI0,
     +      DFI0DX,DFI0DP,DFI0DXDX,DFI0DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FJ0,
     +      DFJ0DX,DFJ0DP,DFJ0DXDX,DFJ0DXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFJ0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFJ0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field I computation at right boundary value
          CALL SCFUNC(SCNDIM,U1,PAR,SCIDIFF,FI1,
     +      DFI1DX,DFI1DP,DFI1DXDX,DFI1DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI1,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI1DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI1DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation at right boundary value
          CALL SCFUNC(SCNDIM,U1,PAR,SCIDIFF,FJ1,
     +      DFJ1DX,DFJ1DP,DFJ1DXDX,DFJ1DXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ1,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFJ1DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFJ1DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at right boundary value
          CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +      H1,DH1DX,DH1DP,DH1DXDX,DH1DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H1=',H1
              PRINT *,'DH1DX='
              CALL DARRPRN1(DH1DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DH1DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          DO K=1,SCNDIM
            FB(K+1)=PAR(SCIPAR0+1)*FI0(K)+PAR(SCIPAR0+2)*FJ0(K)
          END DO
          FB(SCNDIM+2)=PAR(SCIPAR0+1)+PAR(SCIPAR0+2)-1.0D0
          FB(SCNDIM+3)=H1
          DO K=1,SCNDIM
            FB(SCNDIM+3+K)=PAR(SCIPAR0+3)*FI1(K)+PAR(SCIPAR0+4)*FJ1(K)
          END DO
          FB(2*SCNDIM+4)=PAR(SCIPAR0+3)+PAR(SCIPAR0+4)-1.0D0
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)
            DO L=1,SCNDIM
              DBC(L+1,K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI0DX,SCNDIM,SCNDIM,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ0DX,SCNDIM,SCNDIM,L,K)
            END DO
            DBC(SCNDIM+3,SCNDIM+K)=DH1DX(K)
            DO L=1,SCNDIM
              DBC(SCNDIM+L+3,SCNDIM+K)=
     +          PAR(SCIPAR0+3)*DARR2(DFI1DX,SCNDIM,SCNDIM,L,K)+
     +          PAR(SCIPAR0+4)*DARR2(DFJ1DX,SCNDIM,SCNDIM,L,K)
            END DO
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,2*SCNDIM+K)=DH0DP(K)         
            DO L=1,SCNDIM
              DBC(L+1,2*SCNDIM+K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI0DP,SCNDIM,SCNPARX,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ0DP,SCNDIM,SCNPARX,L,K)
            END DO
            DBC(SCNDIM+3,2*SCNDIM+K)=DH1DP(K)         
            DO L=1,SCNDIM
              DBC(SCNDIM+L+3,2*SCNDIM+K)=
     +          PAR(SCIPAR0+3)*DARR2(DFI1DP,SCNDIM,SCNPARX,L,K)+
     +          PAR(SCIPAR0+4)*DARR2(DFJ1DP,SCNDIM,SCNPARX,L,K)
            END DO
          END DO
          DO K=1,SCNDIM
            DBC(K+1,2*SCNDIM+SCIPAR0+1)=FI0(K)
            DBC(K+1,2*SCNDIM+SCIPAR0+2)=FJ0(K)
            DBC(K+1,2*SCNDIM+SCIPAR0+3)=0.0D0
            DBC(K+1,2*SCNDIM+SCIPAR0+4)=0.0D0
            DBC(SCNDIM+K+3,2*SCNDIM+SCIPAR0+1)=0.0D0
            DBC(SCNDIM+K+3,2*SCNDIM+SCIPAR0+2)=0.0D0
            DBC(SCNDIM+K+3,2*SCNDIM+SCIPAR0+3)=FI1(K)
            DBC(SCNDIM+K+3,2*SCNDIM+SCIPAR0+4)=FJ1(K)
          END DO
          DBC(1,2*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(1,2*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(1,2*SCNDIM+SCIPAR0+3)=0.0D0
          DBC(1,2*SCNDIM+SCIPAR0+4)=0.0D0
          DBC(SCNDIM+2,2*SCNDIM+SCIPAR0+1)=1.0D0
          DBC(SCNDIM+2,2*SCNDIM+SCIPAR0+2)=1.0D0
          DBC(SCNDIM+2,2*SCNDIM+SCIPAR0+3)=0.0D0
          DBC(SCNDIM+2,2*SCNDIM+SCIPAR0+4)=0.0D0
          DBC(SCNDIM+3,2*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(SCNDIM+3,2*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(SCNDIM+3,2*SCNDIM+SCIPAR0+3)=0.0D0
          DBC(SCNDIM+3,2*SCNDIM+SCIPAR0+4)=0.0D0
          DBC(2*SCNDIM+4,2*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(2*SCNDIM+4,2*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(2*SCNDIM+4,2*SCNDIM+SCIPAR0+3)=1.0D0
          DBC(2*SCNDIM+4,2*SCNDIM+SCIPAR0+4)=1.0D0
c
        ELSE IF ((SCIPS.EQ.PCP1).OR.(SCIPS.EQ.PCP2)) THEN
c
c         Connection pseudo-eq. - crossing - pseudo-eq. (SCNDIM=2)
c         --------------------------------------------------------
c
c         Right boundary value
          DO K=1,SCNDIM
            U(K)=U1(SCNDIM+K)
          END DO
c
c         Vector field I computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FI0,
     +      DFI0DX,DFI0DP,DFI0DXDX,DFI0DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FJ0,
     +      DFJ0DX,DFJ0DP,DFJ0DXDX,DFJ0DXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFJ0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFJ0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field I computation at r.b.v. of vector field J
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FI1,
     +      DFI1DX,DFI1DP,DFI1DXDX,DFI1DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI1,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI1DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI1DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation at at r.b.v. of vector field J
          CALL SCFUNC(SCNDIM,U,PAR,SCIDIFF,FJ1,
     +      DFJ1DX,DFJ1DP,DFJ1DXDX,DFJ1DXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ1,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFJ1DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFJ1DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at r.b.v. of vector field I
          CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +      H1,DH1DX,DH1DP,DH1DXDX,DH1DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H1=',H1
              PRINT *,'DH1DX='
              CALL DARRPRN1(DH1DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DH1DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at r.b.v. of vector field J
          CALL SCBOUND(SCNDIM,U,PAR,SCIDIFF,
     +      H,DHDX,DHDP,DHDXDX,DHDXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H=',H
              PRINT *,'DHDX='
              CALL DARRPRN1(DHDX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DHDP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          DO K=1,SCNDIM
            FB(K+1)=PAR(SCIPAR0+1)*FI0(K)+PAR(SCIPAR0+2)*FJ0(K)
          END DO
          FB(SCNDIM+2)=PAR(SCIPAR0+1)+PAR(SCIPAR0+2)-1.0D0
          FB(SCNDIM+3)=H1
          FB(SCNDIM+4)=H
          DO K=1,SCNDIM
            FB(SCNDIM+K+4)=PAR(SCIPAR0+3)*FI1(K)+PAR(SCIPAR0+4)*FJ1(K)
          END DO
          FB(2*SCNDIM+5)=PAR(SCIPAR0+3)+PAR(SCIPAR0+4)-1.0D0
          DO K=1,SCNDIM
            FB(2*SCNDIM+5+K)=U0(SCNDIM+K)-U1(K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)
            DO L=1,SCNDIM
              DBC(L+1,K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI0DX,SCNDIM,SCNDIM,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ0DX,SCNDIM,SCNDIM,L,K)
            END DO
            DBC(SCNDIM+3,2*SCNDIM+K)=DH1DX(K)
            DBC(SCNDIM+4,3*SCNDIM+K)=DHDX(K)
            DO L=1,SCNDIM
              DBC(SCNDIM+L+4,3*SCNDIM+K)=
     +          PAR(SCIPAR0+3)*DARR2(DFI1DX,SCNDIM,SCNDIM,L,K)+
     +          PAR(SCIPAR0+4)*DARR2(DFJ1DX,SCNDIM,SCNDIM,L,K)
            END DO
            DBC(2*SCNDIM+5+K,SCNDIM+K)=1.0D0
            DBC(2*SCNDIM+5+K,2*SCNDIM+K)=-1.0D0
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,4*SCNDIM+K)=DH0DP(K)         
            DO L=1,SCNDIM
              DBC(L+1,4*SCNDIM+K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI0DP,SCNDIM,SCNPARX,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ0DP,SCNDIM,SCNPARX,L,K)
            END DO
            DBC(SCNDIM+3,4*SCNDIM+K)=DH1DP(K)
            DBC(SCNDIM+4,4*SCNDIM+K)=DHDP(K)
            DO L=1,SCNDIM
              DBC(SCNDIM+L+4,4*SCNDIM+K)=
     +          PAR(SCIPAR0+3)*DARR2(DFI1DP,SCNDIM,SCNPARX,L,K)+
     +          PAR(SCIPAR0+4)*DARR2(DFJ1DP,SCNDIM,SCNPARX,L,K)
            END DO
          END DO
          DO K=1,SCNDIM
            DBC(K+1,4*SCNDIM+SCIPAR0+1)=FI0(K)
            DBC(K+1,4*SCNDIM+SCIPAR0+2)=FJ0(K)
            DBC(K+1,4*SCNDIM+SCIPAR0+3)=0.0D0
            DBC(K+1,4*SCNDIM+SCIPAR0+4)=0.0D0
            DBC(SCNDIM+K+4,4*SCNDIM+SCIPAR0+1)=0.0D0
            DBC(SCNDIM+K+4,4*SCNDIM+SCIPAR0+2)=0.0D0
            DBC(SCNDIM+K+4,4*SCNDIM+SCIPAR0+3)=FI1(K)
            DBC(SCNDIM+K+4,4*SCNDIM+SCIPAR0+4)=FJ1(K)
            DBC(2*SCNDIM+5+K,4*SCNDIM+SCIPAR0+1)=0.0D0
            DBC(2*SCNDIM+5+K,4*SCNDIM+SCIPAR0+2)=0.0D0
            DBC(2*SCNDIM+5+K,4*SCNDIM+SCIPAR0+3)=0.0D0
            DBC(2*SCNDIM+5+K,4*SCNDIM+SCIPAR0+4)=0.0D0
          END DO
          DBC(1,4*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(1,4*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(1,4*SCNDIM+SCIPAR0+3)=0.0D0
          DBC(1,4*SCNDIM+SCIPAR0+4)=0.0D0
          DBC(SCNDIM+2,4*SCNDIM+SCIPAR0+1)=1.0D0
          DBC(SCNDIM+2,4*SCNDIM+SCIPAR0+2)=1.0D0
          DBC(SCNDIM+2,4*SCNDIM+SCIPAR0+3)=0.0D0
          DBC(SCNDIM+2,4*SCNDIM+SCIPAR0+4)=0.0D0
          DBC(SCNDIM+3,4*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(SCNDIM+3,4*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(SCNDIM+3,4*SCNDIM+SCIPAR0+3)=0.0D0
          DBC(SCNDIM+3,4*SCNDIM+SCIPAR0+4)=0.0D0
          DBC(SCNDIM+4,4*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(SCNDIM+4,4*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(SCNDIM+4,4*SCNDIM+SCIPAR0+3)=0.0D0
          DBC(SCNDIM+4,4*SCNDIM+SCIPAR0+4)=0.0D0
          DBC(2*SCNDIM+5,4*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(2*SCNDIM+5,4*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(2*SCNDIM+5,4*SCNDIM+SCIPAR0+3)=1.0D0
          DBC(2*SCNDIM+5,4*SCNDIM+SCIPAR0+4)=1.0D0
c
        ELSE IF ((SCIPS.EQ.PES1).OR.(SCIPS.EQ.PES2)) THEN
c
c         Connection pseudo-equilibrium - saddle
c         --------------------------------------
c
c         Saddle
          DO K=1,SCNDIM
            S(K)=PAR(SCIPAR0+4+K)
          END DO
c
c         Unstable eigenvalue/eigenvector
          NU=PAR(SCIPAR0+5+2*SCNDIM)
          DO K=1,SCNDIM
            W(K)=PAR(SCIPAR0+4+SCNDIM+K)
          END DO
c
c         Vector field I computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FI0,
     +      DFI0DX,DFI0DP,DFI0DXDX,DFI0DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FJ0,
     +      DFJ0DX,DFJ0DP,DFJ0DXDX,DFJ0DXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFJ0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFJ0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field I computation at the saddle
          CALL SCFUNC(SCNDIM,S,PAR,SCIDIFF,FI1,
     +      DFI1DX,DFI1DP,DFI1DXDX,DFI1DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI1,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI1DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI1DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          DO K=1,SCNDIM
            FB(K+1)=PAR(SCIPAR0+1)*FI0(K)+PAR(SCIPAR0+2)*FJ0(K)
          END DO
          FB(SCNDIM+2)=PAR(SCIPAR0+1)+PAR(SCIPAR0+2)-1.0D0
          DO K=1,SCNDIM
            FB(SCNDIM+K+2)=FI1(K)
            FB(2*SCNDIM+K+2)=DSCPR2(DFI1DX,SCNDIM,SCNDIM,1,K,1,
     +        W,SCNDIM,1,1,1,1,SCNDIM)-NU*W(K)
          END DO
          FB(3*SCNDIM+3)=DSCPR1(W,SCNDIM,1,W,SCNDIM,1,SCNDIM)-1.0D0
          FB(3*SCNDIM+4)=DSCPR1(W,SCNDIM,1,S,SCNDIM,1,SCNDIM)-
     +      DSCPR1(W,SCNDIM,1,U1,SCNDIM,1,SCNDIM)
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)
            DO L=1,SCNDIM
              DBC(L+1,K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI0DX,SCNDIM,SCNDIM,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ0DX,SCNDIM,SCNDIM,L,K)
            END DO
            DBC(3*SCNDIM+4,SCNDIM+K)=-W(K)
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,2*SCNDIM+K)=DH0DP(K)         
            DO L=1,SCNDIM
              DBC(L+1,2*SCNDIM+K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI0DP,SCNDIM,SCNPARX,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ0DP,SCNDIM,SCNPARX,L,K)
            END DO
            DO L=1,SCNDIM
              DBC(SCNDIM+L+2,2*SCNDIM+K)=DARR2(DFI1DP,
     +          SCNDIM,SCNPARX,L,K)
              DBC(2*SCNDIM+L+2,2*SCNDIM+K)=DSCPR3(DFI1DXDP,SCNDIM,
     +          SCNDIM,SCNPARX,1,L,K,1,W,SCNDIM,1,1,1,1,1,1,SCNDIM)
            END DO
          END DO
          DO K=1,SCNDIM
            DBC(K+1,2*SCNDIM+SCIPAR0+1)=FI0(K)
            DBC(K+1,2*SCNDIM+SCIPAR0+2)=FJ0(K)
            DBC(SCNDIM+K+2,2*SCNDIM+SCIPAR0+1)=0.0D0
            DBC(SCNDIM+K+2,2*SCNDIM+SCIPAR0+2)=0.0D0
            DBC(2*SCNDIM+K+2,2*SCNDIM+SCIPAR0+1)=0.0D0
            DBC(2*SCNDIM+K+2,2*SCNDIM+SCIPAR0+2)=0.0D0
          END DO
          DBC(1,2*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(1,2*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(SCNDIM+2,2*SCNDIM+SCIPAR0+1)=1.0D0
          DBC(SCNDIM+2,2*SCNDIM+SCIPAR0+2)=1.0D0
          DBC(3*SCNDIM+3,2*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(3*SCNDIM+3,2*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(3*SCNDIM+4,2*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(3*SCNDIM+4,2*SCNDIM+SCIPAR0+2)=0.0D0
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DBC(L+1,2*SCNDIM+SCIPAR0+4+K)=0.0D0
              DBC(L+1,2*SCNDIM+SCIPAR0+4+SCNDIM+K)=0.0D0
              DBC(SCNDIM+L+2,2*SCNDIM+SCIPAR0+4+K)=DARR2(DFI1DX,
     +          SCNDIM,SCNDIM,L,K)
              DBC(SCNDIM+L+2,2*SCNDIM+SCIPAR0+4+SCNDIM+K)=0.0D0
              DBC(2*SCNDIM+L+2,2*SCNDIM+SCIPAR0+4+K)=DSCPR3(
     +          DFI1DXDX,SCNDIM,SCNDIM,SCNDIM,1,L,K,1,
     +          W,SCNDIM,1,1,1,1,1,1,SCNDIM)
              DBC(2*SCNDIM+L+2,2*SCNDIM+SCIPAR0+4+SCNDIM+K)=DARR2(
     +          DFI1DX,SCNDIM,SCNDIM,K,L)
            END DO
            DBC(1,2*SCNDIM+SCIPAR0+4+K)=0.0D0
            DBC(1,2*SCNDIM+SCIPAR0+4+SCNDIM+K)=0.0D0
            DBC(K+1,2*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
            DBC(SCNDIM+2,2*SCNDIM+SCIPAR0+4+K)=0.0D0
            DBC(SCNDIM+2,2*SCNDIM+SCIPAR0+4+SCNDIM+K)=0.0D0
            DBC(SCNDIM+K+2,2*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
            DBC(2*SCNDIM+K+2,2*SCNDIM+SCIPAR0+4+SCNDIM+K)=
     +        DBC(2*SCNDIM+K+2,2*SCNDIM+SCIPAR0+4+SCNDIM+K)-NU
            DBC(2*SCNDIM+K+2,2*SCNDIM+SCIPAR0+5+2*SCNDIM)=-W(K)
            DBC(3*SCNDIM+3,2*SCNDIM+SCIPAR0+4+K)=0.0D0
            DBC(3*SCNDIM+3,2*SCNDIM+SCIPAR0+4+SCNDIM+K)=2.0D0*W(K)
            DBC(3*SCNDIM+4,2*SCNDIM+SCIPAR0+4+K)=W(K)
            DBC(3*SCNDIM+4,2*SCNDIM+SCIPAR0+4+SCNDIM+K)=S(K)-U1(K)
          END DO
          DBC(1,2*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
          DBC(SCNDIM+2,2*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
          DBC(3*SCNDIM+3,2*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
          DBC(3*SCNDIM+4,2*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
c
        ELSE IF ((SCIPS.EQ.PCS1).OR.(SCIPS.EQ.PCS2)) THEN
c
c         Connection pseudo-equilibrium - crossing - saddle
c         -------------------------------------------------
c
c         Right boundary value
          DO K=1,SCNDIM
            U(K)=U1(SCNDIM+K)
          END DO
c
c         Saddle
          DO K=1,SCNDIM
            S(K)=PAR(SCIPAR0+4+K)
          END DO
c
c         Unstable eigenvalue/eigenvector
          NU=PAR(SCIPAR0+5+2*SCNDIM)
          DO K=1,SCNDIM
            W(K)=PAR(SCIPAR0+4+SCNDIM+K)
          END DO
c
c         Vector field I computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FI0,
     +      DFI0DX,DFI0DP,DFI0DXDX,DFI0DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FJ0,
     +      DFJ0DX,DFJ0DP,DFJ0DXDX,DFJ0DXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFJ0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFJ0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Vector field J computation at the saddle
          CALL SCFUNC(SCNDIM,S,PAR,SCIDIFF,FJ1,
     +      DFJ1DX,DFJ1DP,DFJ1DXDX,DFJ1DXDP,SCFJ(I))
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',SCFJ(I),'='
              CALL DARRPRN1(FJ1,SCNDIM,1,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DX='
              CALL DARRPRN2(DFJ1DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',SCFJ(I),'DP='
              CALL DARRPRN2(DFJ1DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at r.b.v. of vector field I
          CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +      H1,DH1DX,DH1DP,DH1DXDX,DH1DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H1=',H1
              PRINT *,'DH1DX='
              CALL DARRPRN1(DH1DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DH1DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          DO K=1,SCNDIM
            FB(K+1)=PAR(SCIPAR0+1)*FI0(K)+PAR(SCIPAR0+2)*FJ0(K)
          END DO
          FB(SCNDIM+2)=PAR(SCIPAR0+1)+PAR(SCIPAR0+2)-1.0D0
          FB(SCNDIM+3)=H1
          DO K=1,SCNDIM
            FB(SCNDIM+3+K)=U0(SCNDIM+K)-U1(K)
          END DO
          DO K=1,SCNDIM
            FB(2*SCNDIM+K+3)=FJ1(K)
            FB(3*SCNDIM+K+3)=DSCPR2(DFJ1DX,SCNDIM,SCNDIM,1,K,1,
     +        W,SCNDIM,1,1,1,1,SCNDIM)-NU*W(K)
          END DO
          FB(4*SCNDIM+4)=DSCPR1(W,SCNDIM,1,W,SCNDIM,1,SCNDIM)-1.0D0
          FB(4*SCNDIM+5)=DSCPR1(W,SCNDIM,1,S,SCNDIM,1,SCNDIM)-
     +      DSCPR1(W,SCNDIM,1,U,SCNDIM,1,SCNDIM)
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)
            DO L=1,SCNDIM
              DBC(L+1,K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI0DX,SCNDIM,SCNDIM,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ0DX,SCNDIM,SCNDIM,L,K)
            END DO
            DBC(SCNDIM+3,2*SCNDIM+K)=DH1DX(K)
            DBC(SCNDIM+3+K,SCNDIM+K)=1.0D0
            DBC(SCNDIM+3+K,2*SCNDIM+K)=-1.0D0
            DBC(4*SCNDIM+5,3*SCNDIM+K)=-W(K)
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,4*SCNDIM+K)=DH0DP(K)         
            DO L=1,SCNDIM
              DBC(L+1,4*SCNDIM+K)=
     +          PAR(SCIPAR0+1)*DARR2(DFI0DP,SCNDIM,SCNPARX,L,K)+
     +          PAR(SCIPAR0+2)*DARR2(DFJ0DP,SCNDIM,SCNPARX,L,K)
            END DO
            DBC(SCNDIM+3,4*SCNDIM+K)=DH1DP(K)
            DO L=1,SCNDIM
              DBC(2*SCNDIM+L+3,4*SCNDIM+K)=DARR2(DFJ1DP,
     +          SCNDIM,SCNPARX,L,K)
              DBC(3*SCNDIM+L+3,4*SCNDIM+K)=DSCPR3(DFJ1DXDP,SCNDIM,
     +          SCNDIM,SCNPARX,1,L,K,1,W,SCNDIM,1,1,1,1,1,1,SCNDIM)
            END DO
          END DO
          DO K=1,SCNDIM
            DBC(K+1,4*SCNDIM+SCIPAR0+1)=FI0(K)
            DBC(K+1,4*SCNDIM+SCIPAR0+2)=FJ0(K)
            DBC(SCNDIM+3+K,4*SCNDIM+SCIPAR0+1)=0.0D0
            DBC(SCNDIM+3+K,4*SCNDIM+SCIPAR0+2)=0.0D0
            DBC(2*SCNDIM+K+3,4*SCNDIM+SCIPAR0+1)=0.0D0
            DBC(2*SCNDIM+K+3,4*SCNDIM+SCIPAR0+2)=0.0D0
            DBC(3*SCNDIM+K+3,4*SCNDIM+SCIPAR0+1)=0.0D0
            DBC(3*SCNDIM+K+3,4*SCNDIM+SCIPAR0+2)=0.0D0
          END DO
          DBC(1,4*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(1,4*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(SCNDIM+2,4*SCNDIM+SCIPAR0+1)=1.0D0
          DBC(SCNDIM+2,4*SCNDIM+SCIPAR0+2)=1.0D0
          DBC(SCNDIM+3,4*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(SCNDIM+3,4*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(4*SCNDIM+4,4*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(4*SCNDIM+4,4*SCNDIM+SCIPAR0+2)=0.0D0
          DBC(4*SCNDIM+5,4*SCNDIM+SCIPAR0+1)=0.0D0
          DBC(4*SCNDIM+5,4*SCNDIM+SCIPAR0+2)=0.0D0
          DO K=1,SCNDIM
            DO L=1,SCNDIM
              DBC(L+1,4*SCNDIM+SCIPAR0+4+K)=0.0D0
              DBC(L+1,4*SCNDIM+SCIPAR0+4+SCNDIM+K)=0.0D0
              DBC(SCNDIM+3+L,4*SCNDIM+SCIPAR0+4+K)=0.0D0
              DBC(SCNDIM+3+L,4*SCNDIM+SCIPAR0+4+SCNDIM+K)=0.0D0
              DBC(2*SCNDIM+L+3,4*SCNDIM+SCIPAR0+4+K)=DARR2(DFJ1DX,
     +          SCNDIM,SCNDIM,L,K)
              DBC(2*SCNDIM+L+3,4*SCNDIM+SCIPAR0+4+SCNDIM+K)=0.0D0
              DBC(3*SCNDIM+L+3,4*SCNDIM+SCIPAR0+4+K)=DSCPR3(
     +          DFJ1DXDX,SCNDIM,SCNDIM,SCNDIM,1,L,K,1,
     +          W,SCNDIM,1,1,1,1,1,1,SCNDIM)
              DBC(3*SCNDIM+L+3,4*SCNDIM+SCIPAR0+4+SCNDIM+K)=DARR2(
     +          DFJ1DX,SCNDIM,SCNDIM,K,L)
            END DO
            DBC(1,4*SCNDIM+SCIPAR0+4+K)=0.0D0
            DBC(1,4*SCNDIM+SCIPAR0+4+SCNDIM+K)=0.0D0
            DBC(K+1,4*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
            DBC(SCNDIM+2,4*SCNDIM+SCIPAR0+4+K)=0.0D0
            DBC(SCNDIM+2,4*SCNDIM+SCIPAR0+4+SCNDIM+K)=0.0D0
            DBC(SCNDIM+3,4*SCNDIM+SCIPAR0+4+K)=0.0D0
            DBC(SCNDIM+3,4*SCNDIM+SCIPAR0+4+SCNDIM+K)=0.0D0
            DBC(SCNDIM+3+K,4*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
            DBC(2*SCNDIM+K+3,4*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
            DBC(3*SCNDIM+K+3,4*SCNDIM+SCIPAR0+4+SCNDIM+K)=
     +        DBC(3*SCNDIM+K+3,4*SCNDIM+SCIPAR0+4+SCNDIM+K)-NU
            DBC(3*SCNDIM+K+3,4*SCNDIM+SCIPAR0+5+2*SCNDIM)=-W(K)
            DBC(4*SCNDIM+4,4*SCNDIM+SCIPAR0+4+K)=0.0D0
            DBC(4*SCNDIM+4,4*SCNDIM+SCIPAR0+4+SCNDIM+K)=2.0D0*W(K)
            DBC(4*SCNDIM+5,4*SCNDIM+SCIPAR0+4+K)=W(K)
            DBC(4*SCNDIM+5,4*SCNDIM+SCIPAR0+4+SCNDIM+K)=S(K)-U(K)
          END DO
          DBC(1,4*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
          DBC(SCNDIM+2,4*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
          DBC(SCNDIM+3,4*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
          DBC(4*SCNDIM+4,4*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
          DBC(4*SCNDIM+5,4*SCNDIM+SCIPAR0+5+2*SCNDIM)=0.0D0
c
        ELSE IF ((SCIPS.EQ.SCY1).OR.(SCIPS.EQ.SCY2)) THEN
c
c         Sliding cycle
c         -------------
c
c         Vector field I computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FI0,
     +      DFI0DX,DFI0DP,DFI0DXDX,DFI0DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          FB(2)=DSCPR1(FI0,SCNDIM,1,DH0DX,SCNDIM,1,SCNDIM)
          DO K=1,SCNDIM
            FB(2+K)=U0(SCNDIM+K)-U1(K)
            FB(2+SCNDIM+K)=U1(SCNDIM+K)-U0(K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)         
            DBC(2,K)=DSCPR2(DFI0DX,SCNDIM,SCNDIM,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
            DBC(2+K,SCNDIM+K)=1.0D0
            DBC(2+K,2*SCNDIM+K)=-1.0D0
            DBC(2+SCNDIM+K,3*SCNDIM+K)=1.0D0
            DBC(2+SCNDIM+K,K)=-1.0D0
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,4*SCNDIM+K)=DH0DP(K)         
            DBC(2,4*SCNDIM+K)=DSCPR2(DFI0DP,SCNDIM,SCNPARX,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
          END DO
c
        ELSE IF ((SCIPS.EQ.SCC1).OR.(SCIPS.EQ.SCC2)) THEN
c
c         Sliding-crossing cycle
c         ----------------------
c
c         Vector field I computation at left boundary value
          CALL SCFUNC(SCNDIM,U0,PAR,SCIDIFF,FI0,
     +      DFI0DX,DFI0DP,DFI0DXDX,DFI0DXDP,I)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'F',I,'='
              CALL DARRPRN1(FI0,SCNDIM,1,SCNDIM)
              PRINT *,'DF',I,'DX='
              CALL DARRPRN2(DFI0DX,SCNDIM,SCNDIM,1,1,SCNDIM,SCNDIM)
              PRINT *,'DF',I,'DP='
              CALL DARRPRN2(DFI0DP,SCNDIM,SCNPARX,1,1,SCNDIM,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Discontinuity boundary at r.b.v. of vector field I
          CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +      H1,DH1DX,DH1DP,DH1DXDX,DH1DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H1=',H1
              PRINT *,'DH1DX='
              CALL DARRPRN1(DH1DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DH1DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          FB(2)=DSCPR1(FI0,SCNDIM,1,DH0DX,SCNDIM,1,SCNDIM)
          FB(3)=H1
          DO K=1,SCNDIM
            FB(3+K)=U0(SCNDIM+K)-U1(K)
            FB(3+SCNDIM+K)=U0(2*SCNDIM+K)-U1(SCNDIM+K)
            FB(3+2*SCNDIM+K)=U1(2*SCNDIM+K)-U0(K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)         
            DBC(2,K)=DSCPR2(DFI0DX,SCNDIM,SCNDIM,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDX,SCNDIM,SCNDIM,1,K,1,SCNDIM)
            DBC(3,3*SCNDIM+K)=DH1DX(K)
            DBC(3+K,SCNDIM+K)=1.0D0
            DBC(3+K,3*SCNDIM+K)=-1.0D0
            DBC(3+SCNDIM+K,2*SCNDIM+K)=1.0D0
            DBC(3+SCNDIM+K,4*SCNDIM+K)=-1.0D0
            DBC(3+2*SCNDIM+K,5*SCNDIM+K)=1.0D0
            DBC(3+2*SCNDIM+K,K)=-1.0D0
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,6*SCNDIM+K)=DH0DP(K)         
            DBC(2,6*SCNDIM+K)=DSCPR2(DFI0DP,SCNDIM,SCNPARX,1,K,1,
     +        DH0DX,SCNDIM,1,1,1,1,SCNDIM)+
     +        DSCPR2(FI0,SCNDIM,1,1,1,1,
     +        DH0DXDP,SCNDIM,SCNPARX,1,K,1,SCNDIM)
            DBC(3,6*SCNDIM+K)=DH1DP(K)         
          END DO
c
        ELSE IF (SCIPS.EQ.CCY) THEN
c
c         Crossing cycle
c         --------------
c
c         discontinuity boundary at left boundary value
          CALL SCBOUND(SCNDIM,U0,PAR,SCIDIFF,
     +      H0,DH0DX,DH0DP,DH0DXDX,DH0DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H0=',H0
              PRINT *,'DH0DX='
              CALL DARRPRN1(DH0DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH0DP='
              CALL DARRPRN1(DH0DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         discontinuity boundary at right boundary value
          CALL SCBOUND(SCNDIM,U1,PAR,SCIDIFF,
     +      H1,DH1DX,DH1DP,DH1DXDX,DH1DXDP)
c
c         Debug
          IF (DEBUG) THEN
            IF (DBGLEV.GE.DBGL2) THEN
              PRINT *,'H1=',H1
              PRINT *,'DH1DX='
              CALL DARRPRN1(DH1DX,SCNDIM,1,SCNDIM)
              PRINT *,'DH1DP='
              CALL DARRPRN1(DH1DP,SCNPARX,1,SCNPARX)
            END IF
          END IF
c
c         Auto97 boundary value conditions
          FB(1)=H0
          FB(2)=H1
          DO K=1,SCNDIM
            FB(2+K)=U0(SCNDIM+K)-U1(K)
            FB(SCNDIM+2+K)=U0(K)-U1(SCNDIM+K)
          END DO
c
          IF (IJAC.EQ.0) GOTO 10
c
c         Auto97 derivatives with respect to state
          DO K=1,SCNDIM
            DBC(1,K)=DH0DX(K)         
            DBC(2,2*SCNDIM+K)=DH1DX(K)
            DBC(2+K,SCNDIM+K)=1.0D0
            DBC(2+K,2*SCNDIM+K)=-1.0D0
            DBC(SCNDIM+2+K,K)=1.0D0
            DBC(SCNDIM+2+K,3*SCNDIM+K)=-1.0D0
          END DO
c
          IF (IJAC.EQ.1) GOTO 10
c
c         Auto97 derivatives with respect to parameters
          DO K=1,SCNPARX
            DBC(1,4*SCNDIM+K)=DH0DP(K)         
            DBC(2,4*SCNDIM+K)=DH1DP(K)         
          END DO
c
        ELSE
          WRITE(*,510)SCIPS
          STOP
        END IF
c
c       Debug
  10    IF (DEBUG) THEN
          IF (DBGLEV.GE.DBGL1) THEN
            PRINT *,'FB='
            CALL DARRPRN1(FB,NBC,1,NBC)
            PRINT *,'DBC='
            CALL DARRPRN2(DBC,NBC,2*NDIM+NPARX,1,1,
     +        NBC,2*NDIM+NPARX)
            PRINT *,'exit from BCND'
          END IF
        END IF
c
      RETURN
      END
c
c
c     ----------------------------------------------
      SUBROUTINE ICND(NDIM,PAR,ICP,NINT,U,UOLD,UDOT,
     +  UPOLD,FI,IJAC,DINT)
c     ----------------------------------------------
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'slidecont.h'
        INCLUDE 'debug.h'
c
        DIMENSION U(NDIM),UOLD(NDIM),UDOT(NDIM),UPOLD(NDIM)
        DIMENSION FI(NINT),DINT(NINT,*),ICP(*),PAR(*)
c
c       Problem type selection
c       ----------------------
        IF (SCIPS.LT.100) THEN
c
c         AUTO97 problem type
c         -------------------
          CALL SCICND(NDIM,PAR,ICP,NINT,U,UOLD,UDOT,
     +      UPOLD,FI,IJAC,DINT)
c
c       ELSE
c
c         dummy step
c
        END IF
c
      RETURN
      END
c
c
c     -------------------------------------------------
      SUBROUTINE FOPT(NDIM,U,ICP,PAR,IJAC,FS,DFDU,DFDP)
c     -------------------------------------------------
c
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
c
        INCLUDE 'slidecont.h'
        INCLUDE 'debug.h'
c
        DIMENSION U(NDIM),ICP(*),PAR(*),DFDU(NDIM),DFDP(*)
c
c       Problem type selection
c       ----------------------
        IF (SCIPS.LT.100) THEN
c
c         AUTO97 problem type
c         -------------------
          CALL SCFOPT(NDIM,U,ICP,PAR,IJAC,FS,DFDU,DFDP)
c
c       ELSE
c
c         dummy step
c
        END IF
c
      RETURN
      END

